#include <iostream>
#include <string>

using namespace std;

#include <netcdf.hh>
#include <cstdio>
#include <cstdlib>


int main(int argc, char *argv[])
{
  if (argc != 5) {
    cout << "Usage: clamp-value <file> <var> <max/min> <value>" << endl;
    exit(1);
  }
  string file = argv[1];
  string var = argv[2];
  string type_str = argv[3];
  float value = atof(argv[4]);

  enum CLAMP_TYPE { MAX, MIN };
  CLAMP_TYPE type;
  if (type_str == "max" || type_str == "MAX")
    type = MAX;
  else if (type_str == "min" || type_str == "MIN")
    type = MIN;
  else {
    cout << "Invalid clamp type: " << type << endl;
    exit(1);
  }


  NcFile nc(file.c_str(), NcFile::Write);
  if (!nc.is_valid()) {
    cout << "Couldn't open file " << file << " for reading and writing"
         << endl;
    exit(1);
  }
  NcVar *ncv = nc.get_var(var.c_str());
  if (!ncv) {
    cout << "Couldn't find variable " << var << endl;
    exit(1);
  }
  if (ncv->num_dims() != 3) {
    cout << "Variable " << var << "has wrong number of "
         << "dimensions (should be 3)" << endl;
    exit(1);
  }
  long dim0_size = ncv->get_dim(0)->size();
  long dim1_size = ncv->get_dim(1)->size();
  long dim2_size = ncv->get_dim(2)->size();
  float *tmp = new float[dim2_size];

  for (int dim0_idx = 0; dim0_idx < dim0_size; ++dim0_idx)
    for (int dim1_idx = 0; dim1_idx < dim1_size; ++dim1_idx) {
      if (!ncv->set_cur(dim0_idx, dim1_idx, 0) ||
          !ncv->get(tmp, 1, 1, dim2_size)) {
        cout << "Failed to read data!" << endl;
        exit(1);
      }
      bool changed = false;
      for (int dim2_idx = 0; dim2_idx < dim2_size; ++dim2_idx)
        if (tmp[dim2_idx] != -999) {
          if (type == MAX) {
            if (tmp[dim2_idx] > value) {
              tmp[dim2_idx] = value;
              changed = true;
            }
          } else {
            if (tmp[dim2_idx] < value) {
              tmp[dim2_idx] = value;
              changed = true;
            }
          }
        }
      if (changed)
        if (!ncv->set_cur(dim0_idx, dim1_idx, 0) ||
            !ncv->put(tmp, 1, 1, dim2_size)) {
          cout << "Failed to read data!" << endl;
          exit(1);
        }
    }

  nc.sync();
}
