#include <cstdio>
#include <cstdlib>
#include <netcdf.hh>
#include <iostream>
#include <string>
using namespace std;

//----------------------------------------------------------------------
//
//  MAIN PROGRAM
//

int main(int argc, char *argv[])
{
  if (argc != 3) {
    cout << "Usage: ncminmax <infile> <var>" << endl;
    exit(1);
  }
  const char *var = argv[2];
  NcFile in_nc(argv[1]);

  NcDim *in_lat_dim = in_nc.get_dim("lat");
  NcDim *in_lon_dim = in_nc.get_dim("lon");
  NcDim *in_time_dim = in_nc.get_dim("time");
  NcVar *in_lat_var = in_nc.get_var("lat");
  NcVar *in_lon_var = in_nc.get_var("lon");
  NcVar *in_time_var = in_nc.get_var("time");
  NcVar *in_var = in_nc.get_var(var);
  if (!in_lat_dim || !in_lon_dim || !in_time_dim ||
      !in_lat_var || !in_lon_var || !in_time_var ||
      !in_var) {
    cout << "Something went wrong..." << endl;
    exit(1);
  }

  float lons[in_lon_dim->size()], lats[in_lat_dim->size()];
  for (int idx = 0; idx < in_lon_dim->size(); ++idx)
    lons[idx] = in_lon_var->as_float(idx);
  for (int idx = 0; idx < in_lat_dim->size(); ++idx)
    lats[idx] = in_lat_var->as_float(idx);

  float min_val = 1.0E20, max_val = -1.0E20;
  int min_lonidx, min_latidx, min_timeidx;
  int max_lonidx, max_latidx, max_timeidx;

  float tmp[in_lon_dim->size() * in_lat_dim->size()];
  for (int timeidx = 0; timeidx < in_time_dim->size(); ++timeidx) {
    if (timeidx % 1000 == 0) cout << "timeidx=" << timeidx << endl;
    if (!in_var->set_cur(timeidx, 0, 0) ||
        !in_var->get(&tmp[0], 1, in_lat_dim->size(), in_lon_dim->size())) {
      cout << "Failed reading data" << endl;
      exit(1);
    }

    for (int latidx = 0; latidx < in_lat_dim->size(); ++latidx) {
      for (int lonidx = 0; lonidx < in_lon_dim->size(); ++lonidx) {
        if (tmp[latidx * in_lon_dim->size() + lonidx] < min_val) {
          min_val = tmp[latidx * in_lon_dim->size() + lonidx];
          min_lonidx = lonidx;  min_latidx = latidx;  min_timeidx = timeidx;
        }
        if (tmp[latidx * in_lon_dim->size() + lonidx] > max_val) {
          max_val = tmp[latidx * in_lon_dim->size() + lonidx];
          max_lonidx = lonidx;  max_latidx = latidx;  max_timeidx = timeidx;
        }
      }
    }
  }

  cout << "Min: value=" << min_val << "  lon=" << lons[min_lonidx]
       << "  lat=" << lats[min_latidx] << "  time=" << min_timeidx
       << endl;
  cout << "Max: value=" << max_val << "  lon=" << lons[max_lonidx]
       << "  lat=" << lats[max_latidx] << "  time=" << max_timeidx
       << endl;
}
