c *****************************************************************************
c *                                                                           *
c *                         L P J - W H y M e version 1.3.1                   *
c *             Lund-Potsdam-Jena Dynamic Global Vegetation Model             *
c *               with Wetland Hydrology and Methane emissions                *
c *                                                                           *
c *                  Principal Author: Stephen Sitch (1)                      *
c *       Associate Author and Artistic Director: Benjamin Smith (2,3)        *
c *       Author of the Wetland Hydrology: Rita Wania (4)                     *
c *       Author of the Methane Model: Rita Wania (4)                         *
c *       Author of the NetCDF Driver: Ian Ross (5)                           *
c *                                                                           *
c * (1) MetOffice (JCHMR), Maclean Building, Crowmarsh Gifford,               *
c *     Wallingford OX10 8BB, UK                                              *
c * (2) Climate Impacts Group, Department of Ecology, Plant Ecology,          *
c *     Lund University, Ekologihuset, Lund, S-22362 Sweden                   *
c * (3) Max Planck Institute for Biogeochemistry, P.O. Box 100164, D-07701,   *
c *     Jena, Germany                                                         *
c * (4) School of Earth and Ocean Sciences, University of Victoria,           *
c *     PO Box 3065 STN CSC, Victoria, BC, V8W 3V6, Canada                    *
c * (5) Mathematics and Statistics, University of Victoria,                   *
c *     PO Box 3060 STN CSC, Victoria, BC, V8W 3R4, Canada                    *
c *                                                                           *
c *                                                                           *
c * Correspondence about the LPJ-WHy extensions should be directed to         *
c * rita@wania.net, University of Victoria, Canada                            *
c *                                                                           *
c * The start version for this code has been downloaded from the PIK LPJ      *
c * website on 12 Feb 2005, which included the Gerten hydrology (Gerten       *
c * et al. 2004) but no permafrost.                                           * 
c *****************************************************************************

C Copyright (C) 2010 Rita Wania <rita@wania.net>
C   
C  This program is free software; you can redistribute it and/or modify
C  it under the terms of the GNU General Public License as published by
C  the Free Software Foundation; either version 2 of the License, or
C  (at your option) any later version.
C   
C  This program is distributed in the hope that it will be useful,
C  but WITHOUT ANY WARRANTY; without even the implied warranty of
C  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C  GNU General Public License for more details.
C   
C  You should have received a copy of the GNU General Public License
C  along with this program; if not, write to the Free Software
C  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
C   

c -----------------------------------------------------------------------------
c                  TECHNICAL INFORMATION ABOUT THIS FILE
c -----------------------------------------------------------------------------

c Main file name:     lpjmain.f
c Common blocks:      inputvars.cb params.cb, soilvars.cb vegvars.cb ch4vars.cb
c Input parameters:   pftin.dat
c Configuration file: lpj.cfg
c netCDF Driver:      lpjio.cpp
c Documentation:      Using-The-netCDF-Driver.txt
c Makefile:           Makefile
c Language:           FORTRAN 77 with extensions (do ... enddo, do while ... 
c                     enddo, long variable names)
c                     C++ for the netCDF driver
c Compilation:        make
c
c Version dated: 14/10/2010


c *****************************************************************************
c *                                                                           *
c *                                INPUT/OUTPUT                               *
c *                                                                           *
c * The model source code comprises two modules: the main program (this file) *
c * and an input/output module. The two modules should be compiled and linked *
c * to produce a single executable file.  Normally, only the main program     *
c * will be distributed by the authors: IT IS EACH USER'S OWN RESPONIBILITY   *
c * TO PROVIDE SUITABLE INPUT/OUTPUT CODE. The main program and input/output  *
c * module are linked through calls, in the main program, to six subroutines  *
c * in the input/output module.  The structure of the main program, including *
c * calls to input/output subroutines, is as follows:                         *
c *                                                                           *
c *       call initio                                                         *
c *       call getgrid                                                        *
c *       do while (dogridcell)  !loop through grid cells                     *
c *                      :                                                    *
c *         [ grid cell initialisation ]                                      *
c *                      :                                                    *
c *         call getclimate                                                   *
c *         do while (doyear)    !loop through simulation years               *
c *                     :                                                     *
c *           [ annual processes ]                                            *
c *                     :                                                     *
c *           call outnnual                                                   *
c *           call getclimate                                                 *
c *         enddo                                                             *
c *         call outgrid                                                      *
c *         call getgrid                                                      *
c *       enddo                                                               *
c *       call termio                                                         *
c *                                                                           *
c * Specifications of input/output subroutines are given below. Declarations  *
c * and descriptions of argument variables are to be found in the             *
c * variable declarations section of the main program.                        *
c *                                                                           *
c * subroutine initio [no arguments]                                          *
c *   Initialisation of input/output, e.g. opening of files, run-time         *
c *   interaction with user.                                                  *
c *                                                                           *
c * subroutine getgrid(lat,soilcode,dogridcell)                               *
c *   Should return latitude and LPJ soil code (see subroutine                *
c *   soilparameters) for next grid cell. Logical 'dogridcell' should be set  *
c *   to .false. if no further grid cells remain to be processed, and .true.  *
c *   otherwise.                                                              *
c *                                                                           *
c * subroutine getclimate(year,mtemp,mprec,mwet,msun,co2,doyear)              *
c *   Should return monthly temperature, precipitation and sunshine, and      *
c *   atmospheric CO2 concentration, for current grid cell and year of        *
c *   simulation. Year counter 'year' is updated in the main program and      *
c *   should not be modified within the subroutine. Logical 'doyear' should   *
c *   be set to .false. following the last simulation year, and .true.        *
c *   otherwise.                                                              *
c *                                                                           *
c * subroutine outannual(year,present,nind,lm_ind,rm_ind,sm_ind,hm_ind,       *
c *   fpc_grid,anpp,acflux_estab,litter_ag,litter_bg,cpool_fast,cpool_slow,   *
c *   arh,afire_frac,acflux_fire)                                             *
c *   Output subroutine called at the end of each simulation year. The        *
c *   suggested argument list here may be supplemented by other variables     *
c *   if desired (by modifying argument lists both in the subroutine header,  *
c *   and the call in the main program). None of the argument variables       *
c *   should normally be modified within the subroutine.                      *
c *                                                                           *
c * subroutine outgrid()                                                      *
c *   Output subroutine called at the end of the simulation for each grid     *
c *   cell. The suggested argument list here may be supplemented by other     *
c *   variables if desired (by modifying argument lists both in the           *
c *   subroutine header, and the call in the main program). None of the       *
c *   argument variables should normally be modified within the subroutine.   *
c *                                                                           *
c * subroutine termio [no arguments]                                          *
c *   Closing of files etc. at end of model run.                              *
c *                                                                           *
c * NOTES                                                                     *
c *                                                                           *
c *   1. All six subroutines must appear in the input/output module.          *
c *      However, subroutines that are not required (potentially any of the   *
c *      above, except getgrid and getclimate) may be empty, i.e. comprise    *
c *      only a subroutine header, declarations section, and the statements   *
c *      'return' and 'end'.                                                  *
c *                                                                           *
c *   2. It is intended that input/output should be handled independently of  *
c *      the main program code. This will normally require incorporation of   *
c *      one or more common blocks to provide data links between input/       *
c *      output subroutines.                                                  *
c *                                                                           *
c *   3. PFT state variables are defined only for PFTs present at the end of  *
c *      a given simulation year. Among-PFT totals may therefore be           *
c *      calculated within output subroutines using code similar to the       *
c *      following (in this case, for calculation of annual ecosystem NPP):   *
c *                                                                           *
c *      integer pft,npft                                                     *
c *      parameter (npft=12)                                                  *
c *      real anpp_total                                                      *
c *                                                                           *
c *      npp_total=0.0                                                        *
c *      do pft=1,npft                                                        *
c *        if (present(pft)) anpp_total=anpp_total+anpp(pft)                  *
c *      enddo                                                                *
c *                                                                           *
c *   4. Grid cell total vegetation carbon (gC/m2) may be calculated as       *
c *      follows:                                                             *
c *                                                                           *
c *      real vegc                                                            *
c *                                                                           *
c *      vegc=0.0                                                             *
c *      do pft=1,npft                                                        *
c *        if (present(pft)) vegc=vegc+                                       *
c *          (lm_ind(pft)+rm_ind(pft)+sm_ind(pft)+hm_ind(pft))*nind(pft)      *
c *      enddo                                                                *
c *                                                                           *
c *   5. Annual net ecosystem production (NEP, gC/m2) may be calculated as    *
c *      follows:                                                             *
c *                                                                           *
c *      anep = anpp_total + acflux_estab - arh - acflux_fire                 *
c *      where 'anpp_total' is ecosystem annual NPP, as calculated            *
c *      according to Note 3 (above).                                         *
c *                                                                           *
c *   6. LPJ-WHyMe has 4 switches in the code now:                            *
c *      1) peathydr = TRUE if peatland hydrology wanted                      *
c *      2) methane_mod = TRUE if peathydr is TRUE and Methane emissions      *
c *                       wanted                                              *
c *      3) sens_test = TRUE if certain parameters are fed in from external   * 
c *                     file                                                  *
c *      4) litter_monthly = TRUE if litter should be spread equally over     *
c *                          12 months                                        *
c *                                                                           * 
c *****************************************************************************


c -----------------------------------------------------------------------------
c                            SCIENTIFIC SUMMARY
c -----------------------------------------------------------------------------
c
c A comprehensive description of the model can be found in:
c  Sitch, S., B. Smith, I.C. Prentice, A. Arneth, A. Bondeau, W. Cramer
c  J. O. Kaplan, S. Levis, W. Lucht, M. T. Sykes, K. Thonicke, S. Venevsky
c  (2003) Evaluation of ecosystem dynamics, plant geography, and terrestrial
c   carbon cycling in the LPJ dynamic global vegetation model.
c   Global Change Biology, 9, 161-185
c 
c The further development of LPJ to LPJ-WHyMe is described in:
c  Wania R. 2007: Modelling northern peatland land surface processes,
c  vegetation dynamics and methane emissions. Ph.D. Thesis, University
c  of Bristol, United Kingdom.
c
c  The thesis is available as pdf at http://www.wania.net/work.html.
c
c The model simulates vegetation dynamics, hydrology and soil organic matter
c dynamics on an area-averaged grid cell basis using a one-year time step.
c Input parameters are monthly mean air temperature, total precipitation and
c percentage of full sunshine, annual atmospheric CO2 concentration and soil
c texture class. The simulation for each grid cell begins from "bare ground",
c requiring a "spin up" (under non-transient climate) of c. 1000 years to
c develop equilibrium vegetation and soil structure.

c Vegetation is represented by a combination of the following plant functional
c types (PFTs)

c       1. tropical broadleaved evergreen tree
c       2. tropical broadleaved raingreen tree
c       3. temperate needleleaved evergreen tree
c       4. temperate broadleaved evergreen tree
c       5. temperate broadleaved summergreen tree    
c       6. boreal needleleaved evergreen tree
c       7. boreal needleleaved summergreen tree
c       8. boreal broadleaved summergreen tree
c       9. C3 perennial grass
c      10. C4 perennial grass
c      11. flood tolerant C3 graminoid
c      12. Sphagnum moss

c A minimum set of bioclimatic limits (minimum coldest month temperature for
c survival, maximum coldest month temperature for establishment, minimum
c growing degree days [GDD, on 5 degree base except for larches (2 degrees)] 
c for survival) constrain PFT
c occurrence and regeneration within grid cells. Woody PFTs (1-8 above) present
c within a given grid cell are represented by area-averages for the following
c state variables

c       nind        individual density (/m2)
c       lm_ind      individual leaf carbon mass (gC)
c       rm_ind      individual fine root carbon mass (gC)
c       sm_ind      individual sapwood carbon mass (gC)
c       hm_ind      individual heartwood carbon mass (gC)
c       crownarea   individual crown area (m2)

c Individual structure is ignored for grass PFTs (9, 10). Individual density is
c arbitrarily set to the value 1/m2, so that lm_ind and rm_ind represent grid
c cell area-averages (gC/m2). The variables sm_ind and hm_ind are undefined for
c grasses, while crownarea is set to the proportion of the grid cell not
c occupied by trees, and thereby available for colonisation by grasses (see
c below).

c Individual allometry for woody PFTs is defined by the following relations,
c which constrain allocation of annual PFT carbon increments to the three
c living biomass compartments (leaves, fine roots and sapwood):

c      (A) (leaf area) = latosa * (sapwood xs area)
c             (Pipe Model, Shinozaki et al. 1964a,b; Waring et al 1982)
c      (B) lm_ind = lmtorm * rm_ind
c      (C) height = allom2 * (stem diameter)**allom3
c      (D) crownarea = min (allom1 * (stem diameter)**reinickerp,
c                           crownarea_max)

c      where

c      (sapwood xs area) = lm_ind * sla / latosa
c      (stem diameter) = ( 4 * (sm_ind + hm_ind ) / wooddens / pi /
c                        allom2 )**( 1 / (2 + allom3) )
c      latosa, allom1, allom2, allom3, reinickerp, crownarea_max, sla and
c        wooddens are constants.
c      lmtorm is calculated annually based on root-zone soil water
c        availability.

c Grass PFT structure is based solely on equation (B) above.

c PFT proportional cover is represented by foliar projective cover (FPC), which
c in turn is a function of leaf area index (LAI). The relations are:

c      lai_ind  = lm_ind * sla / crownarea      (individual LAI)
c      fpc_ind  = 1 - exp (-0.5 * lai_ind)      (individual FPC)
c      fpc_grid = (crownarea * nind) * fpc_ind  (grid cell FPC)

c Competition for space among woody PFTs is assumed to constrain the sum of
c their grid cell FPCs to marginally less than 1. Establishment of new
c individuals (saplings) is confined to the area of the grid cell not already
c occupied by woody PFTs. The establishment rate per unit area is reduced by
c shading as total woody FPC approaches 1. Competition for light among trees
c and grasses causes grass PFT cover (represented by the variable crownarea) to
c be constrained to the proportion of the grid cell not occupied by trees.

c Carbon uptake through photosynthesis, plant evapotranspiration and soil water
c content (of an upper and lower soil layer) are calculated by a coupled
c photosynthesis and water balance model, closely based on that described by
c Haxeltine & Prentice (1996).  PFTs capture incoming radiation in proportion
c to their grid cell FPCs. The annual biomass increment is based on gross
c primary production (GPP), maintenance and growth respiration, reproduction
c costs, turnover of sapwood to heartwood, and production of leaf and root
c litter.

c Disturbance by fire may cause additional annual biomass losses.  The
c proportion of a grid cell burnt each year is based on litter density and
c moisture content. PFTs may differ in the flammability of their litter.

c Decomposition of litter and soil organic matter is based on temperature
c (Lloyd & Taylor 1994) and soil layer 1 moisture content (Foley 1995).
c Differential decomposition rates apply to the litter, 'fast' and 'slow'
c organic matter (carbon) pools.

c Additional documentation is provided by commenting throughout the source
c code.  References are cited where appropriate and listed at the end of this
c file.

c Changes to the shrub allometry were made according to A. Wolf's
c approach.  For this, the allom1 and allom2 are not the same parameters
c for all woody plants, but are added as pftparameters.  The leaf to
c sapwood ratio, so far described by one parameter (latosa) will be
c changed to be pft specific as well.  LPJ-GUESS has min. mortality
c (=inverse of non-stress longevity) implemented.  This version doesn't
c have min. mortality. Therefore, I haven't implemented any changes in
c the mortality SR yet.

c NOTE: LOG or ALOG are the natural logarithm!
c -----------------------------------------------------------------------------

      program lpj

      IMPLICIT NONE

      include "params.cb"
      include "inputvars.cb"
      include "soilvars.cb"
      include "vegvars.cb"
      include "ch4vars.cb"


c     LOCAL VARIABLES
      LOGICAL dogridcell        !whether more gridcell(s)
      LOGICAL doyear            !whether more year(s)
      INTEGER n                 !for output of pftpar
      REAL aalresp              !annual leaf respiration (gC/m2)
      REAL aaresp               !annual autotrophic respiration (gC/m2)
      REAL acflux_estab         !annual biomass increment due to
                                !establishment (gC/m2)
      REAL acflux_fire          !C flux to atmosphere due to fire (gC/m2)
      REAL afire_frac           !fraction of gridcell burnt this year
      REAL agpp(1:npft)         !annual gridcell GPP (gC/m2)
      REAL aintc                !annual interception evaporation (mm)
      REAL alresp(1:npft)       !annual gridcell leaf respiration (gC/m2)
      real xalresp(npft)        !annual leaf respiration per PFT(gC/m2)
      REAL apet                 !annual potential evaporation (mm/year)
                                !/*DIETER*/
      REAL apet_grid            !annual PET as grid cell means,
                                !incl. demand (mm)
      REAL anpp_add             !annual gridc. NPP of killed PFTs(gC/m2)
      REAL arunoff_drain        !annual drainage (layer 2 runoff) (mm)
      REAL arunoff_surf         !annual surface (layer 1) runoff (mm)
      REAL dpet(1:365)          !daily potential evapotranspiration (mm)
      REAL dphen(1:365,1:npft)  !net daily leaf-on fraction
      REAL dphen_t(1:365,1:npft) !daily leaf-on fraction due to
                                 !temperature phenology
      REAL dprec(1:365)         !daily precipitation (mm)
      REAL dsun(1:365)          !daily sunshine (% full sunshine)
      REAL dayl(1:365)
      REAL drh(365)             !daily C available for methanogens [gC
                                !m-2 d-1]
      REAL dw1(1:365)           !daily soil layer 1 water content
      REAL dwscal365(1:npft)    !daily water scalar for day 365
      REAL fire_length
      REAL gdd_buf(1:npft,1:climbuf) !buffer to store 'climbuf' years of
                                     !GDD totals
      REAL gdd20(1:npft)        !20-year average growing degree days
      REAL heatstress(1:npft)   !reduction in individual density (&
                                !establishment) due to heat induced
                                !mortality (indiv/m2)
      REAL k_fast_ave           !running average k_fast for subroutine
                                !littersom
      REAL k_slow_ave           !running average k_slow for subroutine
                                !littersom
      INTEGER leafondays(1:npft)
      INTEGER leafoffdays(1:npft)
      LOGICAL leafon(1:npft)
      REAL litter_decom_ave     !running average litter_decom for
                                !subroutine littersom
      REAL litter_ag_old(npft)  !above-ground litter after this years'
                                !decomposition (gC/m2)
      REAL litter_bg_old(npft)  !below-ground litter after this years'
                                !decomposition (gC/m2)
      REAL mapar(1:12)
      REAL mlai(1:12)           !monthly LAI per gridcell (only
                                !necessary for HYMN project)
      REAL mcan_cond_tot(1:12)   !canopy conductance summed over all PFTs
                                !(only necessary for HYMN project).
      REAL mdayl(1:12)          !mid-month daylength
      REAL mlresp(1:12,1:npft)  !monthly grid cell leaf respiration
                                !(gC/m2)
      REAL mphen(1:12,1:npft)
      REAL mpar(1:12)
      REAL mnpp_add(1:12)       !monthly gridc. NPP of killed PFTs(gC/m2)
      REAL mintc(1:12)          !monthly interception loss (mm)
      REAL mpet(1:12)           !monthly potential evaporation (mm)
      REAL mmelt(1:12)          !monthly snowmelt (mm)
      REAL mprec_melt(1:12)     !monthly precip plus meltwater (mm)
      REAL mpet_grid(1:12)      !monthly PET as grid cell means,
                                !incl. demand (mm)
      REAL mpar_day(1:12)       !mid-monthly PAR flux (J/m2/day)
      REAL permafrost_index     !permafrost index (unitless)
      REAL sinelat,cosinelat
      REAL sinehh(1:12),hh(1:12),qo(1:12)
      REAL u(1:12),v(1:12)
      REAL wscal(1:npft)        !mean daily water scalar (among leaf-on
                                !days) (0-1 scale)
      
      REAL xmphen(1:12*npft)    !work buffer for driver interface
      REAL xmnpp(1:12*npft)     !work buffer for driver interface
      

c     variables for the o18 analyses
      REAL xmcica(1:12*npft) !monthly average ci/ca 
      REAL mcica(1:12,1:npft)
      REAL gresp(1:12,1:npft)   ! monthly growth respiration
      REAL sresp(1:12,1:npft)   ! monthly sapwood respiration
      REAL rresp(1:12,1:npft)   ! monthly root respiration
      REAL aresp(1:12,1:npft)   ! monthly autotrophic respiration

      INTEGER gridpoint         !counter for regional runs to monitor
                                !progress
      LOGICAL par_data

c -----------------------------------------------------------------------------
c                                MAIN PROGRAM
c -----------------------------------------------------------------------------
      INTEGER d, i

    

c     -------------------------------------------------------------------
c     LITTER SETUP
c     Decide which litter set up to use. Annual is the old way where
c     litter is updated only once a year and all of it is added to the
c     litterpool in December. This leads to jumps in HR from December to
c     January. When you want a monthly updated litter pool set
c     litter_monthly to false.
c     -------------------------------------------------------------------

      litter_monthly = .TRUE.

c     -------------------------------------------------------------------
c     SENSITIVITY TEST SETUP
c     If you're running the methane sensitivity test, where you read in a
c     series of parameter, set sens_test to TRUE.
c     -------------------------------------------------------------------
      sens_test = .FALSE.

      IF (sens_test) THEN
         open (UNIT = 22, file = 'param-list.dat', STATUS='OLD')
         read (22,*) senspar
         close (22)
      ENDIF


c     -----------------------
c     Initialise input/output
c     -----------------------

      gridpoint = 1

      call initio(met_data_type)

c     Obtain latitude and soil type for next gridcell from
c     input/output module

      call getgrid(lat,soilcode,dogridcell)

      do while (dogridcell)

c-----------------------------------      
c
c     INITIALISATION OF VARIABLES

         ! Simple data
         ! -----------
         aalresp = initval
         aaresp = initval
         acflux_estab = initval
         afire_frac = initval
         aintc = initval
         anpp_add = initval
         arunoff_drain = initval
         arunoff_surf = initval
         apet = initval
         apet_grid = initval
         cosinelat = initval
         fire_length = initval
         methane_mod = .FALSE.
         peathydr = .FALSE.
         permafrost_index = initval
         sinelat = initval
         year = 0

         ! Monthly data
         ! ------------
         DO m = 1, 12
            hh(m) = initval
            mapar(m) = initval
            mcan_cond_tot(m) = initval 
            mdayl(m) = initval
            mFwater1(m) = initval !RSCODE
            mFice1(m) = initval !RSCODE
            mintc(m) = initval
            mlai(m) = initval
            mmelt(m) = initval
            mpar_day(m) = initval
            mpet_grid(m) = initval
            mprec_melt(m) = initval
            mrh(m) = initval
            mtemp_soil(m) = initval
            mw1(m) = initval
            mw2(m) = initval
            mwtp(m) = initval
            qo(m) = initval
            sinehh(m) = initval
            u(m) = initval
            v(m) = initval
            DO pft = 1, npft
               aresp(m,pft) = initval
               gresp(m,pft) = initval
               mlresp(m,pft) = initval
               mgpp(m,pft) = initval
               mnpp(m,pft) = initval
               mphen(m,pft) = initval
               rresp(m,pft) = initval
               sresp(m,pft) = initval
            ENDDO
         ENDDO

        ! Daily data
        ! ----------
         DO d = 1, 365
            dayl(d) = initval
            dmelt(d) = initval
            dpet(d) = initval
            dprec(d) = initval
            drh(d) = initval
            drunoff_surf(d) = initval
            drunoff_drain(d) = initval
            dsun(d) = initval
            dtemp(d) = initval
            dw1(d) = initval
            ice1(d) = initval
            ice2(d) = initval
            intercp_tot(d) = initval
            snowdepth(d) = initval
            uw1(d) = initval
            uw2(d) = initval
            uw_ep(d) = initval
            wtp(d) = initval
            DO pft = 1, npft
               dphen(d,pft) = initval
               dphen_t(d,pft) = initval
            ENDDO
            DO i = 1, NLAYERS
               CH4(d,i) = initval
               CO2_soil(d,i) = initval
               O2(d,i) = initval
               CH4_gas(d,i) = initval
               CH4_diss(d,i) = initval
               CH4_diss_mol(d,i) = initval
               CH4_oxid(d,i) = initval
               CH4_prod(d,i) = initval
               CO2_prod(d,i) = initval
               O2_plant(d,i) = initval
               Fice(d,i) = initval
            ENDDO
         ENDDO

      ! Per pft data
      ! ------------
         DO pft = 1, npft
            allom2(pft) = initval
            alresp(pft) = initval
            xalresp(pft) = initval
            anpp(pft) = initval
            agpp(pft) = initval
            bm_inc(pft) = initval
            crownarea(pft) = initval
            dwscal365(pft) = initval
            evergreen(pft) = .FALSE.
            fpc_grid(pft) = initval
            gdd20(pft) = initval
            grass(pft) = .FALSE.
            heatstress(pft) = initval
            height(pft) = initval
            hm_ind(pft) = initval
            lai_ind(pft) = initval
            latosa(pft) = initval
            litter_ag(pft) = initval
            litter_bg(pft) = initval
            lm_ind(pft) = initval
            moss(pft) = .FALSE.
            nind(pft) = initval
            present(pft) = .FALSE.
            raingreen(pft) = .FALSE.
            rm_ind(pft) = initval
            sla(pft) = initval
            sm_ind(pft) = initval
            summergreen(pft) = .FALSE.
            tree(pft) = .FALSE.
            turnover_ind(pft) = initval
            wooddens(pft) = initval
            wscal(pft) = initval
            DO n = 1, npftpar
               pftpar(pft,n) = initval
            ENDDO
            DO n = 1, climbuf
               gdd_buf(pft,n) = initval
            ENDDO
         ENDDO


c     END of Initialisation
c
c-----------------------------------

c        -------------------------------
c        Start of LOOP THROUGH GRIDCELLS
c        -------------------------------

c        Obtain soil parameters

         call soilparameters(soilcode,soilpar)

c        --------------------------------------------------------------
c        SWITCH PEATLAND HYDROLOGY ON/OFF
c        Decide which hydrology you want to use.  If you want to switch
c        off the peatland hydrology, then set peathydr to FALSE for
c        soilcode 8. Peatland hydrology is always switched off for all
c        other soilcodes.
c        --------------------------------------------------------------

         IF (soilcode .EQ. 8) THEN
            peathydr = .TRUE.
         ELSE
            peathydr = .FALSE.
         END IF

c        -------------------------------------------------------------
c        SWITCH METHANE MODULE ON/OFF
c        This is only possible for soilcode 8.  The peatland hydrology
c        must be switched on to run the methane module.
c        -------------------------------------------------------------


         IF (peathydr .AND. soilcode .EQ. 8) THEN
            methane_mod = .TRUE.
         ELSE
            methane_mod = .FALSE.
         END IF


c        ----------------------------------
c        OUTPUT INFO WHILE RUNNING LPJ
c        Count gridcells (for regional run)
c        ----------------------------------
         print*, 'next gridcell, soilcode:', soilcode,
     $        ' gridpoint #:', gridpoint
         gridpoint = gridpoint + 1


c        Obtain plant functional type parameters, define sapling and initial
c        grass mass structure and calculate maximum crown area
        
         call pftparameters(pftpar,sla,tree,evergreen, summergreen
     $        ,raingreen,needle,boreal,peatland,lm_sapl,sm_sapl,hm_sapl,
     $        rm_sapl,latosa,allom1,allom2, wooddens,grass,year,moss
     $        ,inund_height,inund_dura)
        
c        -----------------------
c        Initialise year counter
c        -----------------------
         year=1
 

c       ------------------------------------------------------------
c       Obtain annual climate input parameters and atmospheric CO2
c       concentration from input/output module
c       MET DATA: met_data_type is set up by call to initio
c       ------------------------------------------------------------
         IF (met_data_type .EQ. MET_TYPE_DAILY) THEN
            call getclimatedaily(year,dtemp,dprec,msun,mtemp,mprec,co2
     $           ,doyear,met_data_type)
         ELSE
            call getclimatemonthly(year,mtemp,mprec,mwet,msun,co2,
     $           doyear,mtemp_prev,mtemp_next,met_data_type) 
         ENDIF

c        -----------------------------
c        CHECK CLIMATE DATA FOR ERRORS
c        -----------------------------
         DO m = 1, 12
            IF (mtemp(m) .LT. -100 .OR. mtemp(m) .GT. 100 ) THEN
               PRINT*, 'ERROR: mtemp values out of range', year, m,
     $              mtemp(m)
               stop
            ENDIF
            IF (mprec(m) .LT. 0 .OR. mprec(m) .GT. 30000 ) THEN
               PRINT*, 'ERROR: mprec values out of range', year, m,
     $              mprec(m)
               stop
            ENDIF
            IF (mwet(m) .LT. 0 .OR. mwet(m) .GT. 31 ) THEN
               PRINT*, 'ERROR: mwet values out of range', year, m,
     $              mwet(m)
               stop
            ENDIF
         ENDDO

         IF (met_data_type .NE. MET_TYPE_MONTHLY) THEN
            DO m = 1, 12
               mpar(m) = msun(m) * 0.12 * 3600 * 24 ! Convert uE/m2/s to J/m2/d
            ENDDO
         ENDIF
         

c     Gridcell initialisations
c     ------------------------

         call initgrid(anpp,aresp,arh,CH4,CH4_diss,CH4_diss_mol,CH4_gas
     $        ,CO2_soil,CH4_diff,CO2_diff,CH4_plant,CH4_oxid,CO2_plant
     $        ,CH4_ebull,CH4_prod,CO2_prod,cpool_fast,cpool_slow
     $        ,crownarea ,daet,daep,daept,dwscal365,estab,exu_pool
     $        ,fpc_grid,gdd,gresp,hm_ind,k_fast_ave,k_slow_ave,leafon
     $        ,leafoffdays,leafondays,litter_ag,litter_bg
     $        ,litter_decom_ave,litter_ag_old,litter_bg_old,lm_ind,mcica
     $        ,mCH4_plant,mCH4_diff,mCH4_ebull,mice_25,mice_50,mice_100
     $        ,mice_200,mgpp,mlresp,mnpp,mprec,mrh,mtemp_old,mtemp
     $        ,mtemp_max20,mtemp_min20,mtemp_max,mtsoil_0,mtsoil_10
     $        ,mtsoil_25,mtsoil_50,mtsoil_100,mtsoil_150,mtsoil_200,mw1
     $        ,mw2,O2,present,rm_ind,rresp,sm_ind,snow_days,snowdepth
     $        ,soilpar,sresp,survive,tree,uw1,uw2,w)

         do while (doyear)
c         --------------------------------------
c         Start of LOOP THROUGH SIMULATION YEARS
c         --------------------------------------

c         Interpolate monthly climate data to daily values
            IF (met_data_type .NE. MET_TYPE_DAILY) THEN
               call tempdaily(mtemp,mtemp_prev,mtemp_next,dtemp)
               call prdaily(mprec,dprec,mwet,year)

c          To use the old way of interpolation precipitation without the
c          weather generator, uncomment the next line.
c               call prdaily1(mprec,dprec,.true.)
               IF (met_data_type .NE. MET_TYPE_PAR) call daily(msun
     $              ,dsun) 
            ENDIF

c         Calculate mid-month daily photosynthetically active radiation flux,
c         daylength and mid-month daily potential evapotranspiration

            call petpar(year,dtemp,dsun,lat,mdayl,mpar_day,mpet,
     *           u,v,sinelat,cosinelat,sinehh,hh,qo)

c         IF you have monthly par values, calculate the daily ones by
c         using the DAILY SR instead of the PETPAR. 
            IF (met_data_type .NE. MET_TYPE_MONTHLY) THEN
               DO m = 1, 12
                  mpar_day(m) = mpar(m)
               ENDDO
            ENDIF

            call daily(mpet,dpet)
            call daily(mdayl,dayl) 

c         Adjust daily precipitation by snowmelt and accumulation in snowdepth
            call snow(dtemp,dprec,dmelt,snowdepth,msnowdepth,year,
     $           snowdens)

c         Calculate summergreen phenology

            call summerphenology(pftpar,mtemp,dtemp,gdd,dphen_t,
     *           summergreen,tree)

c     Calculation of 20-year average climate variables

            call climate20(mtemp,dtemp,gdd, gdd_buf,year,mtemp_min20
     $           ,mtemp_max20,gdd20,mtemp_max,pftpar) 

c     Implement PFT bioclimatic limits

            call bioclim(pftpar,peathydr,mtemp_min20,gdd ,mtemp_max
     $           ,survive,estab,year,moss,mtemp_max20,peatland,soilcode)

c     Calculation of GPP and soil water balance 

            call gpp(present,co2,soilpar,pftpar,peathydr,met_data_type,
     $           lai_ind,fpc_grid,mdayl,mtemp,mpar_day, dphen_t,w ,dpet
     $           ,dprec,dmelt,sla,agpp ,alresp,arunoff,mrunoff
     $           ,dwscal365 ,dphen,wscal, mgpp,mlresp,mw1,mw2,dw1,uw1
     $           ,uw2 ,aaet ,leafondays,leafoffdays,leafon,tree
     $           ,raingreen,mpar ,mapar ,mphen,year,maet,litter_ag,mcica
     $           ,lat,maep, aaep ,maept ,mintc,aintc,dayl,daep,daet
     $           ,daept ,intercp_tot, drunoff_surf ,drunoff_drain,uw_ep,
     $           wtp ,soilcode,moss, mwtp,snowdepth ,mthaw_depth,dtemp
     $           ,Tsoil_25,mtemp_soil ,permafrost_index, ice1 ,ice2
     $           ,thawdepth,snowdens,mtsoil_0 , mtsoil_10 ,mtsoil_25
     $           ,mtsoil_50,mtsoil_100 ,mtsoil_150 ,mtsoil_200, mice_25
     $           ,mice_50,mice_100 ,mice_200 ,mprec_melt,Fice
     $           ,inund_height ,inund_dura ,survive,Tsoil ,Fair,aalresp
     $           ,wtot,Fwater ,mcan_cond_tot,mlai,mFwater1 ,mFice1,Fpwp
     $           ,atsoil_25,frozen)


c     Calculation of autotrophic respiration and NPP

            call npp(soilcode,pftpar,senspar,sens_test,methane_mod,dtemp
     $           ,Tsoil_25,tree,dphen,nind,lm_ind,sm_ind,rm_ind,mgpp
     $           ,anpp,mnpp,anpp_red,mnpp_red,bm_inc,present,mlresp
     $           ,sresp ,rresp,gresp,aresp ,year,grass,moss,lm_ind_PFT11
     $           ,exudates ,aaresp,xalresp)


c     Allocation to reproduction

            call reproduction(bm_inc,litter_ag,present,litter_monthly)


c     Calculation of leaf, sapwood, and fine-root turnover

            call turnover(pftpar,present,lm_ind,sm_ind, hm_ind,rm_ind
     $           ,litter_ag,litter_bg,nind ,turnover_ind,moss
     $           ,litter_monthly)

c     Litter and soil decomposition calculations
c     This is done before fire, so that fire probability is calculated
c     on litter remaining after year's decomposition
            
            call littersom(senspar,sens_test,peathydr,litter_ag
     $           ,litter_bg, mw1,mtemp_soil ,cpool_fast ,cpool_slow, arh
     $           ,mrh,year,k_fast_ave ,k_slow_ave ,litter_decom_ave,moss
     $           ,exudates,litter_monthly ,litter_ag_old,litter_bg_old
     $           ,exu_pool)


c     Calculate methane emissions, run the subroutine only after 10
c     years of spin-up and only for organic soiltype.
c     If you want to use the methane subroutine, set soilcode .EQ. 8!
            IF (peathydr .AND. methane_mod .AND. year .GT. 10) THEN
               call methane(year, soilpar, senspar, sens_test, soilcode,
     $              peathydr,anpp_red, drh, mrh, Tsoil , Fair, Fpwp, wtp
     $              ,dphen,CH4, CH4_diss, CH4_diss_mol, CH4_gas ,
     $              CO2_soil, O2 , CH4_diff,CO2_diff, O2_diff ,CH4_plant
     $              , CH4_oxid, CO2_plant,O2_plant, CH4_ebull ,CH4_prod,
     $              CO2_prod, lm_ind_PFT11,Total_water,Fwater ,Fice
     $              ,mCH4_plant, mCH4_diff, mCH4_ebull,frozen
     $              ,CH4_plant_out)
            END IF

c     Removal of PFTs with negative C increment this year
            call killplants(bm_inc,present,tree,lm_ind,rm_ind,hm_ind,
     $           sm_ind,nind,litter_ag,litter_bg,moss)

c     Allocation of annual carbon increment to leaf, stem and fine root
c     compartments
            call allocation(pftpar,allom1,allom2,latosa,wooddens, tree
     $           ,grass,sla,wscal,nind, bm_inc,lm_ind ,sm_ind,hm_ind
     $           ,rm_ind,crownarea,fpc_grid, lai_ind,height ,litter_ag
     $           ,litter_bg,fpc_inc,present,year,moss)

c     Implement light competition between trees and grasses
            call light(present,tree,grass,lm_ind,sm_ind,hm_ind,rm_ind,
     $           crownarea,fpc_grid,fpc_inc,nind,litter_ag,litter_bg,
     $           sla,moss)

c     Implement light competition and background mortality among tree PFTs 
c     (including heat damage and due to lower limit of npp for boreal trees) 

            call mortality(pftpar,present,tree,bm_inc,turnover_ind, sla
     $           ,lm_ind,sm_ind,hm_ind,rm_ind,nind,litter_ag,litter_bg,
     $           dtemp,anpp,mtemp_max,moss)

c     Calculation of biomass destruction by fire disturbance

            call fire(pftpar,dtemp,litter_ag,acflux_fire,afire_frac,
     $           lm_ind,rm_ind,sm_ind,hm_ind,nind,dw1,present,tree,
     $           fire_length)
            
c     Establishment of new individuals (saplings) of woody PFTs,
c     grass establishment, removal of PFTs not adapted to current climate,
c     update of individual structure and FPC.

            call establishment(pftpar,present,survive,estab,nind,lm_ind,
     $           sm_ind,rm_ind,hm_ind,lm_sapl,sm_sapl,rm_sapl,hm_sapl,
     $           crownarea,fpc_grid,lai_ind,height, sla,wooddens,latosa
     $           ,mprec, litter_ag,litter_bg,tree,allom1,allom2,
     $           acflux_estab,leafondays, leafoffdays,leafon,mnpp,anpp
     $           ,mnpp_add,anpp_add,year,grass, moss,vegc)


c     OUTANNUAL
            call outannual(year,present,nind,lm_ind,rm_ind,sm_ind,hm_ind
     $           ,vegc,fpc_grid,anpp,acflux_estab,litter_ag ,litter_bg
     $           ,cpool_fast, cpool_slow,arh,aaresp,aalresp ,alresp
     $           ,xalresp,afire_frac,acflux_fire ,arunoff,sla ,mpar,
     $           mapar,anpp_add,mrunoff,aaet,mrh ,mnpp_add,maet
     $           ,mtemp_soil ,mw1,mw2,maep,aaep,maept ,apet,mintc ,aintc
     $           ,gdd,height ,mgpp,mwtp,mthaw_depth ,permafrost_index,
     $           msnowdepth,snowdepth,wtp,thawdepth ,mnpp,mtsoil_0,
     $           mtsoil_10,mtsoil_25 ,mtsoil_50,mtsoil_100 ,mtsoil_150
     $           ,mtsoil_200,mice_25 ,mice_50,mice_100,mice_200
     $           ,mprec_melt,CH4 ,CH4_diss,ch4_diss_mol ,CH4_gas
     $           ,CO2_soil ,O2 ,CH4_diff ,CH4_plant,CH4_plant_out
     $           ,CH4_ebull ,CO2_diff ,CO2_plant ,O2_diff,CH4_oxid
     $           ,O2_plant,CH4_prod ,CO2_prod ,Total_water,Tsoil,Fwater
     $           ,Fice,drh ,mCH4_plant ,mCH4_diff,mCH4_ebull, wtot
     $           ,drunoff_surf ,drunoff_drain ,mcan_cond_tot ,mlai
     $           ,mFwater1,mFice1 ,atsoil_25, dphen)


            
c     Increment year counter

            year=year+1

c     Obtain annual climate input parameters and atmospheric CO2
c     concentration from input/output module

c     MET DATA: met_data_type is set up by call to initio
            IF (met_data_type .EQ. MET_TYPE_DAILY) THEN
               call getclimatedaily(year,dtemp,dprec,msun,mtemp,mprec
     $              ,co2,doyear,met_data_type)
            ELSE
               call getclimatemonthly(year,mtemp,mprec,mwet,msun,co2,
     $              doyear,mtemp_prev,mtemp_next,met_data_type) 
            ENDIF
            IF (met_data_type .NE. MET_TYPE_MONTHLY) THEN
               DO m = 1, 12
                  mpar(m) = msun(m) * 0.12 * 3600 * 24 ! Convert uE/m2/s to J/m2/d
               ENDDO
            ENDIF


c     call getclimate(year,mtemp,mprec,mwet,msun,co2,doyear,
c     $         mtemp_prev,mtemp_next)  
         enddo                  !loop through simulation years



c     Final output to input/output module

         call outgrid()

c     Obtain latitude and soil type for next gridcell from
c     input/output module

         call getgrid(lat,soilcode,dogridcell)

      enddo                     !loop through gridcells

c     Terminate input/output

      call termio

      end



c -----------------------------------------------------------------------------
c                                SUBROUTINES
c -----------------------------------------------------------------------------

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE TEMPDAILY

c     LPJ-WHy: Linear interpolation of monthly to quasi-daily
c     values. This SR was added to read in the monthly values for the
c     previous and next year as well, so that Jan and Dec values could
c     be properly interpolated. Before that, the Dec temperature was
c     used to interpolate Jan temp of the same year and vice versa. That
c     led to jumps in the temperature curve when using time series
c     data. The problem did not show up when using the climatology.


      subroutine tempdaily(mval,mval_prev,mval_next,dval)

      implicit none

      include "params.cb"

c     ARGUMENTS:
      real mval(nmonth),mval_prev(nmonth),mval_next(nmonth),
     $       dval(ndayyear)

c     LOCAL VARIABLES:
      real dd,todaysval
      INTEGER dayofyear,today,month,midday(nmonth+2)
      INTEGER dayofmonth,day,nextmonth
        data (midday(month),month=1,14)
     *    / -15,16,44,75,105,136,166,197,228,258,289,319,350,381 /
      INTEGER count
      real dval_prem(ndayyear+31)
      

c     Calculate temperatures for the first 15 days in the year
        todaysval=mval_prev(12)
        count = 15
        dd=(mval(1)-mval_prev(12))/
     *       real(midday(2)-midday(1))
        DO dayofyear=1,15
           today = dayofyear
           count = count + 1
           today = dayofyear
           dval(today) = todaysval + dd * count
        ENDDO

        DO month=1,nmonth
           IF (month .LT. nmonth) THEN
              nextmonth = month + 1
           ELSE
              nextmonth = 1
           ENDIF

           IF (month .LT. nmonth) THEN
              dd = (mval(nextmonth) - mval(month)) /
     *             real(midday(month + 2) - midday(month + 1))
           ELSE
              dd = (mval_next(1) - mval(month)) /
     *             real(midday(month + 2) - midday(month + 1))
           ENDIF

           todaysval=mval(month)
           count = 0
           do dayofyear=midday(month+1),midday(month+2)-1
              count = count + 1
              if (dayofyear.le.ndayyear) then
                 today=dayofyear
                 dval(today)=todaysval + dd * count
              endif
           enddo
        ENDDO
        

        return
        end

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE DAILY
c     Linear interpolation of monthly to quasi-daily values

      subroutine daily(mval,dval)

      implicit none

      include "params.cb"

c     ARGUMENTS:
      real mval(nmonth),dval(ndayyear)

c     LOCAL VARIABLES:
      real dd,todaysval
      integer dayofyear,today,month,midday(nmonth+1)
      integer dayofmonth,day,nextmonth
        data (midday(month),month=1,13)
     *    / 16,44,75,105,136,166,197,228,258,289,319,350,381 /

      do month=1,nmonth
        if (month.lt.nmonth) then
          nextmonth=month+1
        else
          nextmonth=1
        endif
        dd=(mval(nextmonth)-mval(month))/
     *    real(midday(month+1)-midday(month))
        todaysval=mval(month)
        do dayofyear=midday(month),midday(month+1)-1
          if (dayofyear.le.ndayyear) then
            today=dayofyear
          else
            today=dayofyear-ndayyear
          endif
          dval(today)=todaysval
          todaysval=todaysval+dd
        enddo
      enddo
      
      return
      end

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE PRDAILY
c     Distribution of monthly precipitation totals to quasi-daily values

      subroutine prdaily(mval_prec,dval_prec,mval_wet,year)
      implicit none

      include "params.cb"
      

      integer year
      integer dayofmonth,day
      real mval_prec(nmonth),dval_prec(ndayyear),mval_wet(nmonth)
      real prob,prob_rain(nmonth)
      real mprec(nmonth)     !average precipitation on wet days
      real mprecip(nmonth)   !acc. monthly generated precipitation
      integer daysum	     !accumulated days at beginning of month

c     Declarations for the random generator
      integer k1(4),k2(4),k3(4),k4(4),k5(4),k6(4),k7(4),k8(4),k9(4),
     *         k10(4),k11(4)
       real vv,v1,random,c1,c2
       data k1 /9,98,915,92/, k2 /135,28,203,85/, k3 /43,54,619,33/, 
     *      k4 /645,9,948,65/, k5 /885,41,696,62/, k6 /51,78,648,0/, 
     *      k7 /227,57,929,37/, k8 /20,90,215,31/, k9 /320,73,631,49/,
     *      k10 /73,24,881,52/, k11 /6,302,597,3/
       c1=1.0  !normalising coefficient for exponential distribution       
       c2=1.2	!power for exponential distribution
       
      day=0
      prob=0.0
      do m = 1,12
         prob_rain(m)=0.0
         mprec(m)=0.0
         mprecip(m)=0.0
      enddo
      daysum=0

      do m=1,nmonth
         if(mval_wet(m).le.1.0) mval_wet(m)=1.0
         prob_rain(m)=mval_wet(m)/real(ndaymonth(m))
         mprec(m)=mval_prec(m)/mval_wet(m)
           do dayofmonth=1,ndaymonth(m)
              day=day+1
        
c Transitional probabilities (Geng et al. 1986)
                if (day.gt.1) then
                 if(dval_prec(day-1).lt.0.1) then
                  prob=0.75*prob_rain(m)
                 else 
                  prob=0.25+(0.75*prob_rain(m))
                 endif
                endif

c Determine wet days randomly and use Krysanova/Cramer estimates of 
c parameter values (c1,c2) for an exponential distribution
              vv=random(k1)
              if(vv.gt.prob) then
                dval_prec(day)=0.0 
              else
                v1=random(k5)
                dval_prec(day)=((-alog(v1))**c2)*mprec(m)*c1
                if(dval_prec(day).lt.0.1) dval_prec(day)=0.0
              endif
              mprecip(m)=mprecip(m)+dval_prec(day)
           enddo

c normalise generated precipitation by monthly CRU values
           if(m.gt.1) daysum=daysum+ndaymonth(m-1)
           if(mprecip(m).lt.1.0) mprecip(m)=1.0
           do dayofmonth=1,ndaymonth(m)
              day=daysum+dayofmonth
              dval_prec(day)=dval_prec(day)*(mval_prec(m)/mprecip(m))
              if (dval_prec(day).lt.0.1) dval_prec(day)=0.0
c              dval_prec(day)=mval_prec(m)/ndaymonth(m)  !no generator
           enddo


       

c Alternative: equal distribution of rain for fixed number of wet days
c                prob=prob_rain(m)+prob
c                  if(prob.ge.1.0) then   
c                    dval_prec(day)=mprec(m)
c                    prob=prob-1.0
c                  else
c                    dval_prec(day)=0.0
c                    prob=prob
c                  endif
      enddo !month 
      return
      end

c##############################
c Random function: provides random numbers {0-1}
       real function random(k)
       implicit none
       
       integer k(4),i
       
       k(4)=3*k(4)+k(2)
       k(3)=3*k(3)+k(1)
       k(2)=3*k(2)
       k(1)=3*k(1)
       i=k(1)/1000
       k(1)=k(1)-i*1000
       k(2)=k(2)+i
       i=k(2)/100
       k(2)=k(2)-100*i
       k(3)=k(3)+i
       i=k(3)/1000
       k(3)=k(3)-i*1000
       k(4)=k(4)+i
       i=k(4)/100
       k(4)=k(4)-100*i
       random=(((float(k(1))*0.001+float(k(2)))*0.01+float(k(3)))*0.001
     *        +float(k(4)))*0.01
     
       end

c\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
c     SUBROUTINE PRDAILY1
c     Linear interpolation of monthly to quasi-daily values

      subroutine prdaily1(mval,dval,iftotals)

      implicit none

      integer nmonth
        parameter (nmonth=12)        !number of months per year
      integer ndayyear
        parameter (ndayyear=365)     !number of days per year
      integer m,ndaymonth(1:12)    !number of days in each month
        data (ndaymonth(m),m=1,12)
     *    / 31,28,31,30,31,30,31,31,30,31,30,31 /

c     ARGUMENTS:
      real mval(nmonth),dval(ndayyear)

c     LOCAL VARIABLES:
      real dd,todaysval
      logical iftotals  !total values rather than averages
      integer dayofyear,today,month,midday(nmonth+1)
      integer dayofmonth,day,nextmonth
        data (midday(month),month=1,13)
     *    / 16,44,75,105,136,166,197,228,258,289,319,350,381 /

      do month=1,nmonth

        if (month.lt.nmonth) then
          nextmonth=month+1
        else
          nextmonth=1
        endif
        dd=(mval(nextmonth)-mval(month))/
     *    real(midday(month+1)-midday(month))
        todaysval=mval(month)
        do dayofyear=midday(month),midday(month+1)-1
          if (dayofyear.le.ndayyear) then
            today=dayofyear
          else
            today=dayofyear-ndayyear
          endif
          dval(today)=todaysval
          todaysval=todaysval+dd
        enddo

      enddo

      if (iftotals) then
        day=0
        do month=1,nmonth
          do dayofmonth=1,ndaymonth(month)
            day=day+1
            dval(day)=dval(day)/real(ndaymonth(month))
          enddo
        enddo
      endif

      return
      end

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE SOILPARAMETERS
c     Obtain soil parameter values given LPJ soil code
c     Soil data 1 and 2 in the array store are kept from LPJ. Soil data 3-5
c     are new in LPJ-WHy. Soilparameters 5-8 are new in LPJ-WHy.

      SUBROUTINE soilparameters(soilcode,soilpar)

      IMPLICIT NONE

      include "params.cb"
      include "soilvars.cb"

c     PARAMETERS:
      REAL k2
      PARAMETER (k2 = 2.0)

c     LOCAL VARIABLES:
      INTEGER i,j
      REAL store(1:5,1:9)

c     -----------------------------------------------------------------
c     Conversion table from BIOME3 soil codes to LPJ soil code
c     -----------------------------------------------------------------
c     BIOME3 soil(1)  BIOME3 soil(2)  LPJ soil code  Description
c     --------------  --------------  -------------  ------------------

c           1              any              1        coarse            
c           2              any              2        medium            
c           3               0               3        fine, non-vertisol
c           4              any              4        medium-coarse     
c           5              any              5        fine-coarse       
c           6              any              6        fine-medium       
c           7              any              7        fine-medium-coarse
c           8              any              8        organic           
c           3               1               9        fine, vertisol     

c     -----------------------------------------------------------------
c     R. Wania: It seems like there is a miss-classification, as
c     vertisols (soiltype 9) do not occur in permafrost areas or N of
c     50deg N. Most of the gridcells classified as soiltype 9 lie above
c     this limit though. So, I changed soil type 9 to 8.

c     Further DESCRIPTION of the soiltypes from the NOAH's LSM User's
c     Guide, Public Release Version 2.2, (last updated 25.June 2001)
c
c     Soil code     TEXTURE                 DESCRITIPTION    QUARTZ
c                                                            content

c        1          coarse                  loamy sand         0.82        
c        2          medium                  silty clay loam    0.10
c        3          fine, non-vertisol      light clay         0.25
c        4          medium-coarse           sandy loam         0.60
c        5          fine-coarse             sandy clay         0.52
c        6          fine-medium             clay loam          0.35
c        7          fine-medium-coarse      sandy clay loam    0.60
c        8          organic                                    0.40 

c     NOAH uses soil code 9 for land ice and describes it as loamy sand
c     with a quartz content of 0.82.

c     SOIL PARAMETERS
c     ---------------
c     1  k1 in percolation formula: Perc=k1*w1**k2
c     2  k2 in percolation formula:
c     3  soil layer 1 water holding capacity (relative)
c     4  soil layer 2 WHC (relative)
c     5  mineral content (volumetric fraction)
c     6  organic content (volumetric fraction)
c     7  Water content at permanent wilting point
c     8  porosity

c     Array 'store'
c     -------------
c     1  empirical parameter in percolation equation (K1) (mm/day)
c     2  volumetric water holding capacity at field capacity minus vol water
c        holding capacity at wilting point (Hmax), as fraction of soil layer
c        depth
c     3  volumetric fraction of organic material (m3 m-3) (Hillel, 1998)
c     4  water held below wilting point, important for heat conductance
c        From the AGRMET Handbook, 2002
c     5  porosity from AGRMET Handbook, 2002

c     NOTE: I left the values for soilcode 9 in there for soil maps
c     which still include 9 soilcodes.

      data ((store(i,j),i=1,5),j=1,9) / 

c    ---------------------------------------------------------------
c           1       2       3        4        5       soilcode
c    ---------------------------------------------------------------

     *    5.0,   0.110,   0.01,    0.029,  0.421,      ! 1
     *    4.0,   0.150,   0.01,    0.119,  0.464,      ! 2
     *    3.0,   0.120,   0.01,    0.139,  0.468,      ! 3
     *    4.5,   0.130,   0.01,    0.047,  0.434,      ! 4
     *    4.0,   0.115,   0.01,    0.020,  0.406,      ! 5
     *    3.5,   0.135,   0.01,    0.103,  0.465,      ! 6
     *    4.0,   0.127,   0.01,    0.069,  0.404,      ! 7
     *    9.0,   0.300,   0.20,    0.066,  0.800,      ! 8
     *    0.2,   0.100,   0.01,    0.364,  0.482/      ! 9

c     Define water holding capacity for both soil layers (fraction)  

      IF (soilcode .LT. 1 .OR. soilcode .GT. 9)
     $     print*, 'soil type not permitted' 

c     Define k1 value (texture-dependent)
      soilpar(1) = store(1,soilcode)
     
c     Define k2 value (not texture-dependant) 
      soilpar(2) = k2
      soilpar(3) = store(2,soilcode) + store(4,soilcode) !whc
      soilpar(4) = store(2,soilcode) + store(4,soilcode) !whc

c     Mineral content = 1 - organic content - porosity
      soilpar(5) = 1 - store(3,soilcode) - store(5,soilcode)
      soilpar(6) = store(3,soilcode)
      soilpar(7) = store(4,soilcode)
      soilpar(8) = store(5,soilcode)

      ! ERROR MESSAGE:
      !---------------
      IF (soilpar(5) .LT. -0.001) THEN
         PRINT*, 'ERROR: soilpar(5) is less than zero, check
     $              your soil parameters!'
         STOP
      ENDIF

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE PFTPARAMETERS
c     Assignment of PFT-specific parameters and bioclimatic limits
c     Definition of initial sapling and grass mass structure
c     Calculation of maximum crown area for woody PFTs
c     check differences between original pftparams and new ones!!!

      subroutine pftparameters(pftpar,sla,tree,evergreen, summergreen
     $     ,raingreen,needle,boreal,peatland,lm_sapl,sm_sapl,hm_sapl,
     $     rm_sapl ,latosa,allom1,allom2, wooddens,grass,year ,moss
     $     ,inund_height,inund_dura)

      implicit none

c     PARAMETERS:
      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"


c     LOCAL VARIABLES:
      integer n
      real table(1:npft,1:npftpar)
      real lai_sapl       !sapling or initial grass LAI
      real x
      real lmtorm         !non-waterstressed leafmass to rootmass ratio
      real stemdiam       !sapling stem diameter
      real height_sapl    !sapling height

c-----------------------------------------------------------------------------
 
c     PFT PARAMETERS
 
c      1  fraction of roots in upper soil layer - Rita: Haxeltine gives
c        very different values. Most of the root fractions are 0.33 in the upper
c        soil layer. Lund uses diff. values as well - values take from Wolf's
c        table for shrubs.
c     2  plants with C4 (1) or C3 (0) photosynthetic pathway
c      3  water scalar value at which leaves shed by drought deciduous PFT
c      4  canopy conductance component (gmin, mm/s) not associated with
c         photosynthesis (Haxeltine & Prentice 1996, Table 4)
c      5  maintenance respiration coefficient
c     acc. to TH the values seem to be too low, but the guess version uses
c     only 1.0, i.e. even lower values.?? 
c      6  flammability threshold
c      7  maximum foliar N content (mg/g)
c         (Haxeltine & Prentice 1996a, Fig 4) Rita: This parameter doesn't do
c         anything. It's called by GPP and Photosynthesis SRs, but never used!
c      8  fire resistance nindex
c      Parameter 9 and 10 for PFTs 1-10 are in accordance with the Sitch version
c      as the Web version was different but I couldn't find explanations in 
c      Gerten et al. 2004 for those differences.
c      9  leaf turnover period (years) Rita: in the SR TURNOVER this param. is
c         called longevity, what it actually represents since turnover is given
c         in year-1. 
c     10  leaf longevity (years) Rita: This is the same as param 9!
c     11  sapwood turnover period (sapwood converted to heartwood) (years)
c     12  root turnover period (years) - too high acc. to TH (Thomas Hickler)
c     13  leaf C:N mass ratio
c     14  sapwood C:N mass ratio
c     15  root C:N mass ratio
c     16  leaf type: broadleaved (1), needleleaved (2), grass (3) or moss (4)
c     17  phenology type: evergreen (1), summergreen (2), raingreen (3),
c         any type (4) 
c     18  leaf to root ratio under non-water stressed conditions
c     19  summergreen phenology ramp, GDD requirement to grow full leaf canopy
c     20  tree maximum crown area (m2)
c     21  sapling (or grass on initialisation) LAI
c     22  sapling [(heartwood mass) + (sapwood mass)] / (sapwood mass)
c     23  boreal pft (1), non-boreal pft (0)  Rita: I think that one doesn't do
c         anything. It was called by mortality SR, but not used there! 
c     24  low temperature limit for CO2 uptake
c     25  lower range of temperature optimum for photosynthesis
c         values taken from Wolf's shrub table. for PFT 8, I took the mean for
c         shade tolerant and shade intolerant trees.
c     26  upper range of temperature optimum for photosynthesis
c     27  high temperature limit for CO2 unptake
  
c     BIOCLIMATIC LIMITS
 
c     28 minimum coldest monthly mean temperature
c     29 maximum coldest monthly mean temperature
c     30 minimum growing degree days (at or above 5 deg C)
c        values taken from Wolf's shrub table. Mean values for shade tol.
c        and intol. used for PFT6 and 8.
c     31 upper limit of temperature of the warmest month 
c     32 lower limit of growth efficiency (g/m2) change to (kg C*m-2*yr-1)
c        (not in use)
    
c     PARAMETERS ADDED LATER
c
c     33 GDD base     
c     34 20-year average min warmest - coldest month temperature range
c     35 emax (mm/day)
c     36 intc (Interception storage parameter, unitless)

c     PARAMETERS FOR INUNDATION AND SHRUB ALLOMETRY (LPJ-WHy)
c     37 inund_height: max WTP tolerated
c     38 Constant 1 in allometry equation, allom1
c     39 Constant 2 in allometry equation, allom2
c     40 Leaf to sapwood area ratio (m2 * m-2), latosa
c     41 Wood density (gC * m-3), wooddens
c     42 max days of inundation tolerated              
 
c     PARAMETER FOR LITTER DENSITY 

c     43 Bulk density (kg biomass/m3) from SPITFIRE v1 (Find out what
c     the BD for dead sphagnum is - Frandsen, W.H. 1997)
c     44 Peatland PFT (0=not a peatland PFT, 1= peatland PFT)


c NOTE ON THE INTRODUCTION OF THE LARCH PFT:
c assume that Larch canopy conductance is higher than for other boreals
c assume that phenology ramp is quicker than for broadleaved trees
c also that ramp is applied to a GDD base of 2, not 5

c LPJ-WHy: GDD base for larches was set to 4 instead 2degC but the GDD
c was set to 350 days. This was done by running LPJ-WHy for different
c setups and deciding which combination worked best (visual comparison
c with distribution maps of larches only, no sophisticated technique
c applied!)

c The list below is not used anymore. PFT parameters are read in from the 
c pftin.dat file. Values between here and pftin.dat may vary.

c$$$      data ((table(pft,n),n=1,8),pft=1,npft) /
c$$$
c$$$c     ---------------------------------------------------------------------
c$$$c          1      2      3      4      5      6      7      8          PFT
c$$$c     ---------------------------------------------------------------------
c$$$
c$$$     *  0.85,   0.0,  0.00,   0.5,  0.20,  0.20, 100.0,  0.60,        !  1  
c$$$     *  0.60,   0.0,  0.20,   0.5,  0.20,  0.30, 100.0,  0.70,        !  2
c$$$     *  0.60,   0.0,  0.00,   0.3,  1.20,  0.30, 100.0,  0.12,        !  3
c$$$     *  0.70,   0.0,  0.00,   0.5,  1.20,  0.30, 100.0,  0.50,        !  4
c$$$     *  0.70,   0.0,  0.00,   0.5,  1.20,  0.30, 120.0,  0.12,        !  5
c$$$     *  0.90,   0.0,  0.00,   0.3,  1.20,  0.35, 100.0,  0.12,        !  6
c$$$     *  0.90,   0.0,  0.00,   0.3,  1.30,  0.35, 100.0,  0.12,        !  7
c$$$     *  0.90,   0.0,  0.00,   0.5,  1.20,  0.35, 100.0,  0.12,        !  8
c$$$     *  0.80,   0.0,  0.10,   0.5,  1.30,  0.20, 100.0,  0.01,        !  9
c$$$     *  0.80,   1.0,  0.10,   0.5,  1.20,  0.20, 100.0,  0.01,        ! 10
c$$$     *  1.00,   0.0,  0.00,   0.3,  1.20,  0.35,  30.0,  0.12,        ! 11
c$$$     *  1.00,   0.0,  0.00,   0.5,  1.20,  0.35,  30.0,  0.12,        ! 12
c$$$     *  0.90,   0.0,  0.35,   0.5,  1.20,  0.20,  30.0,  0.01,        ! 13
c$$$     *  1.00,   0.0,  0.10,   0.1,  2.00,  0.20,  20.0,  0.01/        ! 14
c$$$
c$$$      data ((table(pft,n),n=9,17),pft=1,npft) /
c$$$  
c$$$c     ---------------------------------------------------------------------
c$$$c          9     10     11     12     13     14     15     16     17   PFT
c$$$c     ---------------------------------------------------------------------
c$$$
c$$$     *   2.0,  2.00,  20.0,   2.0,  29.0, 330.0,  29.0,   1.0,   1.0, !  1
c$$$     *   1.0,  0.50,  20.0,   1.0,  29.0, 330.0,  29.0,   1.0,   3.0, !  2
c$$$     *   4.0,  2.00,  20.0,   4.0,  29.0, 330.0,  29.0,   2.0,   1.0, !  3
c$$$     *   1.0,  1.00,  20.0,   1.0,  29.0, 330.0,  29.0,   1.0,   1.0, !  4
c$$$     *   1.0,  0.50,  20.0,   1.0,  29.0, 330.0,  29.0,   1.0,   2.0, !  5
c$$$     *   2.0,  2.00,  20.0,   2.0,  29.0, 330.0,  29.0,   2.0,   1.0, !  6
c$$$     *   1.0,  0.50,  20.0,   1.0,  29.0, 330.0,  29.0,   2.0,   2.0, !  7
c$$$     *   1.0,  0.50,  20.0,   1.0,  29.0, 330.0,  29.0,   1.0,   2.0, !  8
c$$$     *   1.0,  1.00,   1.0,   2.0,  29.0,   0.0,  29.0,   3.0,   4.0, !  9
c$$$     *   1.0,  1.00,   1.0,   2.0,  29.0,   0.0,  29.0,   3.0,   4.0, ! 10 
c$$$     *   2.0,  2.00,  20.0,   2.0,  50.0, 330.0,  50.0,   1.0,   1.0, ! 11
c$$$     *   1.0,  0.50,  20.0,   1.0,  50.0, 330.0,  50.0,   1.0,   2.0, ! 12
c$$$     *   1.0,  1.00,   1.0,   2.0,  40.0,   0.0,  40.0,   3.0,   4.0, ! 13
c$$$     *   1.0,  1.00,   1.0,   1.0,  50.0,   0.0,   0.0,   4.0,   4.0/ ! 14
c$$$c$$$      data ((table(pft,n),n=18,23),pft=1,npft) /
c$$$
c$$$c     ---------------------------------------------------
c$$$c           18      19     20      21     22     23     pft
c$$$c     ---------------------------------------------------
c$$$
c$$$     *     1.0,  1000.0,  15.0,  1.500,  1.2,    0.0,    !  1
c$$$     *     1.0,  1000.0,  15.0,  1.500,  1.2,    0.0,    !  2
c$$$     *     1.0,  1000.0,  15.0,  1.500,  1.2,    0.0,    !  3
c$$$     *     1.0,  1000.0,  15.0,  1.500,  1.2,    0.0,    !  4
c$$$     *     1.0,   200.0,  15.0,  1.500,  1.2,    0.0,    !  5
c$$$     *     1.0,  1000.0,  15.0,  1.500,  1.2,    1.0,    !  6
c$$$     *     1.0,   100.0,  15.0,  1.500,  1.2,    1.0,    !  7
c$$$     *     1.0,   200.0,  15.0,  1.500,  1.2,    1.0,    !  8
c$$$     *    0.75,   100.0,   0.0,  0.001,  1.2,    1.0,    !  9
c$$$     *    0.75,   100.0,   0.0,  0.001,  1.2,    0.0,    ! 10  
c$$$     *     0.3,  1000.0,   1.0,  1.500,  1.2,    1.0,    ! 11
c$$$     *     0.3,   100.0,   1.0,  1.500,  1.2,    1.0,    ! 12
c$$$     *     0.4,   100.0,   0.0,  0.001,  1.2,    1.0,    ! 13
c$$$     *     1.0,    75.0,   0.0,  0.001,  1.2,    1.0/    ! 14     
c$$$
c$$$
c$$$      data ((table(pft,n),n=24,27),pft=1,npft) /
c$$$c     -------------------------------------
c$$$c          24     25     26      27    PFT
c$$$c     -------------------------------------
c$$$     *    2.0,   25.0,  30.0,   55.0, !  1
c$$$     *    2.0,   25.0,  30.0,   55.0, !  2
c$$$     *   -4.0,   20.0,  30.0,   42.0, !  3
c$$$     *   -4.0,   20.0,  30.0,   42.0, !  4
c$$$     *   -4.0,   20.0,  25.0,   38.0, !  5
c$$$     *   -4.0,   15.0,  25.0,   38.0, !  6
c$$$     *   -4.0,   15.0,  25.0,   38.0, !  7
c$$$     *   -4.0,   15.0,  25.0,   38.0, !  8
c$$$     *   -4.0,   10.0,  30.0,   45.0, !  9
c$$$     *    6.0,   20.0,  45.0,   55.0, ! 10    
c$$$     *   -4.0,   10.0,  25.0,   38.0, ! 11
c$$$     *   -4.0,   10.0,  25.0,   38.0, ! 12
c$$$     *   -4.0,   10.0,  30.0,   45.0, ! 13
c$$$     *   -4.0,    5.0,  25.0,   38.0/ ! 14
c$$$
c$$$
c$$$      data ((table(pft,n),n=28,32),pft=1,npft) /
c$$$
c$$$c     ------------------------------------------------------------
c$$$c          28       29       30       31       32       PFT
c$$$c     ------------------------------------------------------------
c$$$     *    15.5,  1000.0,    0.0,    1000.0,    0.0,     !  1
c$$$     *    15.5,  1000.0,    0.0,    1000.0,    0.0,     !  2
c$$$     *    -2.0,    22.0,  900.0,    1000.0,    0.0,     !  3 
c$$$     *     3.0,    18.8, 1200.0,    1000.0,    0.0,     !  4
c$$$     *   -17.0,    15.5, 1200.0,    1000.0,    0.0,     !  5
c$$$     *   -32.5,    -2.0,  550.0,      23.0,    0.0,     !  6
c$$$     * -1000.0,    -2.0,  550.0,      23.0,    0.0,     !  7
c$$$     * -1000.0,    -2.0,  550.0,      23.0,    0.0,     !  8
c$$$     * -1000.0,    15.5,    0.0,    1000.0,    0.0,     !  9
c$$$     *    15.5,  1000.0,    0.0,    1000.0,    0.0,     ! 10
c$$$     * -1000.0,    -2.0,  100.0,    1000.0,    0.0,     ! 11
c$$$     * -1000.0,    -2.0,  100.0,    1000.0,    0.0,     ! 12
c$$$     * -1000.0,    15.5,    0.0,    1000.0,    0.0,     ! 13
c$$$     * -1000.0,    15.5,  100.0,    1000.0,    0.0/     ! 14
c$$$
c$$$      data ((table(pft,n),n=33,37),pft=1,npft) /
c$$$
c$$$c     ---------------------------------------------
c$$$c          33       34        35    36      37      PFT
c$$$c     ---------------------------------------------
c$$$     *     5.0,  -1000.0,     7.0,  0.02,   0.10,   !  1
c$$$     *     5.0,  -1000.0,     7.0,  0.02,   0.10,   !  2
c$$$     *     5.0,  -1000.0,     5.0,  0.06,   0.50,   !  3 
c$$$     *     5.0,  -1000.0,     5.0,  0.02,   0.50,   !  4
c$$$     *     5.0,  -1000.0,     5.0,  0.02,   0.70,   !  5
c$$$     *     5.0,  -1000.0,     5.0,  0.06,   1.20,   !  6
c$$$     *     4.0,   43.0,       5.0,  0.06,   0.70,   !  7
c$$$     *     5.0,  -1000.0,     5.0,  0.06,   1.20,   !  8
c$$$     *     5.0,  -1000.0,     5.0,  0.01,   0.20,   !  9
c$$$     *     5.0,  -1000.0,     7.0,  0.01,   0.10,   ! 10
c$$$     *     5.0,  -1000.0,     5.0,  0.01,   0.40,   ! 11
c$$$     *     5.0,  -1000.0,     5.0,  0.02,   0.40,   ! 12
c$$$     *     5.0,  -1000.0,     5.0,  0.01,   0.05,   ! 13
c$$$     *     5.0,  -1000.0,     1.0,  0.01,   0.00/   ! 14
c$$$
c$$$      data ((table(pft,n),n=38,npftpar),pft=1,npft) /
c$$$
c$$$c     ---------------------------------------------
c$$$c          38       39       40        41      42      PFT
c$$$c     ---------------------------------------------
c$$$     *    100.0,   40.0,    6000.0,  2.0e5,   !0.250,   !  1
c$$$     *    100.0,   40.0,    6000.0,  2.0e5,   !0.250,   !  2
c$$$     *    100.0,   40.0,    6000.0,  2.0e5,   !0.250,   !  3 
c$$$     *    100.0,   40.0,    6000.0,  2.0e5,   !0.250,   !  4
c$$$     *    100.0,   40.0,    6000.0,  2.0e5,   !0.250,   !  5
c$$$     *    150.0,   40.0,    6000.0,  2.0e5,   !0.125,   !  6
c$$$     *    150.0,   40.0,    4000.0,  2.0e5,   !0.125,   !  7
c$$$     *    200.0,   40.0,    4000.0,  2.0e5,   !0.125,   !  8
c$$$     *      0.0,    0.0,       0.0,    0.0,   !0.800,   !  9
c$$$     *      0.0,    0.0,       0.0,    0.0,   !0.800,   ! 10
c$$$     *     10.0,    4.0,     125.0,  2.5e5,   !0.750,   ! 11
c$$$     *     10.0,    4.0,     125.0,  2.5e5,   !0.750,   ! 12
c$$$     *      0.0,    0.0,       0.0,    0.0,   !0.750,   ! 13
c$$$     *      0.0,    0.0,       0.0,    0.0/   !0.950,   ! 14
c$$$
c----------------------------------------------------------------------------

c     par#30: for PFT11-14, GDD 100 taken from
c      (Prentice et al. 1992, J.o.Biogeography)
c     par#33: for PFT11-14, a T0 is chosen to be 0deg C
c     (Prentice et al. 1992, J.o.Biogeography)


c     -----------------
c     for regional run
c     -----------------
      open (UNIT = 21, file = 'pftin.dat', STATUS='OLD')
      read (21,*) table
      close (21)

      do pft=1,npft

c       Transfer parameter values to array pftpar

c     LPJ-WHy: added moss pft below
        do n=1,npftpar
          pftpar(pft,n)=table(pft,n)
        enddo

c       Assign leaf and phenology logicals 

        if (pftpar(pft,16).le.2.0) then
           tree(pft)=.true.
           if (pftpar(pft,16).eq.2.0) then
              needle(pft)=.true.
           else
              needle(pft)=.false.              
           endif
        else
           tree(pft)=.false.
           needle(pft)=.false.
           if (pftpar(pft,16).eq.3.0) then
              grass(pft)=.true.
              moss(pft)=.false.
           else if (pftpar(pft,16).eq.4.0) then
              moss(pft)=.true.
              grass(pft)=.false.
           else
              grass(pft)=.false.
              moss(pft)=.false.
           endif
        endif

        if (pftpar(pft,17).eq.1.0) then
          evergreen(pft)=.true.
          summergreen(pft)=.false.
          raingreen(pft)=.false.
        elseif (pftpar(pft,17).eq.2.0) then
          evergreen(pft)=.false.
          summergreen(pft)=.true.
          raingreen(pft)=.false.
        elseif (pftpar(pft,17).eq.3.0) then
          evergreen(pft)=.false.
          summergreen(pft)=.false.
          raingreen(pft)=.true.
        else
          evergreen(pft)=.true.
          summergreen(pft)=.true.
          raingreen(pft)=.true.
        endif
        
        if (pftpar(pft,23).eq.1.0) then
          boreal(pft)=.true.
        else
          boreal(pft)=.false.
        endif            

        if (pftpar(pft,44).eq.1.0) then
           peatland(pft)=.true.
        else
           peatland(pft)=.false.
        endif

c       Calculate specific leaf area (SLA) for each PFT from leaf longevity
c       Include conversion (multiplier of 2.0) from m2/g(dry wt) to m2/gC
c       Equation based on Reich et al 1997, Fig 1f:

c       SLA = 2e-4 * exp(6.15 - 0.46 ln (leaf_longevity * 12))

c       SLA in m2/gC, leaf_longevity in years

        sla(pft)=2.0e-4*exp(6.15-0.46*log(pftpar(pft,10)*12.0))

c       Define initial mass structure

        lai_sapl=pftpar(pft,21)

        allom1(pft) = pftpar(pft,38)
        allom2(pft) = pftpar(pft,39)
        latosa(pft) = pftpar(pft,40)
        wooddens(pft) = pftpar(pft,41)

        if (tree(pft)) then     !woody PFTs

c         Calculate leafmass for a sapling individual
c          (1) lai = leafmass * sla / (crown area)
c          (2) (leaf area) = latosa * (sapwood xs area)
c                 (Pipe Model, Shinozaki et al. 1964a,b; Waring et al 1982)
c          (3) (crown area) = allom1 * (stem diameter) ** reinickerp
c                 (Reinickes theory)
c         From (1),
c          (4) leafmass = lai * (crown area) / sla
c         From (1) & (3),
c          (5) leafmass = lai * allom1 * (stem diameter)**reinickerp / sla
c         From (2),
c          (6) leafmass = latosa * (sapwood xs area) / sla
c          (7) (sapwood xs area) = pi * (sapwood diameter)**2 / 4
c         From (6) and (7),
c          (8) leafmass = latosa * pi * (sapwood diameter)**2 / 4 / sla
c         From (8),
c          (9) (sapwood diameter) = [ 4 * leafmass * sla / pi / latosa ]**0.5
c         (10) (stem diameter) = (sapwood diameter) + (heartwood diameter)
c         Define x,
c         (11) x = [ (sapwood diameter)+(heartwood diameter) ] / 
c                  (sapwood diameter)
c         From (10) & (11),
c         (12) (stem diameter) = x * (sapwood diameter)
c         From (5), (9) & (12),
c         (13) leafmass = lai * allom1 * x**reinickerp * 
c                       (4*leafmass*sla/pi/latosa)**(reinickerp*0.5) / sla
c         From (13),
c         (14) leafmass = [ lai * allom1 * x**reinickerp *
c              (4*sla/pi/latosa)**(reinickerp*0.5) / sla ]**(2/(2-reinickerp))

          x=pftpar(pft,22)

          lm_sapl(pft)=(lai_sapl*allom1(pft)*x**reinickerp*
     *      (4.0*sla(pft)/pi/latosa(pft))**(reinickerp*0.5) /
     $         sla(pft))**(2.0/(2.0-reinickerp))  !eqn 14

c         Calculate sapling stem diameter
c         From (9) & (12),
c         (15) (stem diameter) = x * [ 4 * leafmass * sla / pi / latosa ]**0.5

          stemdiam=x*(4.0*lm_sapl(pft)*sla(pft)/pi/latosa(pft))**0.5  !Eqn 15

c         Calculate sapling height
c         (16) height = allom2 * (stem diameter)**allom3 (source?)

          height_sapl=allom2(pft)*stemdiam**allom3   !Eqn 16

c         Calculate sapling sapwood mass
c         (17) (sapwood volume) = height * (sapwood xs area)
c         (18) (sapwood xs area) = leafmass * sla / latosa
c         From (17) & (18),
c         (19) (sapwood volume) = height * leafmass * sla / latosa
c         (20) (sapwood mass) = (wood density) * (sapwood volume)
c         From (19) & (20),
c         (21) (sapwood mass) = (wood density) * height * leafmass * sla /
c                latosa

          sm_sapl(pft)=wooddens(pft)*height_sapl*lm_sapl(pft)*sla(pft)/
     *      latosa(pft)   !Eqn 21

c         Calculate sapling heartwood mass
c         From (11),
c         (22) (heartwood mass) = (x-1) * (sapwood mass)

          hm_sapl(pft)=(x-1.0)*sm_sapl(pft)  !Eqn 22

        else ! grass or moss PFT

          lm_sapl(pft)=lai_sapl/sla(pft)

        endif

c       Calculate sapling or initial grass rootmass
c       (23) lmtorm = (leafmass) / (rootmass)

        lmtorm=pftpar(pft,18) 
        rm_sapl(pft)=(1.0/lmtorm)*lm_sapl(pft)  !From Eqn 23
      
c     For inundation stress
        inund_height(pft) = pftpar(pft,37)
        inund_dura(pft) = pftpar(pft,42)

      enddo ! pft loop
         
      return
      
      end


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE INITGRID
c     Initialise state variables (as required) for current grid cell

      subroutine initgrid(anpp,aresp,arh,CH4,CH4_diss,CH4_diss_mol
     $     ,CH4_gas,CO2_soil,CH4_diff,CO2_diff,CH4_plant,CH4_oxid
     $     ,CO2_plant,CH4_ebull,CH4_prod,CO2_prod,cpool_fast,cpool_slow
     $     ,crownarea ,daet,daep,daept,dwscal365,estab,exu_pool,fpc_grid
     $     ,gdd,gresp,hm_ind,k_fast_ave,k_slow_ave,leafon,leafoffdays
     $     ,leafondays,litter_ag,litter_bg,litter_decom_ave
     $     ,litter_ag_old,litter_bg_old,lm_ind,mcica,mCH4_plant
     $     ,mCH4_diff,mCH4_ebull,mice_25,mice_50,mice_100,mice_200 ,mgpp
     $     ,mlresp,mnpp,mprec,mrh,mtemp_old,mtemp,mtemp_max20
     $     ,mtemp_min20,mtemp_max,mtsoil_0,mtsoil_10,mtsoil_25
     $     ,mtsoil_50,mtsoil_100,mtsoil_150,mtsoil_200,mw1,mw2,O2
     $     ,present,rm_ind,rresp,sm_ind,snow_days,snowdepth ,soilpar
     $     ,sresp,survive,tree,uw1,uw2,w)

      IMPLICIT NONE
      include "params.cb"
      include "inputvars.cb"
      include "soilvars.cb"
      include "vegvars.cb"
      include "ch4vars.cb"

c     PARAMETERS
      real rainscalar
        parameter (rainscalar=2500.)


c     ARGUMENTS
      real k_fast_ave,k_slow_ave
      real litter_decom_ave
      real litter_ag_old(npft)  !above-ground litter after this years'
                                !decomposition (gC/m2)
      real litter_bg_old(npft)  !below-ground litter after this years'
                                !decomposition (gC/m2)
      real dwscal365(1:npft)
      integer leafondays(1:npft),leafoffdays(1:npft)
      logical leafon(1:npft)
      real mcica(1:12,1:npft)
      real gresp(1:12,1:npft)   ! monthly growth respiration
      real mlresp(1:12,1:npft)  ! monthly leaf respiration
      real sresp(1:12,1:npft)   ! monthly sapwood respiration
      real rresp(1:12,1:npft)   ! monthly root respiration
      real aresp(1:12,1:npft)   ! monthly autotrophic respiration

c     LOCAL VARIABLES
      integer i,d
      real aprec,air_temp

      arh = initval
      cpool_fast = 0.0
      cpool_slow = 0.0
      exu_pool = 0.0
      k_fast_ave = 0.0
      k_slow_ave = 0.0
      litter_decom_ave = 0.0
      mtemp_max = initval
      mtemp_max20 = initval
      mtemp_min20 = initval
      snow_days = 0

      DO d = 1, 365
         daep(d) = 0.0
         daet(d) = 0.0
         daept(d) = 0.0
         snowdepth(d) = 0.0
         CH4_diff(d) = 0.0
         CH4_plant(d) = 0.0
         CH4_ebull(d) = 0.0
         CO2_diff(d) = 0.0
         CO2_plant(d) = 0.0
         O2_diff(d) = 0.0
         DO i = 1, NLAYERS
            CH4(d,i) = 0.0
            CO2_soil(d,i) = 0.0
            O2(d,i) = 0.0
            CH4_diss(d,i) = 0.0
            CH4_diss_mol(d,i) = 0.0
            CH4_gas(d,i) = 0.0
            CH4_prod(d,i) = 0.0
            CH4_oxid(d,i) = 0.0
            CO2_prod(d,i) = 0.0
         ENDDO
      ENDDO

      do pft=1,npft
        present(pft)=.FALSE.
        if (.not.tree(pft)) crownarea(pft)=1.0 
        dwscal365(pft)=1.0 
        gdd(pft) = 0.0
        lm_ind(pft) = 0.0
        sm_ind(pft) = 0.0
        hm_ind(pft) = 0.0
        rm_ind(pft) = 0.0
        fpc_grid(pft) = 0.0    
        leafondays(pft)=0
        leafoffdays(pft)=0
        leafon(pft)=.FALSE.
        litter_ag(pft) = 0.0
        litter_bg(pft) = 0.0
        litter_ag_old(pft) = 0.0
        litter_bg_old(pft) = 0.0
        survive(pft) = .FALSE.
        estab(pft) = .FALSE.
        do m=1,12
           mnpp(m,pft) = 0.0
           mgpp(m,pft) = 0.0
           mcica(m,pft)=initval
           mlresp(m,pft)=initval
           sresp(m,pft)=initval
           rresp(m,pft)=initval
           gresp(m,pft)=initval
           aresp(m,pft)=0.0
        enddo          
        anpp(pft)=0.0
      enddo 
      aprec=0.0                        
      do m=1,12
         aprec=aprec+mprec(m)
         mtemp_old(m)=mtemp(m)
         mCH4_plant(m) = 0.0
         mCH4_diff(m) = 0.0
         mCH4_ebull(m) = 0.0
         mrh(m) = 0.0
         mtsoil_0(m) = 0.0
         mtsoil_10(m) = 0.0
         mtsoil_25(m) = 0.0
         mtsoil_50(m) = 0.0
         mtsoil_100(m) = 0.0
         mtsoil_150(m) = 0.0
         mtsoil_200(m) = 0.0
         mice_25(m) = 0.0
         mice_50(m) = 0.0
         mice_100(m) = 0.0
         mice_200(m) = 0.0
      enddo    


c     initialise soil water buckets
      air_temp=0.0
      do i=1,12   
         air_temp=air_temp+mtemp(i)/12.0
      enddo 
     
      if ((air_temp.le.0.0.and.mtemp(1).le.0.0)) then
         w(1)=max(aprec/rainscalar,0.1)
         if(w(1).ge.1.0) w(1)=1.0
         w(2)=1.0
         do m=1,12
            mw1(m)=w(1)
            mw2(m)=w(2)
         enddo
         do d = 1, 365
            uw1(d)=w(1)*(d1*soilpar(3))
            uw2(d)=w(2)*(d2*soilpar(4))
         enddo
      else
         w(1)=max(aprec/rainscalar,0.1)
         if(w(1).ge.1.0) w(1)=1.0
         w(2)=1.0
         w(2)=w(1) 
         do m=1,12
            mw1(m)=w(1)
            mw2(m)=w(2)
         enddo
         do d = 1, 365
            uw1(d)=w(1)*(d1*soilpar(3))
            uw2(d)=w(2)*(d2*soilpar(4))
         enddo
      endif

      return
      end
c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE PETPAR
c     Calculates mid-month daily photosynthetically active radiation flux,
c     daylength and daily potential evapotranspiration given temperature,
c     sunshine percentage and latitude
          
      subroutine petpar(year,dtemp,dsun,lat,mdayl,mpar_day,mpet,
     *      u,v,sinelat,cosinelat,sinehh,hh,qo)

      implicit none

      include "inputvars.cb"
      include "params.cb"

c     PARAMETERS:
      real a,b                          !empirical constants (Eqn 18)
        parameter (a=107.0,b=0.2)       !(a in W/m2; b unitless)
      real qoo
        parameter (qoo=1360.0)          !solar constant (1360 W/m2)
      real beta                         !global average short-wave albedo
        parameter (beta=0.17)           !(fraction)
      real c,d                          !empirical constants (Eqn 1)
        parameter (c=0.25,d=0.5)        !(unitless)
      real degtorad                     !conversion factor from degrees to
        parameter (degtorad=0.01745329) !radians (pi/180)
      real k                            !conversion factor from solar angular
        parameter (k=13750.98708)       !units to seconds (12/pi*3600)

c     ARGUMENTS:
      real dsun(1:365)
      real mdayl(1:12),mpar_day(1:12)
      integer year
      real u(1:12),v(1:12)
      real sinelat,cosinelat
      real sinehh(1:12),hh(1:12),qo(1:12)

c     LOCAL VARIABLES:
      integer dm,day
      real ni,delta,w,rs_day
      real rl,gamma,lambda,s,uu,vv,hn
      integer midday(1:12)
      real mpet(1:12)
      data (midday(m),m=1,12)

     *  / 16,44,75,105,136,166,197,228,258,289,319,350 /

      day=0  
      do m=1,12  !months
 
        do dm=1,ndaymonth(m)   !days of month

          day=day+1  !increment day of year

          if (day.eq.midday(m)) then
             if (dsun(day).eq.0.0) dsun(day)=10
             ni=dsun(day)/100.0   !proportion of bright sunshine today

c         CALCULATION OF NET DOWNWARD SHORT-WAVE RADIATION FLUX
c         (also known as insolation or incident solar radiation)
c         Refs: Prentice et al 1993, Monteith & Unsworth 1990,
c               Henderson-Sellers & Robinson 1986
c          (1) rs = (c + d*ni) * (1 - beta) * Qo * cos Z * k
c                (Eqn 7, Prentice et al 1993)
c          (2) Qo = Qoo * ( 1 + 2*0.01675 * cos ( 2*pi*i / 365) )
c                (Eqn 8, Prentice et al 1993; angle in radians)
c          (3) cos Z = sin(lat) * sin(delta) + cos(lat) * cos(delta) * cos h
c                (Eqn 9, Prentice et al 1993)
c          (4) delta = -23.4 * pi / 180 * cos ( 2*pi*(i+10) / 365 )
c                (Eqn 10, Prentice et al 1993, angle in radians)
c          (5) h = 2 * pi * t / 24 = pi * t / 12
c              where rs    = instantaneous net downward shortwave radiation
c                            flux (W/m2 = J/m2/s)
c                    c, d  = empirical constants (c+d = clear sky
c                            transmissivity)
c                    ni    = proportion of bright sunshine
c                    beta  = average 'global' value for shortwave albedo
c                           (not associated with any particular vegetation)
c                    i     = julian date, (1-365, 1=1 Jan)
c                    Qoo   = solar constant, 1360 W/m2
c                    Z     = solar zenith angle (angular distance between the
c                            sun's rays and the local vertical)
c                    k     = conversion factor from solar angular units to
c                            seconds, 12 / pi * 3600
c                    lat   = latitude (+=N, -=S, in radians)
c                    delta = solar declination (angle between the orbital
c                            plane and the Earth's equatorial plane) varying
c                            between +23.4 degrees in northern hemisphere
c                            midsummer and -23.4 degrees in N hemisphere
c                            midwinter
c                    h     = hour angle, the fraction of 2*pi (radians) which
c                            the earth has turned since the local solar noon
c                    t     = local time in hours from solar noon
c
c         From (1) and (3), shortwave radiation flux at any hour during the
c         day, any day of the year and any latitude given by
c          (6) rs = (c + d*ni) * (1 - beta) * Qo * ( sin(lat) * sin(delta) +
c                   cos(lat) * cos(delta) * cos h ) * k
c         Solar zenith angle equal to -pi/2 (radians) at sunrise and pi/2 at
c         sunset.  For Z=pi/2 or Z=-pi/2,
c          (7) cos Z = 0
c         From (3) and (7),
c          (8)  cos hh = - sin(lat) * sin(delta) / ( cos(lat) * cos(delta) )
c               where hh = half-day length in angular units
c         Define
c          (9) u = sin(lat) * sin(delta)
c         (10) v = cos(lat) * cos(delta)
c         Thus 
c         (11) hh = acos (-u/v)
c         To obtain the net DAILY downward short-wave radiation flux, integrate
c         equation (6) from -hh to hh with respect to h,
c         (12) rs_day = 2 * (c + d*ni) * (1 - beta) * Qo *
c                       ( u*hh + v*sin(hh) ) * k
c         Define
c         (13) w = (c + d*ni) * (1 - beta) * Qo
c         From (12) & (13),
c         (14) rs_day = 2 * w * ( u*hh + v*sin(hh) ) * k

            if (year.eq.1) then
              qo(m)=qoo*(1.0+2.0*0.01675*cos(2.0*pi*real(day)/365.0))  !Eqn 2
              delta=-23.4*degtorad*cos(degtorad*360.0*                 !Eqn 4
     *               (real(day)+10.0)/365.0)   
              if (m.eq.1) then
                 sinelat=sin(lat*degtorad)   
                 cosinelat=cos(lat*degtorad)
              endif
              u(m)=sinelat*sin(delta) !Eqn 9
              v(m)=cosinelat*cos(delta) !Eqn 10 
     
c           Calculate half-day length in angular units, hh
c           In Eqn (11), hh defined for u in range -v to v
c           For u >= v, hh = pi (12 hours, i.e. polar day)
c           For u <= -v, hh = 0 (i.e. polar night)
              if (u(m).ge.v(m)) then       !polar day
                hh(m)=pi
              elseif (u(m).le.-v(m)) then  !polar night
                hh(m)=0.0
              elseif (-u(m)/v(m).gt.1) then  	!polar night
                hh(m)=0.0  
              elseif (-u(m)/v(m).lt.-1) then  	!polar day
                hh(m)=pi  
              else              	   !normal day and night
                hh(m)=acos(-u(m)/v(m))     !Eqn 11
              endif
              sinehh(m)=sin(hh(m)) 
            endif          

            w=(c+d*ni)*(1.0-beta)*qo(m)                              !Eqn 13

c           Obtain daylength in hours from hh
            mdayl(m)=24.0*hh(m)/pi

c           Calculate total net downward shortwave radiation for this day
            rs_day=2.0*w*(u(m)*hh(m)+v(m)*sinehh(m))*k  !Eqn 14
            if (rs_day.eq.0.0) then
     	    rs_day=2.0*w*(u(m)*hh(m)+v(m)*sin(0.002))*k  !Eqn 14 
            endif
c           Convert to PAR
            mpar_day(m)=rs_day*0.5

c         CALCULATION OF DAILY POTENTIAL EVAPOTRANSPIRATION
c         (PET, equilibrium evapotranspiration, or evaporative demand)
c         Refs: Jarvis & McNaughton 1986, Prentice et al 1993
c         (15) pet = ( s / (s + gamma) ) * rn / lambda
c                (Eqn 5, Prentice et al 1993)
c         (16) s = 2.503E+6 * exp ( 17.269 * temp / (237.3 + temp) ) /
c                  (237.3 + temp)**2
c                (Eqn 6, Prentice et al 1993)
c         (17) rn = rs - rl
c         (18) rl = ( b + (1-b) * ni ) * ( a - temp )
c                (Eqn 11, Prentice et al 1993)
c              where pet    = instantaneous evaporative demand (mm/s)
c                    gamma  = psychrometer constant, c. 65 Pa/K
c                    lambda = latent heat of vapourisation of water,
c                             c. 2.5E+6 J/kg
c                    temp   = temperature (deg C)
c                    rl     = net upward longwave radiation flux
c                             ('terrestrial radiation') (W/m2)
c                    a, b   = empirical constants
c         Note: gamma and lambda are weakly dependent on temperature. Simple
c               linear functions are used to obtain approximate values for a
c               given temperature
c         Define
c         (19) uu = w * u - rl
c         (20) vv = w * v
c         Total daily PET is instantaneous PET integrated over the period
c         during which rn >= 0. Limits for the integration (half-period
c         hn) are obtained by solving for
c         (21) rn=0
c         From (17) & (21),
c         (22) rs - rl = 0
c         From (6), (19), (20) and (22),
c         (23) uu + vv * cos hn = 0
c         From (23),
c         (24) hn = acos ( -uu/vv )
c         Integration of (15) w.r.t. h in the range -hn to hn leads to the
c         following formula for total daily PET (mm)
c         (25) pet=2*(s/(s+gamma)/lambda)*(uu*hn+vv*sin(hn))*k

          rl=(b+(1.0-b)*ni)*(a-dtemp(day))   !Eqn 18
c          rl=32.0*10e-5*(1.0+4.0*ni)*(100.0-dtemp(day))
          gamma=65.05+dtemp(day)*0.064
          lambda=2.495e6-dtemp(day)*2380.0
          s=2.503e6*exp(17.269*dtemp(day)/(237.3+dtemp(day)))/
     *      (237.3+dtemp(day))**2             !Eqn 16
          uu=w*u(m)-rl  !Eqn 19
          vv=w*v(m)     !Eqn 20
       
c         Calculate half-period with positive net radiation, hn
c         In Eqn (24), hn defined for uu in range -vv to vv
c         For uu >= vv, hn = pi (12 hours, i.e. polar day)
c         For uu <= -vv, hn = 0 (i.e. polar night)

          if (uu.ge.vv) then       !polar day
            hn=pi
          elseif (uu.le.-vv) then  !polar night
            hn=0.0
          else                     !normal day and night
            hn=acos(-uu/vv)        !Eqn 24
          endif

c Calculate total PET for this day
          mpet(m)=2.0*(s/(s+gamma)/lambda)*(uu*hn+vv*sin(hn))*k  !Eqn 25
          endif
        enddo  !ndaymonth loop 
        
      enddo    !month loop
             
      return
      end    

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE SNOW
c     Adjust daily precipitation by snowmelt and accumulation in snowdepth
c     Ref: Haxeltine & Prentice 1996
c     LPJ-WHy uses a varying snow density instead of a fixed one.

      subroutine snow(dtemp,dprec,dmelt,snowdepth,msnowdepth,year,
     $     snowdens)
      implicit none

      include "params.cb"

c     PARAMETERS:
      real km,tsnow
c     km changed from 0.7 to 3.0  7/00 for non-permafrost as in HBV
c     note below: new algorithm
      parameter (tsnow=-2.0, km=3.0)


      REAL snowdens_start       !snow density at start of snow season [kg m-3]
      REAL snowdens_end         !snow density at end of snow season [kg m-3]
      PARAMETER (snowdens_start = 150)
      PARAMETER (snowdens_end = 500)



c     ARGUMENTS:      
      integer year

      real dtemp(1:365),dprec(1:365)
      real snowdepth(1:365)     !in [mm]
      real dmelt(1:365)
      real msnowdepth(1:12)     !in [mm]
      REAL snowdens(1:365)      !snow density in [kg m-3]

c     LOCAL VARIABLES:
      integer d,dm
      INTEGER snow_days         !count of days with snowdepth > 0 mm
      INTEGER snow_days_prev    !count of days in last year with
                                !snowdepth > 0 mm
      LOGICAL all_melted        !indicates if there is still some snow to
                                !be melted
      real snowmelt,newsnow
      real swe                  !snow water equivalent [mm = kg m-2]
      real mswe(1:12)           ! monthly aver. snow water equivalent [m] 

      SAVE

c     INITIALISE
      IF (year .EQ. 1) THEN
         swe = 0
         snow_days = 1
         snow_days_prev = 1
         all_melted = .FALSE.
      ENDIF

      d = 0  
      DO m = 1, 12                 !months
        mswe(m) = 0.0
        msnowdepth(m) = 0.0
        DO dm = 1, ndaymonth(m)   !days of month

c       Calculate snow melt and new snow for today
          d = d + 1  !increment day of year
          IF (dtemp(d) .LT. tsnow) THEN
           newsnow = dprec(d)
           snowmelt = 0.0
           all_melted = .FALSE.
          ELSE
           newsnow=0.0 
c           snowmelt=km*(dtemp(d)-tsnow)
           snowmelt=(1.5+0.007*dprec(d))*(dtemp(d)-tsnow)
          ENDIF

c       Set maximum snowmelt to size of SWE
c          IF (.NOT. all_melted .AND. snowmelt .GT. swe) THEN
          IF (snowmelt .GT. swe) THEN
             snowmelt = swe
             IF (swe .GT. 0.0) all_melted = .TRUE.
             swe = 0.0
          ENDIF
          IF (all_melted) THEN
             snow_days_prev = snow_days
             snow_days = 1
             all_melted = .FALSE.
          ENDIF      

c       Update SWE store
          swe = MAX(0.0, swe + newsnow - snowmelt)
          IF (swe .GT. 0.01) THEN
             snow_days = snow_days + 1
          ENDIF

c     set maximum swe = 10m 
          swe = min(swe, 10000.0)

c     set maximum snow_days to 365 (in some areas you can have permanent
c     snowcover
          snow_days = min(snow_days, 365)

c     Monthly SWE [m]
          mswe(m) = mswe(m) + swe / ndaymonth(m) / 1000.0

c     Calculate SNOWDENS - use a fixed value for the first year, then
c     count the days on which you have a snowcover and use that estimate
c     for the next year. For the first 3/4 (or half, if desired) of the
c     snow cover duration, the snow density stays fixed, for the last
c     1/4 (or second half), the density increases linearly.
c     ------------------------------------------------------------------

          IF (year .LE. 2) THEN
             snowdens(d) = snowdens_start
          ELSE
             IF (snow_days .LT. snow_days_prev * 0.75) THEN
                snowdens(d) = snowdens_start
             ELSE
                IF (snow_days .LT. snow_days_prev) THEN
                   snowdens(d) =
     $                snowdens_start + (snowdens_end - snowdens_start) /
     $                (snow_days_prev * 0.25) *
     $                (snow_days - snow_days_prev * 0.75)
                ELSE
                   snowdens(d) = snowdens_end
                ENDIF
             ENDIF
          ENDIF

c     Calculate effective water supply (as daily values in mm).  SWE
c     units are either mm or kg m-2. Hence you divide kg m-2 by kg m-3
c     and you get m.  Multiply by 1000 to get from m to mm
          snowdepth(d) = swe / snowdens(d) * 1000 
          dprec(d) = dprec(d) - newsnow
          dmelt(d) = snowmelt
c     Calculate mean snow depth for output
          msnowdepth(m) = msnowdepth(m) + snowdepth(d) / ndaymonth(m)
       ENDDO 
      ENDDO

      RETURN
      END

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE SUMMERPHENOLOGY
c     Temperature-based phenology for summergreen PFTs

      subroutine summerphenology(pftpar,mtemp,dtemp,gdd,dphen_t,
     *  summergreen,tree)

      implicit none

      include "params.cb"

c     ARGUMENTS
      real pftpar(1:npft,1:npftpar)
      real mtemp(1:12)
      real dtemp(1:365)
      real gdd(1:npft)
      real dphen_t(1:365,1:npft)
      logical summergreen(1:npft),tree(1:npft)

c     LOCAL VARIABLES
      integer warmest,month,midsummer,firstday,pft,day
      real ramp
      real leafon
      integer cm1,cm2,cm3,coldest
      integer midday(1:12)
      data (midday(month),month=1,12)
     *  / 16,44,75,105,136,166,197,228,258,289,319,350 /
      integer d
      real aphen
      real gddbase          !GDD base for greenup, pft parameter

c     INITIALISE
c     ----------
      DO d = 1, 365
         DO pft = 1, npft
            dphen_t(d,pft) = 0.0
         ENDDO
      ENDDO

c     First find warmest month

      warmest=1
      do month=1,12
        if (mtemp(month).gt.mtemp(warmest)) warmest=month
      enddo 

c     find the coldest month
      coldest=1
      do month=1,12
        if (mtemp(month).lt.mtemp(coldest)) coldest=month
      enddo

      midsummer=-15
      do month=1,warmest
        midsummer=midsummer+ndaymonth(month)
      enddo

      do pft=1,npft
        
c        Now midsummer is middle day (roughly) of warmest month
c        Find day of leaf abscission at end of summer

         gddbase=pftpar(pft,33)
         firstday=midsummer+1
         do while (dtemp(firstday).ge.gddbase.and.
     *        firstday.ne.midsummer)
            firstday=firstday+1
            if (firstday.gt.365) firstday=1
         enddo


         if (summergreen(pft)) then !summergreen taxa
            ramp=pftpar(pft,19) !number of GDDs to attain full leaf cover
            
            if (firstday.eq.midsummer) then !no leaf abscission
               
               do day=1,365
                  dphen_t(day,pft)=1.0
               enddo

            else

               gdd(pft)=0.0     !accumulated growing degree days
               leafon=0.0       !proportional leaf-on today

               day=firstday+1 
               if (day.gt.365) day=365
c     This do while loop means that once the firstday is reached, leafon
c     is set to 0. This leads to an abrupt end of dphen in autum.
               do while (day.ne.firstday)
                  if (dtemp(day).gt.gddbase) then !growing day
                     gdd(pft)=gdd(pft)+dtemp(day)-gddbase
                     if (ramp.gt.0.0) then
                        leafon=min(gdd(pft)/ramp,1.0)
                     else
                        leafon=1.0
                     endif
                  endif
                  dphen_t(day,pft)=leafon
                  day=day+1
                  if (day.gt.365) day=1
               enddo              
            endif   


c     constrain woody deciduous phenology to max= 9 months 
            if (tree(pft)) then
               aphen=0.
               do day=1,365
                  aphen=aphen+dphen_t(day,pft) ! calculate the total
                                               ! number of days with
                                               ! foliage
               enddo 

               if (aphen.gt.210) then !limit summergreen to 210 days
                                      !(approx 5 months)
                  do d=midday(coldest),midday(coldest)+75 ! set 75 days
                                                          ! after
                                                          ! coldest with
                                                          ! no foliage
                     if (d.le.365) then
                        day=d
                     else
                        day=d-365      
                     endif
                     dphen_t(day,pft)=0.0
                  enddo
                  do d=midday(coldest)-75,midday(coldest)
                     if(d.ge.1) then
                        day=d
                     else
                        day=365+d
                     endif
                     dphen_t(day,pft)=0.0
                  enddo
               endif
            endif          

         else                   !non-summergreen taxa
            do day=1,365
               dphen_t(day,pft)=1.0
            enddo

         endif
         
      enddo                     !pft

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE CLIMATE20
c     Calculation of 20-year average coldest-month temperatures and
c     growing degree days

      subroutine climate20(mtemp,dtemp,gdd, gdd_buf,year,mtemp_min20
     $     ,mtemp_max20,gdd20,mtemp_max,pftpar)

      implicit none

      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"
      

c     ARGUMENTS
      real mtemp(1:12),dtemp(1:365)
      real mtemp_max_buf(1:climbuf) !buffer to store 'climbuf' years of
                                    !coldest month temperatures
                                    !temperature (deg C)
      real mtemp_min_buf(1:climbuf) !buffer to store 'climbuf' years of
                                    !coldest month temperatures
      real gdd_buf(1:npft,1:climbuf)
      real gdd20(1:npft)

c     LOCAL VARIABLES
      integer n,bufsize,d
      real mtemp_min
      real gddbase                    !GDD base for greenup, pft parameter

      SAVE

c     INITIALISATION
      mtemp_min = initval

c     Find mean temperature of the coldest month this year

      mtemp_min=mtemp(1)
      do m=2,12
        if (mtemp(m).lt.mtemp_min) mtemp_min=mtemp(m)
      enddo
     
c     Find mean temperature of the warmest month this year
      mtemp_max=mtemp(1)
      do m=2,12
        if (mtemp(m).gt.mtemp_max) mtemp_max=mtemp(m)
      enddo     

c     Find growing degree days

      do pft=1,npft
         gddbase=pftpar(pft,33)
         gdd(pft)=0.0
         do d=1,365
            gdd(pft)=gdd(pft)+max(dtemp(d)-gddbase,0.0)
         enddo
      enddo

      if (year.gt.climbuf) then

c       Shift stored yearly values up, adding new values for this year

        do n=2,climbuf
          mtemp_min_buf(n-1)=mtemp_min_buf(n)
          mtemp_max_buf(n-1)=mtemp_max_buf(n)
          do pft=1,npft
             gdd_buf(pft,n-1)=gdd_buf(pft,n)
          enddo
        enddo

        mtemp_min_buf(climbuf)=mtemp_min
        mtemp_max_buf(climbuf)=mtemp_max
        do pft=1,npft
           gdd_buf(pft,climbuf)=gdd(pft)
        enddo

      else

c       Average over number of simulation years so far if less than
c       climbuf (20)

        mtemp_min_buf(year)=mtemp_min
        mtemp_max_buf(year)=mtemp_max
        do pft=1,npft
           gdd_buf(pft,year)=gdd(pft)
        enddo

      endif

c     Calculate averages

      mtemp_min20=0.0
      mtemp_max20=0.0
      do pft=1,npft
         gdd20(pft)=0.0
      enddo

      bufsize=min(year,climbuf)

      do n=1,bufsize
        mtemp_min20=mtemp_min20+mtemp_min_buf(n)/real(bufsize)
        mtemp_max20=mtemp_max20+mtemp_max_buf(n)/real(bufsize)
        do pft=1,npft
           gdd20(pft)=gdd20(pft)+gdd_buf(pft,n)/real(bufsize)
        enddo
      enddo

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE BIOCLIM
c     Apply bioclimatic limits on PFT survival and establishment

      subroutine bioclim(pftpar,peathydr,mtemp_min20,gdd ,mtemp_max
     $     ,survive,estab,year,moss,mtemp_max20,peatland,soilcode)

      implicit none

      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"

c     ARGUMENTS
      real twmax

c     LOCAL VARIABLES
      real tcmin,tcmax,gddmin
      real min_temprange              !minimum 20-year average warmest-coldest
                                      !month temperature range

      do pft=1,npft

c       Limits based on 20-year running averages of coldest-month mean
c       temperature and growing degree days (5 degree base, except larches
c       (2 degrees)), and minimal temperature range required (larches).
c       For SURVIVAL, coldest month temperature and GDD should be
c       at least as high as PFT-specific limits.
c       For REGENERATION, PFT must be able to survive AND coldest month
c       temperature should be no higher than a PFT-specific limit.

        tcmin=pftpar(pft,28)   !PFT-specific minimum coldest-month temperature
        tcmax=pftpar(pft,29)   !PFT-specific maximum coldest-month temperature 
        gddmin=pftpar(pft,30)  !PFT-specific minimum GDD
        twmax=pftpar(pft,31)   !PFT-specific upper limit of
                               !warmest-month temperature
        min_temprange=pftpar(pft,34) !PFT-specific lower limit of 
                                     !20-year temperature range


        if (mtemp_min20 .ne. mtemp_min20) stop
        if (mtemp_min20.ge.tcmin) then
          survive(pft)=.true.
          if (mtemp_min20.le.tcmax.and.gdd(pft).ge.gddmin.and.
     *              mtemp_max.le.twmax) then
            estab(pft)=.true.
            !Restrict peatland PFTs to when peatland hydrology is
            !switched on and to the organic grid cells
            IF (peatland(pft) .AND. .not. peathydr) estab(pft) = .FALSE.
          else
            estab(pft)=.false.
         endif
        else
          survive(pft)=.false.
        endif

        if ((mtemp_max20-mtemp_min20).lt.min_temprange) then
           survive(pft)=.false.  
        endif 
      enddo   

      return
      end

          
c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE GPP
c     Calculation of GPP, explicitly linking photosynthesis and water
c     balance through canopy conductance feedback

      subroutine gpp(present,co2,soilpar,pftpar,peathydr,met_data_type,
     $     lai_ind,fpc_grid, mdayl,mtemp,mpar_day,dphen_t,w,dpet ,dprec
     $     ,dmelt,sla,agpp, alresp,arunoff,mrunoff,dwscal365,dphen
     $     ,wscal, mgpp,mlresp,mw1,mw2,dw1,uw1,uw2,aaet,leafondays
     $     ,leafoffdays,leafon,tree,raingreen,mpar,mapar,mphen,year,maet
     $     , litter_ag,mcica,lat,maep, aaep,maept,mintc,aintc,dayl,daep
     $     ,daet,daept,intercp_tot ,drunoff_surf,drunoff_drain,uw_ep,
     $     wtp ,soilcode,moss, mwtp ,snowdepth,mthaw_depth,dtemp
     $     ,Tsoil_25,mtemp_soil ,permafrost_index,ice1 ,ice2
     $     ,thawdepth ,snowdens ,mtsoil_0 ,mtsoil_10 ,mtsoil_25
     $     ,mtsoil_50,mtsoil_100 ,mtsoil_150 ,mtsoil_200, mice_25
     $     ,mice_50,mice_100,mice_200, mprec_melt ,Fice ,inund_height
     $     ,inund_dura,survive,Tsoil ,Fair,aalresp ,wtot,Fwater
     $     ,mcan_cond_tot,mlai,mFwater1,mFice1 ,Fpwp,atsoil_25,frozen)

      IMPLICIT NONE

      include "params.cb"
      include "inputvars.cb"
      include "soilvars.cb"
      include "vegvars.cb"

c     PARAMETERS

      real epsilon
        parameter (epsilon=0.1)  !min precision of solution in bisection method

      REAL acro_co2_max         ! CO2 concentration in acrotelm porewater
c      PARAMETER (acro_co2_max = 370) !atmospherich CO2 conc.
      PARAMETER (acro_co2_max = 934) !value taken from Smolders et
                                   !al. 2001 which give an average CO2
                                   !conc. of 70 sites as 934 mimol L-1

c     ARGUMENTS:
      real aalresp              ! annual leaf respiration (gC/m2)
      real agpp(1:npft),alresp(1:npft)
      real apet_grid,mpet_grid(1:12)
      real mdayl(1:12),mpar_day(1:12),dwscal365(1:npft)
      real dphen_t(1:365,1:npft)
      real dphen(1:365,1:npft),dpet(1:365)
      real wscal(1:npft), mlresp(1:12,1:npft)
      real dprec(1:365),dayl(1:365)
      real dw1(1:365)
      real aintc
      integer leafondays(1:npft),leafoffdays(1:npft)
      logical leafon(1:npft)
      real mphen(1:12,1:npft)
      real mpar(1:12)
      real mapar(1:12)
      real mintc(1:12)
      real mcica(1:12,1:npft)
      real f1,f2
      REAL mprec_melt(12)       !precipitation plus meltwater [mm]
      real apet
      REAL fluct(1:12)          !fluctuation of WTP


c     LOCAL:
      integer IDX               !soil layer index
      integer aleafdays(1:npft),b,d,dm, i
      real k(2)
      real ca(1:npft)
c      real ca
      real awscal(1:npft),dwscal(1:npft)
      real minwscal(1:npft),gminp(1:npft)
      real rootprop(1:2,1:npft)
      real emax(1:npft),intc(1:npft)
      real meanfpc(1:12,1:npft)
      real nmax(1:npft),mrunoff_surf(1:12),mrunoff_drain(1:12)
      real arunoff_surf,arunoff_drain
      real meangmin(1:12,1:npft),tsecs(1:12),fpar,gp(1:12)
      real meangc(1:12,1:npft)
      real rd,agd,dval(1:365),dgp(1:365,1:npft)
      real dgc(1:365,1:npft)
      real gpd,x1,x2,xmid,fmid,rtbis,dx,adt1,adt2,adtmm
      logical c4(1:npft)
      real longevity(1:npft)
      real inhibx1(1:npft),inhibx2(1:npft)
      real inhibx3(1:npft),inhibx4(1:npft)
      real dw2(1:365)
      real fpc_tree
      real prec(1:365)
      REAL count(1:npft)        !auxiliary variable for inundation stress
      REAL icount(1:12,1:npft)  !number of days where inundation exceeds
                                !max. inundation height
      REAL istress(1:npft)      !inundation stress
      REAL permafrost_index
      REAL thaw_top
      REAL w_ep                 !fraction of available water holding
                                !capiacity evap-layer
      REAL acro_co2             !acrotelm pore water CO2 concentration [miMol]
      REAL wtd
      REAL awtd                 !annual mean wtd [mm]
      REAL dmoss_cap(1:365)     !daily moss photosynthetic capacity
      REAL mmoss_cap(1:365)     !monthly moss photosynthetic capacity
      REAL awtp
c     For OUTPUT purposes only
      REAL can_cond(1:12,1:npft) !canopy conductance averaged over mean
                                 !FPC of the relevant PFT (only
                                 !necessary for HYMN project)
      REAL mcan_cond_tot(1:12)  !canopy conductance summed over all PFTs
                                !(only necessary for HYMN project).
      REAL lai_pft(1:12,1:npft) !monthly LAI per pft per gridcell (only
                                !necessary for HYMN project)
      REAL mlai(1:12)           !monthly LAI per gridcell (only
                                !necessary for HYMN project)

      integer yout              ! output variable only

      SAVE

c     INITIALISATIONS
c     ---------------

      IF (year .EQ. 1) THEN
         DO i = 1, NLAYERS
            Fpwp(i) = soilpar(7)
            DO d = 1, 365
               Fice(d,i) = 0.0
               wtot(d,i) = 0.0
               IF (i .LE. 2) THEN !NOTE: you need to change this if you
                                  !change the layer setup!!
                  Fwater(d,i) = 0.0
                  Fice(d,i) = 0.0
               ELSE
                  Fwater(d,i) = Fpwp(i)
               ENDIF
               Fair(d,i) = 0
               Tsoil(d,i) = 0.0
            ENDDO
         ENDDO
         DO pft = 1, npft
            dwscal(pft) = 0.0
         ENDDO
      ENDIF
      awtp = 0.0
      arunoff_surf=0.0
      arunoff_drain=0.0
      aintc=0.0
      aaet=0.0
      aaep=0.0
      aaept=0.0
      apet=0.0
c      ca=co2 * 1.0e-6
      atsoil_25 = 0.0
      thaw_top = 0.0

      DO d = 1, 365
         daept(d) = 0.0
         dmoss_cap(d) = 0.0
         drunoff_surf(d) = 0.0
         drunoff_drain(d) = 0.0
         intercp_tot(d) = 0.0
         prec(d) = 0.0
         Tsurf(d) = 0.0
         Tsoil_0(d) = 0.0
         Tsoil_5(d) = 0.0
         Tsoil_10(d) = 0.0
         Tsoil_15(d) = 0.0
         Tsoil_20(d) = 0.0
         Tsoil_25(d) = 0.0
         Tsoil_30(d) = 0.0
         Tsoil_40(d) = 0.0
         Tsoil_50(d) = 0.0
         Tsoil_60(d) = 0.0
         Tsoil_80(d) = 0.0
         Tsoil_100(d) = 0.0
         Tsoil_120(d) = 0.0
         Tsoil_150(d) = 0.0
         Tsoil_200(d) = 0.0
         uw_ep(d) = 0.0
         DO pft = 1, npft
            dgc(d,pft) = 0.0
            dgp(d,pft) = 0.0
         ENDDO
      ENDDO
      DO m=1,12
         fluct(m) = 0.0
         mrunoff_surf(m)=0.0
         mrunoff_drain(m)=0.0 
         mrunoff(m)=0.0
         maet(m)=0.0
         mintc(m)=0.0
         maep(m)=0.0
         maept(m)=0.0
         mprec_melt(m)=0.0
         mtemp_soil(m) = 0.0
         mtsoil_0(m)   = 0.0
         mtsoil_10(m)  = 0.0
         mtsoil_25(m)  = 0.0
         mtsoil_50(m)  = 0.0
         mtsoil_100(m) = 0.0
         mtsoil_150(m) = 0.0
         mtsoil_200(m) = 0.0
         mice_25(m)  = 0.0
         mice_50(m)  = 0.0
         mice_100(m) = 0.0
         mice_200(m) = 0.0
         mmoss_cap(m) = 0.0
         mcan_cond_tot(m) = 0.0
         mlai(m) = 0.0
         mFwater1(m) = 0.0  !RSCODE
         mFice1(m) = 0.0  !RSCODE
         DO pft = 1, npft
            mlresp(m,pft) = 0.0
         ENDDO
      ENDDO


      DO pft = 1, npft
         count(pft) = 0.0
         emax(pft) = 0.0
         gminp(pft) = 0.0
         intc(pft) = 0.0
         rootprop(1,pft) = 0.0
         rootprop(2,pft) = 0.0
      ENDDO
      DO i = 1, NLAYERS
         frozen(i) = .TRUE.
         DO d = 1, 365
            frozen_day(d,i) = .TRUE.
         ENDDO
      ENDDO

      IDX   = NLAYERS - NSOILL + 1

c     LPJ-WHy uses a different atmospheric CO2 concentration for mosses
c     as they can access the CO2 dissolved in acrotelm water. If the
c     water table is high, mosses can access all of the acrotelm CO2,
c     but as the water table drops they CO2 concentrations available to
c     mosses is a mixture between atmospheric CO2 and acrotelm CO2.
c     NOTE: for ideal gases, ppmv = not just mimol L-1 but also
c     mimol Mol-1.
c     The mean annual wtd of the previous year is used here, as the
c     water table is only calculated after the CO2 concentration is
c     needed

      DO pft = 1, npft
         IF (.not. moss(pft)) THEN
            ca(pft)=co2*1.0e-6  !from ppmv to mole fraction
         ELSE
            acro_co2 = MIN(acro_co2_max,
     $           acro_co2_max - (acro_co2_max - co2) / z1 * awtd)
            ca(pft) = acro_co2 * 1.0e-6 
         ENDIF
      ENDDO


c     Initialise awtd for the current year
      awtd = 0.0

      k(1)=soilpar(1)
      k(2)=soilpar(2)

      fpc_tree=0.0
      do pft=1,npft
        if (tree(pft).and.present(pft)) fpc_tree=fpc_tree+fpc_grid(pft)
      enddo

      do pft=1,npft
        longevity(pft)=pftpar(pft,10)
c
c      define pft inhibition function parameters
c
        inhibx1(pft)=pftpar(pft,24)
        inhibx2(pft)=pftpar(pft,25)
        inhibx3(pft)=pftpar(pft,26)
        inhibx4(pft)=pftpar(pft,27)

        if (present(pft)) then

          aleafdays(pft)=0
          awscal(pft)=0.0
          minwscal(pft)=pftpar(pft,3)
          if (pftpar(pft,2).eq.1.0) then
             c4(pft)=.true.
          else
             c4(pft)=.false.
          endif

c         gminp = PFT-specific min canopy conductance scaled by fpc
c         assuming full leaf cover

          gminp(pft)=pftpar(pft,4)*fpc_grid(pft)
          rootprop(1,pft)=pftpar(pft,1)
          rootprop(2,pft)=1.0-pftpar(pft,1)
          IF (moss(pft)) rootprop(2,pft) = 0.0
          emax(pft)=pftpar(pft,35)
          intc(pft)=pftpar(pft,36)
          agpp(pft)=0.0
          alresp(pft)=0.0
          nmax(pft)=pftpar(pft,7)

       endif
      enddo  !pft

      d=0  !day of year

      do pft=1,npft

         if (present(pft)) then

c         FIND THE POTENTIAL CANOPY CONDUCTANCE REALISABLE UNDER
c         NON-WATER-STRESSED CONDITIONS

            do m=1,12

c     Initialisations
               meanfpc(m,pft)=0.0
               meangmin(m,pft)=0.0
               meangc(m,pft)=0.0
               mphen(m,pft)=0.0
               mcica(m,pft)=0.0
               istress(pft)=0.0
               icount(m,pft) = 0.0
               can_cond(m,pft) = 0.0
               lai_pft(m,pft) = 0.0

               tsecs(m)=3600.0*mdayl(m) !number of daylight seconds/day

c     Calculate non-water-stressed net daytime photosynthesis
c     assuming full leaf cover

               fpar = fpc_grid(pft)

               call photosynthesis(ca(pft),mtemp(m),fpar,mpar_day(m)
     $              ,mdayl(m),c4(pft),sla(pft),nmax(pft),lambdam,rd ,agd
     $              ,adtmm,inhibx1(pft),inhibx2(pft) ,inhibx3(pft)
     $              ,inhibx4(pft),moss(pft),peathydr)

               if (tsecs(m).gt.0.0) then
c     Calculate non-water-stressed canopy conductance
c     (gp) mm/sec basis averaged over entire grid cell
c     Eqn 21 Haxeltine & Prentice 1996

                  gp(m)=(((1.6*adtmm)/(ca(pft)*(1.0-lambdam)))/tsecs(m))
     $                 + gminp(pft)
               else
                  gp(m)=0.0
               endif
            enddo               !month

c         Linearly interpolate mid-monthly gp to daily values

            call daily(gp,dval)
            do d=1,365
               dgp(d,pft)=dval(d)
            enddo

         endif
         
      enddo                     !pft


      d=0 !day of year
      do m=1,12
         mpar(m)=0.0
         mapar(m)=0.0
      enddo

      do m=1,12
c       CALCULATE ACTUAL EVAPOTRANSPIRATION AND SOIL WATER BALANCE TODAY
c       FOR EACH DAY THIS MONTH

c     Move the next few lines to initialisation above!!
        mw1(m)=0.0
        mw2(m)=0.0
        mwtp(m) = 0.0
        mthaw_depth(m) = 0.0
        do dm=1,ndaymonth(m)
          d=d+1

          do pft=1,npft
            if (present(pft)) then
c             Use yesterday's potential water scalar to determine
c             today's drought phenology 
              if (d.eq.1) then
                dwscal(pft)=dwscal365(pft)
              endif


c             Drought phenology and net phenology for today.
c             Drought deciduous PFTs shed their leaves when their 
c             water scalar falls below their PFT specific minimum 
c             value (minwscal). Leaves are replaced immediately (i.e. daily)
c             once the minimum water scalar is exceeded.  
              
              if (dwscal(pft).gt.minwscal(pft).and.leafon(pft)) then
                 dphen(d,pft)=dphen_t(d,pft)
                 leafondays(pft)=leafondays(pft)+1
              else
                 dphen(d,pft)=0.0
              endif

c  stop deciduous vegetation behaving like evergreen when climate
c  permits
         if (raingreen(pft).and.tree(pft)) then
            if (real(leafondays(pft)).ge.(365.0*longevity(pft))) then
                    leafoffdays(pft)=leafoffdays(pft)+1
                    leafon(pft)=.false.
              if (real(leafoffdays(pft)).ge.(365.0*longevity(pft))) then
                       leafoffdays(pft)=0
                       leafondays(pft)=0
                       leafon(pft)=.true.
                    endif
                 endif
              endif

            endif
          enddo  !pft

          call waterbalance(d,peathydr,present,rootprop,w,uw1,uw2,dgp
     $         ,dpet,dphen,dgc,dmelt,dprec,k,whc,drunoff_drain
     $         ,drunoff_surf ,dwscal,daet,m,year,fpc_grid,daep
     $         ,daept ,w_ep,tree, lai_ind ,intercp_tot,dayl,lat,gminp
     $         ,emax,intc ,uw_ep,prec, wtp ,soilcode,icount,pftpar, moss
     $         ,Fwater ,Fice,thaw_top,ice1,ice2 ,soilpar,snowdepth,wtd
     $         ,inund_height,dmoss_cap,wtot,Fwater_365 ,Fice_365,Fpwp)

          call soiltemp(d,year,pftpar,soilpar,soilcode,peathydr,dtemp
     $         ,snowdepth,thawdepth,Fwater,Fice,permafrost_index
     $         ,thaw_top ,Tsoil_0,Tsoil_5,Tsoil_10,Tsoil_15,Tsoil_20
     $         ,Tsoil_25,Tsoil_30,Tsoil_40,Tsoil_50,Tsoil_60,Tsoil_80
     $         ,Tsoil_100,Tsoil_120,Tsoil_150,Tsoil_200,Tsurf,dmelt
     $         ,snowdens,Fpwp,litter_ag,Tsoil,Fair,Fwater_365,Fice_365
     $         ,frozen_day)


c         Store today's water content in soil layer 1
          dw1(d)=w(1)
c          dw1(d)=w_ep !upper 20 cm only
          dw2(d)=w(2)

c         Increment monthly totals
          mrunoff_surf(m)=mrunoff_surf(m)+drunoff_surf(d)
          mrunoff_drain(m)=mrunoff_drain(m)+drunoff_drain(d)
          aaet=aaet+daet(d)
          maet(m)=maet(m)+daet(d)
          aaep=aaep+daep(d)
          aaept=aaept+daept(d)
          maep(m)=maep(m)+daep(d)
          maept(m)=maept(m)+daept(d)
          aintc=aintc+intercp_tot(d)
          mintc(m)=mintc(m)+intercp_tot(d)
          mprec_melt(m) = mprec_melt(m) + prec(d)
c         LPJ-WHy: Accumulate monthly moss cap (see SR Waterbalance for
c         details)
          mmoss_cap(m) = mmoss_cap(m) + dmoss_cap(d)

c         Increment monthly w total and monthly u total
          mw1(m)=mw1(m)+w(1)
          mw2(m)=mw2(m)+w(2)
          mwtp(m) = mwtp(m) + wtp(d)
          mthaw_depth(m) = mthaw_depth(m) + thawdepth(d)
          mtsoil_0(m)   = mtsoil_0(m) + Tsoil_0(d) 
          mtsoil_10(m)  = mtsoil_10(m) + Tsoil_10(d) 
          mtsoil_25(m)  = mtsoil_25(m) + Tsoil_25(d) 
          mtsoil_50(m)  = mtsoil_50(m) + Tsoil_50(d) 
          mtsoil_100(m) = mtsoil_100(m) + Tsoil_100(d) 
          mtsoil_150(m) = mtsoil_150(m) + Tsoil_150(d) 
          mtsoil_200(m) = mtsoil_200(m) + Tsoil_200(d) 
          mFwater1(m) = mFwater1(m) + Fwater(d,4) !make sure, the right layer 
          mFice1(m) = mFice1(m) + Fice(d,4) ! is used in both lines!
                                            ! Layer 4 means 0-10cm.
          awtd = awtd - wtp(d) / 365.0

c         Check if each soil layer was unfrozen for at least one day
c         during this year

          DO i = IDX, NLAYERS
             IF (frozen(i)) THEN
                frozen(i) = frozen_day(d,i)
             ELSE
                frozen(i) = .FALSE.
             ENDIF
          ENDDO
          
          do pft=1,npft
            if (present(pft)) then

c             Accumulate count of days with some leaf cover and
c             pft-specific annual water scalar used in allocation

               if (dphen(d,pft).gt.0.0) then
                aleafdays(pft)=aleafdays(pft)+1
                awscal(pft)=awscal(pft)+dwscal(pft)
               endif 

c             Accumulate mean monthly fpc, actual (gc) and minimum
c             (gmin) canopy conductances, incorporating leaf
c             phenology

              meanfpc(m,pft)=meanfpc(m,pft)+
     *          fpc_grid(pft)*dphen(d,pft)/real(ndaymonth(m))
              meangc(m,pft)=meangc(m,pft)+
     *          dgc(d,pft)/real(ndaymonth(m))
              meangmin(m,pft)=meangmin(m,pft)+
     *          gminp(pft)*dphen(d,pft)/real(ndaymonth(m))
              mphen(m,pft)=mphen(m,pft)+dphen(d,pft)/real(ndaymonth(m))
c            Save final daily water scalar for next year
             if (d.eq.365) dwscal365(pft)=dwscal(pft)

            endif
          enddo ! end pft loop
        enddo !day of month
        
        DO pft=1,npft
           IF (present(pft)) THEN
        
c       LPJ-WHy: Calculate monthly inundation stress. icount is the
c       number of days, the WTP was above the PFTs tolerance
c       level. Compare icount with inund_dura, i.e. the number of days,
c       the PFT can tolerate inundation.
              IF (icount(m,pft) .GE. inund_dura(pft)) THEN
                 istress(pft) = 1.0
                 count(pft) = count(pft) + 1
              ELSE 
                 istress(pft) = icount(m,pft) / inund_dura(pft) 
              ENDIF

c       Plants which experience high inundation stress (= 1.0) will be
c       killed this year (currently not implemented).
c              IF (count(pft) .GE. 2) survive(pft) = .FALSE.
                 
           ENDIF
        ENDDO


c     For OUTPUT purposes only:
        DO pft = 1, npft
           IF (present(pft)) THEN
              mcan_cond_tot(m) = mcan_cond_tot(m) + meangc(m,pft)
              lai_pft(m,pft) = lai_ind(pft) * meanfpc(m,pft) 
              mlai(m) = mlai(m) + lai_pft(m,pft)
           ENDIF
        ENDDO


c       Increment annual runoff totals

        mrunoff(m)=mrunoff_surf(m)+mrunoff_drain(m)
        arunoff_surf=arunoff_surf+mrunoff_surf(m)
        arunoff_drain=arunoff_drain+mrunoff_drain(m)

 
c       LPJ-WHy: Get mean moss cap (see SR Waterbalance for details)
        mmoss_cap(m) = mmoss_cap(m) / real(ndaymonth(m))


c       CALCULATE GPP FOR EACH PFT
c       Find water-limited daily net photosynthesis (And) and
c       ratio of intercellular to ambient partial pressure of CO2
c       (lambda) by solving simultaneously Eqns 2, 18 and 19
c       (Haxeltine & Prentice 1996). 

c       Using a tailored implementation of the bisection method
c       with a fixed 10 bisections, assuming root (f(lambda)=0)
c       bracketed by f(0.02)<0 and f(lambdam+0.05)>0.

        do pft=1,npft 

          if (present(pft)) then

c           Convert canopy conductance assoc with photosynthesis
c           (actual minus minimum gc) (gpd) from mm/sec to mm/day

            gpd=tsecs(m)*(meangc(m,pft)-meangmin(m,pft))
            fpar=meanfpc(m,pft)  !cover including phenology  

            if (gpd.gt.1.0e-5) then

c             Implement numerical solution

              x1=0.02           !minimum bracket of the root 
              x2=lambdam+0.05   !maximum bracket of the root
              rtbis=x1          !root of the bisection
              dx=x2-x1

              b=0  !number of tries towards solution
              fmid=epsilon+1.0

              do while (abs(fmid).gt.epsilon.and.b.le.10)
  
c               Abort search if >10 iterations needed to find solution

                b=b+1
                dx=dx*0.5
                xmid=rtbis+dx

c               Calculate total daytime photosynthesis implied by
c               canopy conductance from water balance routine and
c               current guess for lambda (xmid).  Units are mm/m2/day
c               (mm come from gpd value, mm/day)
c               Eqn 18, Haxeltine & Prentice 1996

                adt1=(gpd/1.6)*(ca(pft)*(1.0-xmid))

c               Call photosynthesis to determine alternative total
c               daytime photosynthesis estimate (adt2) implied by
c               Eqns 2 & 19, Haxeltine & Prentice 1996, and current
c               guess for lambda (xmid)

                call photosynthesis(ca(pft),mtemp(m),fpar,mpar_day(m)
     $               ,mdayl(m),c4(pft),sla(pft),nmax(pft),xmid,rd
     $               ,agd ,adt2,inhibx1(pft),inhibx2(pft) ,inhibx3(pft)
     $               ,inhibx4(pft),moss(pft),peathydr)

c               Evaluate fmid at the point lambda=xmid
c               fmid will be an increasing function with xmid, with
c               a solution (fmid=0) between x1 and x2  
        
                fmid=adt2-adt1

                if (fmid.lt.0.0) rtbis=xmid

              enddo !bisection

            else  !infinitesimal canopy conductance

              agd=0.0
              rd=0.0

            endif

c           Estimate monthly gross photosynthesis and monthly leaf
c           respiration from mid-month daily values
c           Agd = And + Rd (Eqn 2 Haxeltine & Prentice 1996)

            mgpp(m,pft)=real(ndaymonth(m))*agd

c           LPJ-WHy: reduce mgpp and mlresp linearly by average monthly
c           inundation stress

            mgpp(m,pft) = MAX(0.0, mgpp(m,pft) - (mgpp(m,pft) *
     $           istress(pft)))
            mlresp(m,pft)=real(ndaymonth(m))*rd
            mlresp(m,pft) = MAX(0.0, mlresp(m,pft) - (mlresp(m,pft) *
     $           istress(pft)))
            alresp(pft)=alresp(pft)+mlresp(m,pft)
            mcica(m,pft)=xmid
            mapar(m)=mapar(m)+mpar_day(m)*fpar*real(ndaymonth(m))*0.5
          else
           alresp(pft) = 0.0
          endif
        enddo  !pft


        mpar(m)=mpar(m)+mpar_day(m)*real(ndaymonth(m))
        mw1(m)=mw1(m)/real(ndaymonth(m)) 
        mw2(m)=mw2(m)/real(ndaymonth(m)) 

        ! Set monthly soil water contents to zero when they fall below a
        ! threshold value.
        IF (mw1(m) .LT. 0.0001) mw1(m) = 0.0
        IF (mw2(m) .LT. 0.0001) mw2(m) = 0.0
        mwtp(m) = mwtp(m) / real(ndaymonth(m)) 
        mthaw_depth(m) = mthaw_depth(m) / real(ndaymonth(m))
        mtsoil_0(m)   = mtsoil_0(m) / real(ndaymonth(m)) 

        DO pft = 1, npft 
           IF (present(pft)) THEN
c             LPJ-WHy: GPP of mosses and flood-tolerant graminoids are
c             artificially restricted to certain conditions. For the
c             moss restrictions, look at the end of the SR Waterbalance.

              IF (moss(pft)) mgpp(m,pft) = mgpp(m,pft) * mmoss_cap(m)

c     -------------------------------------------------------------------
c     LIMIT OF FLOOD-TOLERANT GRAMINOIDS

c     Flood tolerant graminoids are sparsely distributed in bogs but are
c     very abundant in fens. The water table in bogs fluctuates much
c     more than in fens and is generally higher in fens. As we can't
c     distinguish between those two habitats yet, as fens are influenced
c     by groundwater influx, I'm setting a limit for the maximum
c     productivity of flood tolerant graminoids.  When the annual
c     mean WTP stays close to the surface, they'll grow well, but when
c     the WTP drops below about 10cm, their NPP will be reduced.

c     -------------------------------------------------------------------

              IF (pft .EQ. 11 .AND. mwtp(m) .LT. -200) THEN
                 ! The following lines will reduce the mgpp of
                 ! flood-tolerant grasses when the water table drops
                 ! below -20cm. This should reflect that these grasses
                 ! are usually found where the water table stays close
                 ! to the surface for most of the time (i.e. in fens).
                 ! Note that mwtp is negative!
c$$$                 mgpp(m,pft) = mgpp(m,pft) + mgpp(m,pft) *
c$$$     $                (mwtp(m) + 100) / 200 

                 mgpp(m,pft) = mgpp(m,pft) + mgpp(m,pft) *
     $                (mwtp(m) + 200) / 200 
              ENDIF
              agpp(pft) = agpp(pft) + mgpp(m,pft)
           ENDIF
        ENDDO

        mtsoil_10(m)  = mtsoil_10(m) / real(ndaymonth(m)) 
        mtsoil_25(m)  = mtsoil_25(m) / real(ndaymonth(m)) 
        mtsoil_50(m)  = mtsoil_50(m) / real(ndaymonth(m)) 
        mtsoil_100(m) = mtsoil_100(m) / real(ndaymonth(m)) 
        mtsoil_150(m) = mtsoil_150(m) / real(ndaymonth(m)) 
        mtsoil_200(m) = mtsoil_200(m) / real(ndaymonth(m)) 
        mice_25(m) = mice_25(m) / real(ndaymonth(m)) 
        mice_50(m) = mice_50(m) / real(ndaymonth(m)) 
        mice_100(m) = mice_100(m) / real(ndaymonth(m)) 
        mice_200(m) = mice_200(m) / real(ndaymonth(m)) 
        mtemp_soil(m) = mtsoil_25(m) 
        mFwater1(m) = mFwater1(m) / real(ndaymonth(m))  !RSCODE
        mFice1(m) = mFice1(m) / real(ndaymonth(m))  !RSCODE
        awtp = awtp + mwtp(m)
        atsoil_25 = atsoil_25 + mtsoil_25(m) / 12
      enddo  !month

      awtp = awtp / 12

c     Convert water scalar to average daily basis using phenology

      do pft=1,npft
        if (present(pft)) then
          if (aleafdays(pft).ne.0) then
            wscal(pft)=awscal(pft)/real(aleafdays(pft))
          else
            wscal(pft)=1.0
          endif
c     LPJ-WHy: The following if statement is new and prevents LPJ-WHy
c     from entering a cycle, where dphen is alternating between 0 and 1
c     from one day to the other. This happens when supply is very low
c     (eg. Siberia).
          IF (wscal(pft) .EQ. 0) present(pft) = .FALSE.
        endif
      enddo

c     Calculate annual total runoff

      arunoff=arunoff_surf+arunoff_drain

c     Calculate annual leaf respiration
      
      aalresp = 0.0
      DO pft = 1, npft
         aalresp = aalresp + alresp(pft)
      ENDDO

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE PHOTOSYNTHESIS
c     Adapted from Farquhar (1982) photosynthesis model, as simplified by
c     Collatz et al 1991, Collatz et al 1992 and Haxeltine & Prentice 1996

      subroutine photosynthesis(ca,temp,fpar,par,dayl,c4,sla,nmax,
     *  lambda,rd,agd,adtmm,x1,x2,x3,x4,moss,peathydr)

      implicit none


c     PARAMETERS
      real po2,p,bc3,bc4,bc5,theta,q10ko,q10kc,q10tau,ko25,kc25,tau25
      real alphaa,alphac3,alphac4,cmass,lambdamc4,lambdamc3,n0,m
      real e0,t0c3,t0c4,cq,tk25,tmc4,tmc3,vmmax

      parameter (po2=20.9e3)    !O2 partial pressure in Pa
      parameter (p=1.0e5)       !atmospheric pressure in Pa
      parameter (bc3=0.015)     !leaf respiration as fraction of Vmax for C3 plants
      parameter (bc4=0.02)      !leaf respiration as fraction of vmax for C4 plants
      parameter (bc5=0.03)      !leaf respiration as fraction of vmax for mosses
      parameter (theta=0.7)     !colimitation (shape) parameter
      parameter (q10ko=1.2)     !q10 for temperature-sensitive parameter ko
      parameter (q10kc=2.1)     !q10 for temperature-sensitive parameter kc
      parameter (q10tau=0.57)   !q10 for temperature-sensitive parameter tau
      parameter (ko25=3.0e4)    !value of ko at 25 deg C
      parameter (kc25=30.0)     !value of kc at 25 deg C
      parameter (tau25=2600.0)  !value of tau at 25 deg C
      parameter (alphaa=0.6)    !fraction of PAR assimilated at
                                !ecosystem level relative to leaf level
      parameter (alphac3=0.08)  !intrinsic quantum efficiency of CO2 uptake in
                                !C3 plants
      parameter (alphac4=0.053) !C4 intrinsic quantum efficiency
      parameter (cmass=12.0)    !atomic mass of carbon
      parameter (cq=4.6e-6)     !conversion factor for solar radiation
                                !at 550 nm from J/m2 to E/m2 (E=mol
                                !quanta)
      parameter (lambdamc4=0.4) !optimal ratio of intercellular to
                                !ambient CO2 concentration (lambda) in
                                !C4 plants
      parameter (lambdamc3=0.8) !optimal (maximum) lambda in C3 plants
      parameter (n0=7.15)       !leaf N concentration (mg/g) not
                                !involved in photosynthesis - Rita: n0
                                !is not used
      parameter (m=25.0)        !corresponds to parameter p in Eqn 28,
                                !Haxeltine & Prentice 1996
      parameter (t0c3=250.0)    !base temperature (K) in Arrhenius
                                !temperature response function for C3
                                !plants
      parameter (t0c4=260.0)    !base temperature in Arrhenius func for C4 plants
      parameter (e0=308.56)     !parameter in Arrhenius temp response function
      parameter (tk25=298.15)   !25 deg C in Kelvin
      parameter (tmc3=45.0)     !maximum temperature for C3 photosynthesis
      parameter (tmc4=55.0)     !maximum temperature for C4 photosynthesis
c      parameter (vmmax=0.15)   ! use vmmax as a multiplier
      parameter (vmmax=14.5)    !maximum Vmax for mosses [gC m-2 d-1]
                                !from St.Hilaire et al. 2008 (summer
                                !Vmax of 14 umol m-2 s-1)

c     ARGUMENTS
      real lambda,par,dayl,temp,rd,agd,adtmm,ca,fpar,sla,nmax
      logical c4
      real x1,x2,x3,x4
      integer npft
      integer pft
      parameter (npft=12)
      logical moss, peathydr

c     LOCAL VARIABLES
      real apar,pi,ko,kc,tau,gammastar,c1,c2,b,phipi
      real s,sigma,vm,and,pa,je,jc,cn,tk,t0,adt
      real tstress,k1,k2,k3,low,high 

c     Return without performing calculations if daylength 0 hours

      if (dayl.lt.0.01) then
        agd=0.0
        adtmm=0.0
        rd=0.0
        return
      endif

c     APAR in J/m2/day
c     alphaa = scaling factor for absorbed PAR at ecosystem, versus leaf, scale
c     See Eqn 4, Haxeltine & Prentice 1996

      apar=par*fpar*alphaa

c     calculate temperate inhibition function

      if (temp.lt.x4) then
         k1=2.*ALOG((1./0.99)-1.)/(x1-x2)
         k2=(x1+x2)/2.
         low=1./(1.+ EXP(k1*(k2-temp)))
         k3=ALOG(0.99/0.01)/(x4-x3)
         high=1.-0.01*EXP(k3*(temp-x3))
         tstress=(low*high)
      else
         tstress=0.
      endif
      if (tstress.lt.1e-2) tstress=0.  

c     First calculate catalytic capacity of rubisco, Vm, assuming optimal
c     (non-water-stressed) value for lambda, i.e. lambdamc3

      if (.not.c4) then  !C3 photosynthesis

c       Temperature-adjusted values of kinetic parameters, Eqn 22,
c       Haxeltine & Prentice 1996a

        ko=ko25*q10ko**((temp-25.0)/10.0) !Michaelis constant of rubisco for O2
        kc=kc25*q10kc**((temp-25.0)/10.0) !Michaelis constant for CO2
        tau=tau25*q10tau**((temp-25.0)/10.0) !CO2/O2 specificity ratio

c       CO2 compensation point (CO2 partial pressure, Pa)
c       Eqn 8, Haxeltine & Prentice 1996

        gammastar=po2/(2.0*tau)

c       Convert ambient CO2 level, ca, from mole fraction to partial pressure
c       in Pa

        pa=ca*p

c       Non-water-stressed intercellular CO2 partial pressure in Pa
c       Eqn 7, Haxeltine & Prentice 1996

        pi=lambdamc3*pa

c       Calculation of C1C3, Eqn 4, Haxeltine & Prentice 1996
c       Notes: - there is an error in this equation in the above paper (missing
c                2.0* in denominator) which is fixed here (see Eqn A2, Collatz
c                et al 1991)
c              - There is no longer an explicit temperature inhibition function
c                (low-temperature inhibition is now done mechanistically
c                by imposing a temperature-dependent upper limit on Vm, see
c                below)
c              - There is no longer any reduction in maximum photosynthesis due
c                to leaf age (phic)
c              - alphaa, the upscaling parameter accounting for the reduction
c                in PAR utilisation in ecosystems compared with leaf level,
c                appears in the calculation of APAR instead of here
c              - Cmass, the atomic weight of carbon, used in unit conversion
c                from molC to g appears in the calculation of Vm instead of
c                here
 
        c1=tstress*alphac3*((pi-gammastar)/(pi+2.0*gammastar))

c       High temperature inhibition modelled primarily by suppression of LUE
c       by decreased relative affinity of rubisco for CO2 relative to O2 with
c       increasing temperature, but we also implement a step function to
c       prohibit any C3 photosynthesis above 45 degrees (Table 3.7, Larcher
c       1983)

        if (temp.gt.tmc3) c1=0.0

c       Calculation of C2C3, Eqn 6, Haxeltine & Prentice 1996

        c2=(pi-gammastar)/(pi+kc*(1.0+po2/ko))

        b=bc3   !Choose C3 value of b for Eqn 10, Haxeltine & Prentice 1996
        t0=t0c3 !base temperature for temperature response of rubisco

c       LPJ-WHy: Special case for mosses (following the value given by
c       A. Wolf)
        if (moss) b=bc5

      else  !C4 photosynthesis

c       Specify C1C4, C2C4
c       Eqns 14,15, Haxeltine & Prentice 1996
c       Notes:
c              - alphaa, the upscaling parameter accounting for the reduction
c                in PAR utilisation in ecosystems compared with leaf level,
c                appears in the calculation of APAR instead of here
c              - Cmass, the atomic weight of carbon, used in unit conversion
c                from molC to g appears in the calculation of Vm instead of
c                here
c              - parameter phipi is not needed for calculation of optimal Vm
c                which assumes optimal intercellular CO2 concentration
c                (lambdamc4)

        c1=tstress*alphac4

c       High-temperature inhibition modelled conservatively as a step function
c       prohibiting photosynthesis above 55 deg C (Table 3.7, Larcher 1983)

        if (temp.gt.tmc4) c1=0.0

        c2=1.0
        b=bc4    !Choose C4 value of b for Eqn 10, Haxeltine & Prentice 1996
        t0=t0c4  !base temperature for temperature response of rubisco

      endif

c     Eqn 13, Haxeltine & Prentice 1996
      s=(24.0/dayl)*b

c     Eqn 12, Haxeltine & Prentice 1996
      sigma=sqrt(max(0.0,1.0-(c2-s)/(c2-theta*s)))

c     Calculation of optimal rubisco capacity, Vm, in gC/m2/day
c     Eqn 11, Haxeltine & Prentice 1996
      vm=(1.0/b)*(c1/c2)*((2.0*theta-1.0)*s-(2.0*theta*s-
     *  c2)*sigma)*apar*cmass*cq

c      if (peathydr) vm = vmmax* vm
      if (peathydr) vm = MIN(vmmax, vm)

c     Now use this Vm value to calculate actual photosynthesis
      if (.not.c4) then  !C3 photosynthesis

c       Intercellular CO2 partial pressure in Pa
c       Eqn 7, Haxeltine & Prentice 1996
        pi=lambda*pa

c       Recalculation of C1C3, C2C3 with actual pi
        c1=tstress*alphac3*((pi-gammastar)/(pi+2.0*gammastar))
        if (temp.gt.tmc3) c1=0.0  !high-temperature inhibition

        c2=(pi-gammastar)/(pi+kc*(1.0+po2/ko))

      else  !C4 photosynthesis

c       Parameter accounting for effect of reduced intercellular CO2
c       concentration on photosynthesis, Phipi.
c       Eqn 14,16, Haxeltine & Prentice 1996
c       Fig 1b, Collatz et al 1992
 
        phipi=min(lambda/lambdamc4,1.0)
        c1=tstress*phipi*alphac4
        if (temp.gt.tmc4) c1=0.0  !high-temperature inhibition

      endif

c     je is PAR-limited photosynthesis rate molC/m2/h, Eqn 3
c     Convert je from daytime to hourly basis

c     Calculation of PAR-limited photosynthesis rate, JE, molC/m2/h
c     Eqn 3, Haxeltine & Prentice 1996

      je=c1*apar*cmass*cq/dayl
      
c     Calculation of rubisco-activity-limited photosynthesis rate JC, molC/m2/h
c     Eqn 5, Haxeltine & Prentice 1996

      jc=c2*vm/24.0

c     Calculation of daily gross photosynthesis, Agd, gC/m2/day
c     Eqn 2, Haxeltine & Prentice 1996
c     Note: - there is an error in this equation in the above paper (missing
c             theta in 4*theta*je*jc term) which is fixed here

      agd=(je+jc-sqrt((je+jc)**2.0-4.0*theta*je*jc))/(2.0*theta)*dayl


c     Daily leaf respiration, Rd, gC/m2/day
c     Eqn 10, Haxeltine & Prentice 1996

      rd=b*vm
      
c     Daily net photosynthesis (at leaf level), And, gC/m2/day

      and=agd-rd

c     Total daytime net photosynthesis, Adt, gC/m2/day
c     Eqn 19, Haxeltine & Prentice 1996

      adt=and+(1.0-dayl/24.0)*rd

c     Convert adt from gC/m2/day to mm/m2/day using
c     ideal gas equation

      adtmm=adt/cmass*8.314*(temp+273.3)/p*1000.0

      return
      end

      

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE WATERBALANCE
c     Calculations of soil hydrology, plant water balance, daily canopy
c     conductance and PFT daily water stress factor

c------------------------------------------------------------------------
c     LPJ-WHy

c     The hydrology scheme is split into 2 paths, the 'normal' one is
c     taken when soil code is not 8 and the new wetland hydrology one is
c     taken when soil code is 8 (organic).  Granberg et al. 1999's
c     approach is used to calculate WTD and the water content in each
c     layer of the unsaturated soil. Simpson's rule is used to integrate
c     over the area between the surface and the water table. The
c     acrotelm depth is assumed to stay constant, as over short time
c     scales, peat accumulates only very slowly. Little changes will be
c     balanced out by mosses growing on the top and drawing the water
c     upwards by capillary forces. Peat is not accumulated in this
c     version.
c------------------------------------------------------------------------

      SUBROUTINE waterbalance(d,peathydr,present,rootprop,w,uw1,uw2,dgp
     $     ,dpet,dphen,dgc,dmelt,dprec,k,whc,drunoff_drain,drunoff_surf
     $     ,dwscal,daet,m,year,fpc_grid,daep,daept,w_ep,tree,
     $     lai_ind ,intercp_tot,dayl,lat,gminp,emax,intc,uw_ep,prec, wtp
     $     ,soilcode,icount,pftpar, moss,Fwater,Fice,thaw_top,ice1,ice2
     $     ,soilpar,snowdepth,wtd,inund_height,dmoss_cap,wtot,Fwater_365
     $     ,Fice_365,Fpwp)

      IMPLICIT NONE

      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"


c     PARAMETERS:
c     -----------

      REAL alpham, gm, pt
      PARAMETER (gm = 3.26)     ! empirical parameter in demand function
      PARAMETER (alpham = 1.391) ! Priestley-Taylor coefficient (Demand)
      PARAMETER (pt = 1.32)     ! Priestley-Taylor coefficient


c     PARAMETERS for the calculation of the WTP
c     ------------------------------------------
      REAL Dz_sph               !depth of each sphagnum layer [mm]
      PARAMETER (Dz_sph = 10.0)

      INTEGER NSUBLAYERS
      PARAMETER (NSUBLAYERS = 30)

      REAL surfw                !surface water content when WTD is at 20 mm
      PARAMETER (surfw = 0.34)  

      REAL minvtot              !minimum water content at surface in
      PARAMETER (minvtot = 0.25) !mm3/mm3 look up Haywood and Clymo
                                 !1982)

      REAL zmin                 !limit where water content at surface =
      PARAMETER (zmin = 100.0)  !minuw1 (mm)

      REAL grad_lim             !gradient limit of upper gradient [mm]
      PARAMETER (grad_lim = 50.0)

c     ARGUMENTS:
c     ----------
      INTEGER d
      REAL Fpwp_m               !fraction of water held below perm. wilt. point
      REAL dayl(1:365)
      REAL dgc(1:365,1:npft)
      REAL dgp(1:365,1:npft)
      REAL dpet(1:365)
      REAL dphen(1:365,1:npft)
      REAL dprec(1:365)
      REAL dmoss_cap(1:365)     !daily moss photosynthetic capacity
      REAL dwscal(1:npft)
      REAL emax(1:npft)
      REAL Ftotal(NLAYERS)      !Water + ice fraction in each layer
      REAL gminp(1:npft)
      REAL intc(1:npft)         !LAI parameter (Kergoat 1998, Tab. 1)
      REAL icount(1:12,1:npft)  !number of days where inundation exceeds
                                !max. inundation height
      REAL k(1:2)               !variables in percolation equation
      REAL rootprop(1:2, 1:npft) !Fraction of roots in upper/lower layer
      REAL thaw_top             !thawing depth, where no more ice is found [mm]
      REAL w_ep                 !fractional soil moisture in evaporation layer

c     LOCAL

      INTEGER IDX, MIDX         !index variables
      INTEGER i, j, jj, mm, count   !index variables
      REAL aet(1:npft)
      REAL aettotal(1:2)
      REAL beta(1:2)
      REAL cover                ! currently plant-covered fraction of soil
      REAL demand
      REAL demandpot
      REAL demand_tot
      REAL excess               ! excess water in evaporation layer [mm]
      REAL f1
      REAL f2
      REAL int_store(1:npft)    !water stored in the canopy
      REAL intercep(1:npft)
      REAL lat
      REAL pet_grid
      REAL pet_s
      REAL pet_p
      REAL perc
      REAL perc_evap
      REAL prec(1:365)
      REAL q
      REAL supply
      REAL supply_tot
      REAL wet                  !fraction of day when canopy is wet
      REAL wr

c     NEW variables
      REAL acro_por             !acrotelm porosity minus gas fraction
      REAL awhc(NLAYERS)        !actual WHC in upper and lower layer -
                                !updated with ice content
      REAL az                   !linearly decreasing gradient in the top 100 mm
      REAL az1                  !linearly decreasing gradient in the top
                                !peat (0 - 50mm) [mm-1]
      REAL az2                  !linearly decreasing gradient in the 
                                !peat (50 - 100mm) [mm-1]
      REAL depth                !soil depth [mm]
      REAL Dz(NLAYERS)          !soil layer thickness [mm]
      REAL Dz_sub               !Thickness of sub layers in acrotelm
      REAL even                 !dummy variable for Simpson's rule
      REAL fcoarse              !fraction of coarse material for wetland
                                !drainage
      REAL fpc_moss_total       !needed to calculate daep from mosses
      REAL frac                 !a weighting fraction for each soil
                                !layer depending on its depth and the
                                !depth of the upper/lower soil layer
      REAL itol(1:npft)         !inundation tolerance factor
      REAL surfw_sat            !water content at the surface [mm3 mm-3]
      REAL sphlayer             !sphagnum layer
      REAL sub_water(1:30)      !water in top 300mm
      REAL subsub_water(1:10)   !water in top 10mm of acrotelm
      REAL odd                  !dummy variable for Simpson's rule
      REAL por(NLAYERS)         !porosity of each soil layer
      REAL qmax                 !max drainage rate (mm d-1)
      REAL stand_water          !standing water
      REAL uw1_365, uw2_365, uw_ep_365 !store the last value of the year
                                       !and recall it on the first day
                                       !of the next year
      REAL value(30)
      REAL vtot                 !liquid water content of acrotelm [mm]
                                !imported is not bigger than the exported one
      REAL vtot_init            !initial water content of acrotelm [mm]
                                !to avoid negative vtot values
      REAL vtot_old             !yesterdays water + ice content in layer 1 [mm]
      REAL vtot_old2            !yesterdays water + ice content in layer 2 [mm]
      REAL wtd                  !water table depth = -wtp [mm]
      REAL wtot_lo              !Total water content in lower soil layer [mm]
      REAL wtot_up              !Total water content in upper soil layer [mm]
      REAL wtot_365(NLAYERS)
      REAL hcycle_in
      REAL hcycle_out
      REAL hcycle_store
      REAL test
      REAL test1
      REAL test2
      REAL test3
      REAL storage_change
      REAL input
      REAL output
      REAL corr_f               !correction factor for AET
      LOGICAL run_on
      REAL drunon               !daily run-on to the gridcell
      SAVE

c     ----------
c     Initialise
c     ----------

      aettotal(1) = 0.0
      aettotal(2) = 0.0
      demand_tot = 0.0
      supply_tot = 0.0      
      perc = 0.0
      intercp_tot(d) = 0.0
      drunon = 0.0
      drunoff_drain(d) = 0.0
      drunoff_surf(d) = 0.0
      daet(d) = 0.0
      q = 0.0
      cover = 0.0 
      fpc_moss_total = 0.0
      ice1(d) = 0.0
      ice2(d) = 0.0
      Dz_sub = 0.0
      storage_change = 0.0
      wtot_up = 0.0
      wtot_lo = 0.0
      uw_ep(d) = 0.0

      IF (year .EQ. 1 .AND. d .EQ. 1) THEN

         input = 0
         output = 0
         uw1_365 = 0.0
         uw2_365 = 0.0
         w_ep = w(1)
         uw_ep_365 = 0.0
         wtd = 0.0
         vtot = 0.0
         vtot_old = 0.0
         vtot_old2 = 0.0
         stand_water = 0.0
         hcycle_in = 0.0
         hcycle_out = 0.0
         hcycle_store = 0.0
         DO i = 1, NLAYERS
            awhc(i) = 0.0
            Ftotal(i) = 0.0
            Fice_365(i) = 0.0
            Fwater_365(i) = 0.0
            wtot_365(i) = 0.0
            Fice(d,i) = 0.0
         ENDDO
         DO i = 1, 30
            sub_water(i) = 0.0
         ENDDO
      ENDIF


      DO pft=1,npft
         intercep(pft)=0.0
         int_store(pft)=0.0
      ENDDO

      
c     We assume that fractional water holding capacity exhibits only minor
c     differences between lower and upper bucket

c     --------------
c     START PFT LOOP
c     --------------

      do pft=1,npft

c     ------------------------------------------------------------------
c     CALCULATE ACTUAL CANOPY CONDUCTANCE, POTENTIAL WATER SCALAR,
c     INTERCEPTION LOSS AND ACTUAL EVAPOTRANSPIRATION (AET) FOR EACH PFT
c     ------------------------------------------------------------------
         if (present(pft)) then
            
c     Calculate effective supply function in root zone today
c     f2=fraction of active fraction of roots uptaking water 
c     from lower soil layer
c     f1=the remainder in the top layer


c     INTRODUCE the calculation of the available water here
c     *****************************************************
            f2=rootprop(2,pft)
            f1=rootprop(1,pft)

            wr=w(1)*f1+w(2)*f2

c     ------------------------------------------------------------------
c     LPJ-WHy
c     Water availability for mosses - use the top 3 sub layers (=3*10mm)
c     and calculate the mean water content.
c     ------------------------------------------------------------------

            IF (moss(pft)) THEN
               wr = (sub_water(1) + sub_water(2) + sub_water(3)) / 3
            ENDIF

c     The calculation of 'supply' has changed from Gerten et al. (2004)
c     Eq. 6 to what was in the webversion (F77), but neither Sitch et
c     al. 2003 nor Gerten et al. 2004 mention it. LPJ-GUESS uses a
c     similar Eqn.  

c     R. Wania: Supply should be scaled by FPC, because Emax is in mm/d
c     which can be seen as L/m2/day. Emax is the maximum transpiration
c     per grid cell. If a PFT has an FPC of only 0.5, then the supply
c     rate will be only half.

            supply=emax(pft)*wr*dphen(d,pft)*fpc_grid(pft) 
           
c     Calculate PFT-specific INTERCEPTION LOSS (Kergoat 1998)
c     -------------------------------------------------------
            int_store(pft)=min(dprec(d),
     *           (intc(pft)*lai_ind(pft)*dphen(d,pft)*dprec(d)))
            if(dpet(d).lt.0.0001) then
               wet=0.0
            else
               wet=min(int_store(pft)/(dpet(d)*pt),0.99) 
            endif
            intercep(pft)=dpet(d)*pt*wet

c     Calculate AET demand function and potential demand 
c     assuming full leaf cover (Eqn 23, Haxeltine & Prentice 1996)
c     Note fraction of daylength
c     The demand is calculated differently by Gerten than by Sitch.
c     Sitch uses *(1-exp(-dgp/gm)),
c     whereas Gerten uses /(1+gm/dgp), which leads to a slower
c     increase in the potential demand. 
c     This is Gerten's version:
c               demand=(1.0-wet)*dpet(d)*alpham/
c     *              (1+gm/(dgp(d,pft)*dphen(d,pft)))

c     The problem with both, Gerten and Sitch's version, is that the
c     scaling of demand by fpc_grid is implicit in dgp. In this
c     equation, dgp influences demand in a non-linear way however. This
c     causes the following problem:
c     Assume we have 2 PFTs with the same physiological parameters.
c     We allocate some numbers to the variables in the eqn.
c     wet = 0.0
c     dpet(d) = 10
c     alpham = 1.391
c     gm = 3.26
c     non-fpc-scaled dgp = 5
c     dphen = 1
c
c     Situation 1:
c     PFT1: fpc_grid = 0.9
c     PFT2: fpc_grid = 0.1
c     demand(pft1) = (1-0)*10*1.391/(1+3.26/5*0.9*1) = 8.1
c     demand(pft2) = (1-0)*10*1.391/(1+3.26/5*0.1*1) = 1.8
c     total demand = 9.9

c     Situation 2:
c     PFT1: fpc_grid = 0.6
c     PFT2: fpc_grid = 0.4
c     demand(pft1) = (1-0)*10*1.391/(1+3.26/5*0.6*1) = 6.7
c     demand(pft2) = (1-0)*10*1.391/(1+3.26/5*0.4*1) = 5.3
c     total demand = 12.0
c     
c     The total demand differs because of a different balance of FPCs.
c     A suggestion to solve this by Rita Wania (19/12/2008)
c     1. Take out the scaling by fpc_grid from dgp
c     2. Scale the entire demand equation by fpc_grid.
c     ----------------------------------------------------------------

            if (dphen(d,pft).gt.0.0.and.dgp(d,pft).gt.0.0) then
               demand=(1.0-wet)*dpet(d)*alpham/ (1+gm/(dgp(d,pft)
     $              /fpc_grid(pft))) * fpc_grid(pft) * dphen(d,pft)

           else
               demand=0.0
            endif
            if (dphen(d,pft).eq.1.0) then
               demandpot=demand
            elseif (dgp(d,pft).gt.0.0) then 
               demandpot=dpet(d)*alpham/(1+gm/(dgp(d,pft)
     $              /fpc_grid(pft)))* fpc_grid(pft) * dphen(d,pft)

            else
               demandpot = 0.0
            endif


c     Calculate daily potential water scalar dwscal is what Sitch et
c     al. 2003 calls water stress factor (omega) dwscal decides over
c     leaf 'on' or 'off' for those PFTs which have a minimum wscal set
c     (at the moment, trees don't have a minimum value)
c     ------------------------------------------------------------------

            if (dphen(d,pft) .NE. 0.0 .AND. demandpot .NE. 0.0) then
               dwscal(pft)=min((supply/dphen(d,pft))/demandpot,1.0)
            else
               dwscal(pft)=1.0
            endif     

c     Calculate actual canopy conductance (gc), according to balance
c     between supply and demand Eqn 25, Haxeltine & Prentice 1996
c     R. Wania: Note that aet can exceed dpet because dpet is
c     equilibrium evaporation, not potential evaporation!
c     -------------------------------------------------------------

            if(supply.ge.demand) then
               dgc(d,pft)=dgp(d,pft)*dphen(d,pft) 
            else 
               if(dpet(d).gt.0.0) then
                  dgc(d,pft)=max(0.0,(
     *                 gm*supply/((1.0-wet)*dpet(d)*alpham
     *                 -supply)))
               else
                  dgc(d,pft)=0.0
               endif
            endif
            aet(pft)=min(supply,demand)  


c     LPJ-WHy: No transpiration from mosses
c     --------------------------------------
            IF (moss(pft)) aet(pft) = 0.0

c     Accumulate total AET and interception loss
c     -------------------------------------------
            if (wr.eq.0.0) then
               beta(1)=0.0
               beta(2)=0.0
            else
               beta(1)=f1*w(1)/wr
               beta(2)=f2*w(2)/wr
            endif
            
            aettotal(1)=aettotal(1)+beta(1)*aet(pft)
            aettotal(2)=aettotal(2)+beta(2)*aet(pft)
            intercp_tot(d)=intercp_tot(d)+intercep(pft)*fpc_grid(pft)



c     Calculate area covered by PFTs
c     --------------------------------
            cover=cover+(fpc_grid(pft)*dphen(d,pft))
         endif
      enddo                     !pft

c     ---------------
c     END OF PFT LOOP
c     ---------------

c     Compute total transpiration and effective rainfall
c     ---------------------------------------------------

      daet(d)=aettotal(1)+aettotal(2)
      dprec(d)=dprec(d)-intercp_tot(d)

c     LAYER SETUP
c     IDX = top of soil layers
c     MIDX = a mixed layer on top of soil layer that consists of litter
c            air and in peatlands also of water.
c     ------------------------------------------------------------------
      IDX = NLAYERS - NSOILL + 1
      MIDX = IDX - 1 

c     ------------------------------------------------------------------
c     Allocate the ice content according to the changes from the
c     Soiltemp SR, 
c     ------------------------------------------------------------------
      

      DO i = IDX, NLAYERS
         IF (d .eq. 1) THEN
            Fice(1,i) = Fice_365(i)
            wtot(1,i) = Fwater_365(i) * Dz(i)
         ELSE
            Fice(d,i) = Fice(d-1,i)
            wtot(d,i) = Fwater(d-1,i) * Dz(i)
         ENDIF
         whc(i) = soilpar(3)
      ENDDO


c     Set fractions of soil parameters for each soil layer
c     ----------------------------------------------------
      j = 0
      DO i = IDX, NLAYERS
         j = j + 1
         Dz(i) = Dz_soil(j)
      ENDDO


c=======================================================================
c     HYDROLOGY SEPARATION ---starts here---
c     Mineral soils:
c=======================================================================
      IF (.not. peathydr) THEN

c     -------------------------------------------------------------
c     Calculate SOIL WATER components 
      !Rita: changed w_ep to w_ep**2 according to C++ version and
      !Dieter's advice.
c     -------------------------------------------------------------
         prec(d) = dprec(d) + dmelt(d)

         daep(d) = dpet(d) * pt * w_ep ** 2 * (1.0 - cover)

c     ------------------------------------------------------------------
c     NOTE: WHC means now total water holding capacity including water
c     BELOW the permanent wilting point. Each layer has it's own WHC and
c     AWHC (actual WHC which considers ice content). The original
c     meaning of WHC, namely the fraction between Field capacity and PWP
c     can be derived by WHC - Fpwp.
c     ------------------------------------------------------------------

c     ---------------------------
c     Reduce WHC by ice fraction 
c     ---------------------------

         DO i = IDX, NLAYERS
            awhc(i) = whc(i) - Fice(d,i)
            IF (awhc(i) .LT. 0.0001) awhc(i) = 0.0
         ENDDO
         

c     Think about the amount of water which can be stored when ice is
c     in the soil? is the WHC really reduced by ice?
c     The way drunoff_surf is calculated appears to be a bit weird as
c     it depends on the whc. Shouldn't it depend on the perc rate? i.e.
c     if the perc rate is too low and not all of the water which is
c     in excess of the whc can be transported downwards, it will run off.
c     Dieter explained that this is because of the lack of porosity values
c     so far. If one used porosity, then you can calculate percolation in
c     a slightly 'better' way. (That depends on the quality of the porosity
c     values though!)


c     -------------------------------------------------------------------
c     LPJ-WHy EVAPORATION SETUP Use the top 2 layers, which cover the
c     top 20cm as the new evaporation layer. Subtract evaporation daep
c     only from those 2 layers
c     -------------------------------------------------------------------
         
         depth = 0.0

         DO i = IDX, NLAYERS - nlayer1
            frac = Dz(i) / d1
            depth = depth + Dz(i)
            IF (year .EQ. 1 .AND. d .EQ. 1) THEN
               wtot(d,i) = uw1(d) * frac
            ENDIF
            ! Subtract evaporation only from the top 2 soil layers
            ! Evapotransporation is slightly higher in the top layers,
            ! i.e. evaporation layers, than in the layers below (Gerten
            ! et al. 2004, Sect. 2.2.3.)
            wtot(d,i) = wtot(d,i) + prec(d) * frac - aettotal(1) * frac
     $           - daep(d) * frac
         ENDDO
         
         
         ! Calculate excess water for each layer
         ! -------------------------------------
         DO i = IDX, NLAYERS - nlayer1
            IF (wtot(d,i) .GT. awhc(i) * Dz(i)) THEN
               drunoff_surf(d) = drunoff_surf(d) + (wtot(d,i) - awhc(i)
     $              *Dz(i))
               wtot(d,i) = awhc(i) * Dz(i)
            ENDIF

            IF (wtot(d,i) .LT. 0.0) wtot(d,i) = 0.0
            wtot_up = wtot_up + wtot(d,i) 
         ENDDO
         theta(d) = wtot_up / d1

c     ------------------------------------------------------------------
c     Here, use whc to calculate how much water is available for plants
c     To calculate how much water is available for plants, use the water
c     content uw1 and the whc and Fpwp from one of the layers in the 
c     upper soil layer, in this case the first soil layer IDX is used.
c     Since whc and Fpwp are the same for all sublayers within the top
c     layer, it's okay to use the values of any layer. 
c     ------------------------------------------------------------------

         uw1(d) = MAX(0.0, wtot_up - Fpwp(IDX) * d1)
         w(1) = uw1(d)  / ((whc(IDX) - Fpwp(IDX)) * d1) 
         prec(d) = prec(d) - drunoff_surf(d)! - aettotal(1) - daep(d)


c     -----------------
c     EVAPORATION LAYER
c     -----------------
         depth = 0
         DO i = IDX, NLAYERS - nlayer1
            depth = depth + Dz(i)
            IF (depth .LE. 200) THEN
               uw_ep(d) = uw_ep(d) + wtot(d,i) - Fpwp(IDX) * Dz(i)
            ENDIF
         ENDDO

         IF (uw_ep(d) .LE. 0.0001) uw_ep(d) = 0.0
         w_ep = uw_ep(d) / ((whc(IDX) - Fpwp(IDX)) * d_evap)

         
         IF (w_ep .GE. 1.0) THEN
            excess = (w_ep - 1.0) * ((awhc(IDX) + awhc(IDX + 1)) / 2) *
     $           d_evap
            uw_ep(d) = uw_ep(d) - excess
            w_ep = 1.0
         ENDIF

         ! Gerten's version
         ! ----------------

c$$$         if(prec(d).ge.0.1) then
c$$$            perc=k(1)*w(1)**k(2)
c$$$            if(perc.gt.prec(d)) perc=prec(d)
c$$$            perc_evap=k(1)*2/5*w_ep**k(2)
c$$$            if(perc_evap.gt.prec(d)) perc_evap=prec(d)
c$$$            q=0.5*k(1)*w(2)**k(2)
c$$$            if(q.gt.prec(d)) 
c$$$     *           q=prec(d)
c$$$         else 
c$$$            perc=0.0
c$$$            perc_evap=0.0
c$$$         endif
         
         ! LPJ-WHy version
         ! ---------------

         IF(drunoff_surf(d) .GT. 0.1) THEN
            perc = k(1) * w(1) ** k(2)
            IF(perc .GT. drunoff_surf(d)) perc = drunoff_surf(d)
            drunoff_surf(d) = drunoff_surf(d) - perc
c           The 2/5 factor comes from the fact that evaporation happens
c           only from the top 20cm of the upper layer, which is 50cm
c           deep.
            perc_evap = k(1) * 2.0 / 5.0 * w_ep ** k(2) 
c            IF(perc_evap .GT. prec(d)) perc_evap = prec(d)
            ! If drainage should happen as soon as there is precip,
            ! uncomment these lines. At the moment, drainage from the
            ! lower layer happens only when field capacity is exceeded.
c            q = 0.5 * k(1) * w(2) ** k(2)
c            IF (q .GT. prec(d)) 
c     *           q = prec(d)
         ELSE 
            perc = 0.0
c            perc_evap = 0.0
         ENDIF

         uw1(d)=uw1(d)-perc
         IF (uw1(d) .LE. 0.0) uw1(d) = 0.0
c         uw_ep(d) = uw_ep(d) - perc_evap
         
c         w_ep = uw_ep(d) / ((whc(IDX) - Fpwp(IDX)) * d_evap)
         w(1) = uw1(d) / ((whc(IDX) - Fpwp(IDX)) * d1)

c$$$c     -----------------
c$$$c     EVAPORATION LAYER
c$$$c     -----------------
c$$$         depth = 0
c$$$         DO i = IDX, NLAYERS - nlayer1
c$$$            depth = depth + Dz(i)
c$$$            IF (depth .LE. 200) THEN
c$$$               uw_ep(d) = uw_ep(d) + wtot(d,i) - Fpwp(IDX) * Dz(i)
c$$$            ENDIF
c$$$         ENDDO
c$$$
c$$$         IF (uw_ep(d) .LE. 0.0001) uw_ep(d) = 0.0
c$$$         w_ep = uw_ep(d) / ((whc(IDX) - Fpwp(IDX)) * d_evap)
c$$$
c$$$         
c$$$         IF (w_ep .GE. 1.0) THEN
c$$$            excess = (w_ep - 1.0) * awhc(IDX) * d_evap
c$$$            uw_ep(d) = uw_ep(d) - excess
c$$$            w_ep = 1.0
c$$$         ENDIF
c$$$
c$$$         IF(prec(d) .GE. 0.1) THEN
c$$$            perc = k(1) * w(1) ** k(2)
c$$$            IF(perc .GT. prec(d)) perc = prec(d)
c$$$            perc_evap = k(1) * 2.0 / 5.0 * w_ep ** k(2) 
c$$$            IF(perc_evap .GT. prec(d)) perc_evap = prec(d)
c$$$            ! If drainage should happen as soon as there is precip,
c$$$            ! uncomment these lines. At the moment, drainage from the
c$$$            ! lower layer happens only when field capacity is exceeded.
c$$$c            q = 0.5 * k(1) * w(2) ** k(2)
c$$$c            IF (q .GT. prec(d)) 
c$$$c     *           q = prec(d)
c$$$         ELSE 
c$$$            perc = 0.0
c$$$            perc_evap = 0.0
c$$$         ENDIF
c$$$
c$$$         uw1(d)=uw1(d)-perc
c$$$         IF (uw1(d) .LE. 0.0) uw1(d) = 0.0
c$$$         uw_ep(d) = uw_ep(d) - perc_evap
c$$$         
c$$$         w_ep = uw_ep(d) / ((whc(IDX) - Fpwp(IDX)) * d_evap)
c$$$         w(1) = uw1(d) / ((whc(IDX) - Fpwp(IDX)) * d1)
c$$$


c     --------------------------------------------------------------------
c     BOTTOM LAYER

c     Note: When there is no vegetation because it is too dry, then no
c     water gets subtracted from the bottom layer as there is no
c     transpiration, but water can get to the bottom layer through
c     occasional rain events will lead to percolation and increase the
c     water content in the bottom layer. Hence, when looking at a global
c     picture of soil moisture, keep in mind that mw2 won't be accurate
c     in dry areas.

c     I'll try to fix this by allowing drunoff_drain on days when rain
c     occurs, like Gerten did.
c     --------------------------------------------------------------------
         DO i = IDX + nlayer1, NLAYERS
            frac = Dz(i) / d2

            !Initialise wtot
            IF (year .EQ. 1 .AND. d .EQ. 1) THEN
               wtot(d,i) = uw2(d) * frac 
            ENDIF

            wtot(d,i) = wtot(d,i) + perc * frac - aettotal(2) * frac - q
     $           * frac

            ! ----------------------------------
            ! Calculate RUNOFF from lower layer 
            ! ----------------------------------     

            IF (wtot(d,i) .GT. awhc(i) * Dz(i)) THEN
               drunoff_drain(d) = drunoff_drain(d) + (wtot(d,i) -
     $              awhc(i)* Dz(i))
               wtot(d,i) = awhc(i) * Dz(i) 
            ELSE
               drunoff_drain(d) = drunoff_drain(d) + q * frac
            ENDIF
            IF (wtot(d,i) .LE. 0.0) wtot(d,i) = 0.0         
            wtot_lo = wtot_lo + wtot(d,i)
         ENDDO


c     ------------------------------------------------------------------
c     Here, use WHC to calculate how much water is available for plants
c     wtot_lo is the water content from 500 to 2000mm. uw2 considers
c     only water between 500 and 1500mm; therefore wtot(d,NLAYERS) which
c     is 500mm thick is subtracted.  Further, the water below the
c     permanent wilting point is subtracted as well.
c     ------------------------------------------------------------------
c$$$         uw2(d) = MAX(0.0, wtot_lo - wtot(d, NLAYERS) - Fpwp(IDX + nlayer1
c$$$     $        + 1) * (d2 -Dz(NLAYERS)))

c     ALTERNATIVE WAY that gives the same result: Subtract the volume of
c     water held below the wilting point = Fpwp * d2 from the total
c     water in the lower layer. Use this to calculate w(2)


         uw2(d) = MAX(0.0, wtot_lo - Fpwp(IDX + nlayer1 + 1) * d2)

c     ------------------------------------------------------------------
c     The fractional water content available for plants refers to the
c     WHC minus the fraction below the PWP and hence Fpwp is subtracted.
c     Multiplication by 1000 because the traditional thickness of the
c     bottom layer is 1000mm. 
c     ------------------------------------------------------------------
c$$$         w(2) = uw2(d) / ((whc(IDX + nlayer1 + 1) - Fpwp(IDX + nlayer1 +
c$$$     $        1)) * 1000) 

c     ALTERNATIVE WAY: use entire lower layer, i.e. 500-2000mm depth to
c     calculate fractional water content.  This way gives the same
c     results as the way above, where only the water content of
c     500-1500mm is considered.

         w(2) = uw2(d) / ((whc(IDX + nlayer1 + 1) - Fpwp(IDX + nlayer1 +
     $        1)) * d2) 

         DO i = IDX, NLAYERS
            Fwater(d,i) = wtot(d,i) / Dz(i)
         ENDDO
         wtp(d) = MISSING_VALUE


      ELSE                      !soilcode = 8, peathydr = TRUE

c=======================================================================
c     WETLAND HYDROLOGY ---starts here---
c=======================================================================

         DO pft = 1, npft 
            IF (present(pft)) THEN
               IF (moss(pft)) THEN
                  fpc_moss_total = fpc_moss_total + fpc_grid(pft)
               ENDIF
            ENDIF
         ENDDO 

         prec(d) = dprec(d) + dmelt(d)

c     --------------------------------------------------------------------
c     EVAPOTRANSPIRATION

c     Calculate transpiration and evaporation from wetlands together.
c     dpet is the equilibrium ET but is the same as the potential ET in
c     Kim and Verma (1996).  (BAD choice of variable names in original
c     LPJ!)  Hence, we can use dpet as approximation for potential
c     evapotranspiration and scale it by the water table depth.  daept
c     is allowed to happen even if the wtp is at -300mm because plants
c     will be able to extract water from the unsaturated acrotelm or the
c     saturated catotelm.  This however makes it impossible to close the
c     hydrological cycle in LPJ-WHy. 
c     Kim and Verma (1996): ET = PET * (1.02 + 0.00075 * WTP)
c     --------------------------------------------------------------------

         IF (snowdepth(d) .LT. 25) THEN
            daept(d) = dpet(d) * (1.02 + 0.00075 * (-wtd))
         ELSE
            daept(d) = 0.0 ! sublimation rates needed
         ENDIF
c     ------------------------------------------------------------------
c     Set POROSITY for each layer. Subtract a constant gas fraction from
c     the total porosity.  This is done because peat contains a lot of
c     gas in form of CO2 and methane.  The gas influences the thermal
c     properties and therefore was included in the model.  The porosity
c     of the acrotolm is set to be higher than for the catotelm.
c     ------------------------------------------------------------------
         acro_por = acrotelm_por - Fgas 

         DO i = IDX, NLAYERS - NCATOTELM
            por(i) = acrotelm_por - Fgas 
         ENDDO
         DO i = IDX + NACROTELM, NLAYERS
            por(i) = soilpar(8) - Fgas  
         ENDDO
         por(MIDX) = 1.0


c-------------------------------------
c     GRANBERG et al.'s 1999 approach 
c-------------------------------------

c     ----------------------------------------------------------------
c     GRADIENT, with which water content decreases with WTP
c     Granberg used a gradient for the upper 100mm of the peat
c     which decreased the water content at the surface too slowly. The
c     water content decreases rapidly when WT drops from 0 to 50mm and
c     then decreases more slowly. I got the water content of 0.34 at
c     50mm depth from Figure 7 (Hayward and Clymo 1982), taking an
c     average of both S. species. Then I separated the entire gradient
c     az into 2 parts, one for the top 50mm = az1, which covers a water
c     saturation from 0.98 (por) to 0.34 (surfw) and the bottom 50mm =
c     az2, which covers a water saturation from 0.34 (surfw) to 0.25
c     (minvtot). (This is not implemented at the moment.)
c     ----------------------------------------------------------------
c$$$         az1 = (acro_por - surfw) / grad_lim
c$$$         az2 = (surfw - minvtot) / (zmin - grad_lim)

c     -----------------------------------------------------------------
c     GRADIENT along which water content in unsaturated layer decreases
c     -----------------------------------------------------------------
         az = (acro_por - minvtot) / zmin

c     --------------
c     DRAINAGE rate
c     --------------

c     ------------------------------------------------------------------
c     Ingram #000219 p.125: gives permeability values for peat, which
c     not necessarily reflect seepage though. The permeability depends
c     on the water potential and ranges from 5.5mm d-1 to 10.4mm d-1.
c     More generally spoken, it is hard to test catotelm drainage in
c     realistic field experiments and lab experiments are said not to be
c     suitable.
c     ------------------------------------------------------------------
         IF (Fice(d, 7) .EQ. 0) THEN
            q = 0.0 ! 0.5
         ELSE IF (Fice(d,7) .LT. 0.5) THEN
            q = 0.0 !0.1 arbitrary number
         ELSE
            q = 0.0
         ENDIF
         IF (wtd .GT. z1 * acro_por - q) q = 0.0

         drunoff_drain(d) = q


c     -------------------------
c     Calculate AVAILABLE WATER
c     -------------------------
         
         vtot = vtot + prec(d) - daept(d) - q

         IF (vtot .LT. 0.0) vtot = 0.0

        !----------------------------------------------------------------
        ! To calculate the water content in each layer, add the ice
        ! volume to the water volume, calculate the hypothetical WTP and
        ! then substract the ice from each layer to get the water
        ! content.
        !----------------------------------------------------------------


         !-------------------------------------------------------------
         ! Drunoff from surface: this eqn is made up. It reflects the
         ! very rapid runoff when the WTP is high and the decrease when
         ! the WT drops
         !-------------------------------------------------------------

         IF (wtd .LT. 150) THEN
            drunoff_surf(d) = exp(-wtd * 0.01)
         ELSE
            drunoff_surf(d) = 0.0
         ENDIF

         IF  (Fice(d,4) .GT. 0.05 .OR. snowdepth(d) .GT. 10)
     $        drunoff_surf(d) = 0.0

c$$$         IF (wtd .LT. 0) THEN 
c$$$            drunoff_surf(d) = 2.5 !arbitrary number so far
c$$$         ELSE IF (wtd .GE. 0 .AND. wtd .LT. 50) THEN
c$$$            drunoff_surf(d) = 0.5 !arbitrary number so far
c$$$         ELSE IF (wtd .GE. 50 .AND. wtd .LT. 150) THEN
c$$$            drunoff_surf(d) = 0.05 !arbitrary number so far
c$$$         ELSE
c$$$            drunoff_surf(d) = 0.0 !arbitrary number so far
c$$$         ENDIF

         !-----------------------------------------------------------
         ! For sites like the one at Stordalen, North Sweden, it is
         ! necessary to add some contant water influx. For the global
         ! run, switch run_on off.
         !-----------------------------------------------------------
         IF (year .lt. 1000) THEN
            run_on = .FALSE.
         ELSE
            run_on = .FALSE.
         ENDIF

         IF (run_on) drunon = 0.5

         vtot_init = vtot
         vtot = vtot + stand_water + drunon - drunoff_surf(d)
         IF (vtot .LT. 0) THEN
            vtot = 0
            drunoff_surf(d) = vtot_init
         ENDIF
        !----------------------------------------------------------------
        ! If total water volume is greater than the porosity then the
        ! WTP is above the surface. The maximum height of standing water
        ! is limited by maxh. If the watertable is above that height, it
        ! will run off.  Snow melt season: 1. If there is a standing
        ! water layer, which is obviously frozen in winter, then the
        ! snow melt will run off and only when all the ice of the stand
        ! layer is thawed, the water will be added to the soil
        ! layers. 2. If there is no standing water layer, then the
        ! snowmelt water...should penetrate into the soil.
        !----------------------------------------------------------------
         IF (vtot .GT. z1 * acro_por) THEN 
            wtp(d) = vtot - z1 * acro_por  !the litter fraction has not
                                           !been considered when
                                           !calculating the wtp above
                                           !the surface!!
            IF (wtp(d) .GT. maxh) THEN
               drunoff_surf(d) = drunoff_surf(d) + wtp(d) - maxh
               wtp(d) = maxh
            ENDIF
            wtd = - wtp(d)
            vtot = z1 * acro_por 
            stand_water = wtp(d)
            Fwater(d,MIDX) = wtp(d) / maxh


         !When there is standing water, sort into 2 classes, i.e. 50mm
         !and 100mm. If the wtp is only a few mm above the surface, it
         !might lead to instabilities in the temperature SR.
         !-------------------------------------------------------------
         ! Calculate the wtd by using Granberg's equations. There are 2
         ! equations, one for wtd .LE. zmin, and the other for wtd
         ! .GT. zmin.  NOTE: At the moment, water + ice is used to
         ! calculate the WTD and afterwards, water and ice are separated
         ! again.
         !-------------------------------------------------------------
         ELSE
            wtd = SQRT (3 * (acro_por * z1 - vtot) / (2 * az))
            IF (wtd .GT. zmin) THEN !this happens IF vtot < 245.33 mm
                                    !if acro_por = 0.98
               wtd = 3 * (acro_por * z1 - vtot) /
     $              (2 * (acro_por - minvtot))
            ENDIF   
            IF (wtd .GT. z1) wtd = z1 !this happens if vtot < 148 mm
                                      !if acro_por = 0.98
            wtp(d) = -wtd
            stand_water = 0
            Fwater(d,MIDX) = 0.0
            Fice(d,MIDX) = 0.0
         ENDIF   

         ! Calculate the water content of the evaporation layer.  w_ep
         ! and uw_ep are actually not used when the peatland hydrology
         ! is on.
         w_ep = 0.0
         DO i = 1, 20
            w_ep = w_ep + sub_water(i)
         ENDDO
         w_ep = w_ep / 20
         uw_ep(d) = w_ep *  d_evap


c     -----------------------------------------------------------------
c     WATER CONTENT in each layer: Use Simpson's rule to integrate over
c     the acrotelm. You integrate between point a (=surfw_sat) and b
c     (=por). The integral is divided into N (=NSUBLAYERS) slices.
c     -----------------------------------------------------------------

         ! Calculate the thickness of each NSUBLAYER
         ! -----------------------------------------
         
         Dz_sub = NACROTELM * Dz_acro / NSUBLAYERS

         IF (wtd .GT. 0) THEN
            surfw_sat = MAX(minvtot, acro_por - az * wtd )
            odd = 0.0
            even = 0.0

           !Odd-numbered layers
           !--------------------
            count = 1
            DO j = 1, NSUBLAYERS / 2 
               value(count) =
     $              min(acro_por, surfw_sat + (acro_por - surfw_sat) *
     $              ((2 * j - 1) * dz_sub / wtd) ** 2)
               odd = odd + value(count)
               count = count + 2
            ENDDO

            !Even-numbered layers 
            !--------------------
            count = 2
            DO j = 1, NSUBLAYERS / 2 - 1
               value(count) =
     $              min(acro_por, surfw_sat + (acro_por - surfw_sat) *
     $              (2 * j * dz_sub / wtd) ** 2) 
               even = even + value(count)
               count = count + 2
            ENDDO

            !Last layer
            !----------
            value(30) = acro_por

            ! Average values to get the water content in each sub_water
            ! layer.
            !---------------------------------------------------------
            sub_water(1) = (surfw_sat + value(1)) / 2
            DO j = 2, NSUBLAYERS
               sub_water(j) = (value(j - 1) + value(j)) / 2
            ENDDO


c     This way diverges from the WIKIPEDIA one. It calculates the middle
c     points of each layer and uses that mean value to calculate the
c     water content.

            DO i = IDX, NLAYERS - NCATOTELM
               Ftotal(i) = 0.0
            ENDDO

!----------------------------------------------------------------------
! Calculate water content in each acrotelm layer
! This is written in a bit bumpy way, any improvements?
! The steps: e.g. if there are 3 acrotelm layers and 30 sublayers, then
! you add sub_water content 1-10 to Ftotal(1), sub_water content 11-20
! to Fwater(2) and sub_water content 21-30 to Fwater(3).
!----------------------------------------------------------------------
            jj = 1
            count = 1
            DO i = IDX, NLAYERS - NCATOTELM
               DO j = count, NSUBLAYERS / NACROTELM * jj 
                  ! Division by 10 comes from having 10 sublayers
                  Ftotal(i) = Ftotal(i) + sub_water(j) / 10
                   ENDDO
               count = count + 10
               jj = jj + 1
            ENDDO
c         ENDIF
!---------------------------------------------------------------------
!Subtract ice content and water content below the permanent wilting
!point
!---------------------------------------------------------------------
!--------
!ACROTELM
!--------

         uw1(d) = 0.0
         DO i = IDX, NLAYERS - NCATOTELM
            Fwater(d,i) = MAX(0.0, Ftotal(i) - Fice(d,i))
            uw1(d) = uw1(d) + (Fwater(d,i) - Fpwp(i)) * Dz(i)
         ENDDO
         w(1) = MIN(1.0, uw1(d) / ((por(IDX) - Fpwp(IDX)) * z1)) 

!--------
!CATOTELM
!--------
         uw2(d) = 0.0
         DO i = IDX + NACROTELM, NLAYERS
            Fwater(d,i) = MAX(0.0, por(i) - Fice(d,i))
            uw2(d) = uw2(d) + (Fwater(d,i) - Fpwp(i)) * Dz(i)
         ENDDO
         
         w(2) = MIN(1.0, uw2(d) / ((por(IDX + NACROTELM) - Fpwp(IDX +
     $        NACROTELM)) * z2))
!---------------------
!Standing water layer
!---------------------

         ELSE !standing water
            uw1(d) = 0.0
            uw2(d) = 0.0
            ! --------
            ! ACROTELM
            ! --------
            DO i = IDX, NLAYERS - NCATOTELM
               Fwater(d,i) = MAX(0.0, por(i) - Fice(d,i))
               uw1(d) = uw1(d) + (Fwater(d,i) - Fpwp(i)) * Dz(i)
            ENDDO
            ! --------
            ! CATOTELM
            ! --------
            DO i = IDX + NACROTELM, NLAYERS
               Fwater(d,i) = MAX(0.0, por(i) - Fice(d,i))
               uw2(d) = uw2(d) + (Fwater(d,i) - Fpwp(i)) * Dz(i)
            ENDDO
            ! -----------
            ! MIXED LAYER
            ! -----------
            !Calculate the water fraction of the standing water
            Fwater(d,MIDX) = MAX(0.0, Fwater(d,MIDX) - Fice(d,MIDX))
         ENDIF
         
         w(1) = MIN(1.0, uw1(d) / ((por(IDX) - Fpwp(IDX)) * z1)) 
         w(2) = MIN(1.0, uw2(d) / ((por(IDX + nlayer1 + 1) - Fpwp(IDX +
     $        nlayer1+ 1)) *z2))

         DO i = MIDX, NLAYERS
            IF (Fwater(d,i) .LT. 0.0001) Fwater(d,i) = 0.0
         ENDDO

c     -------------------------------------------------------------------
c     MOSS PHOTOSYNTHESIS

c     It is kept at 100% while WTP is above -150mm, decreases linearly
c     between -150 and -280mm and is held at 30% when the WTP drops
c     below -280mm. (find citations). The current water table setup does
c     not lead to drying out of mosses at the surfaces as a distance of
c     30cm can be overcome by capillary forces and draw water from -30cm
c     to the surface.
c     ------------------------------------------------------------------

c$$$         IF (wtp(d) .GT. -150) THEN
c$$$            dmoss_cap(d) = 1
c$$$         ELSE IF (wtp(d) .GT. -280) THEN
c$$$            dmoss_cap(d) = 1- (wtd - 150) * ((1 - 0.3) / (-150 + 280))
c$$$         ELSE
c$$$            dmoss_cap(d) = 0.3
c$$$         ENDIF

         IF (wtp(d) .GT. -200) THEN
            dmoss_cap(d) = 1
         ELSE IF (wtp(d) .GT. -290) THEN
            dmoss_cap(d) = 1- (wtd - 200) * ((1 - 0.6) / (-200 + 290))
         ELSE
            dmoss_cap(d) = 0.6
         ENDIF

c     --------------------------------------------------------------------
c     INUNDATION STRESS

c     I've added inundation stress (istress) here, which will reduce GPP
c     by a factor calculated from the WTP, the rooting depth of each PFT
c     and the stress tolerance level (pftpar 37).  PFT parameter 37
c     gives the maximum WTP, which can be tolerated by the PFT.  PFT
c     param 42 gives the number of days the inundation can be tolerated.

c     To calculate the fraction of inundated lower soil layer, you need to
c     add the WTP (which is negative) to the soil depth and add the upper
c     soillayer depth as well.
c     --------------------------------------------------------------------

         DO pft = 1, npft
            IF (present(pft)) THEN
               IF (wtp(d) .GT. inund_height(pft)) THEN
                  icount(m,pft) = icount(m,pft) + 1
               ENDIF
            ENDIF
         ENDDO                  !pft

      ENDIF                     !end soilcode = 8, peathydrology

      IF (d .EQ. 365) THEN
         uw1_365 = uw1(d)
         uw2_365 = uw2(d)
         uw_ep_365 = uw_ep(d)
      ENDIF

      RETURN
      END SUBROUTINE WATERBALANCE


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE SOILTEMP
c     COMPLETELY NEW SOIL TEMPERATURE calculation based on an implicit
c     scheme = CRANK-NICHOLSON to solve the diffusion equation. The general
c     code structure was written by Ian Ross (Nov 2005) and then adapted for
c     LPJ by Rita Wania (Dec 2005).

      subroutine soiltemp(d,year,pftpar,soilpar,soilcode,peathydr,dtemp
     $     ,snowdepth,thawdepth,Fwater,Fice,permafrost_index,thaw_top
     $     ,Tsoil_0,Tsoil_5,Tsoil_10,Tsoil_15,Tsoil_20,Tsoil_25,Tsoil_30
     $     ,Tsoil_40,Tsoil_50,Tsoil_60,Tsoil_80,Tsoil_100,Tsoil_120
     $     ,Tsoil_150,Tsoil_200,Tsurf,dmelt,snowdens,Fpwp,litter_ag
     $     ,Tsoil,Fair,Fwater_365,Fice_365,frozen_day)

      IMPLICIT NONE

      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"

      INTEGER PAD_LAYERS        !number of padding layers
      REAL PAD_DEPTH            !padding depth [mm]
      PARAMETER (PAD_LAYERS = 4)
      PARAMETER (PAD_DEPTH = 8000) 

      REAL Dt                   !time step [day]
      PARAMETER (Dt = 0.5)

      REAL Dz_litter            !depth of each litter layer [mm]
      PARAMETER (Dz_litter = 100.0)

c     ________________________________________________________________________

      REAL Cair                 !heat capacity of air [J m-3 K-1]
      REAL Cwater               !heat capacity of water [J m-3 K-1]
      REAL Cice                 !heat capacity of ice [J m-3 K-1]
      REAL Corg                 !heat capacity of organic matter [J m-3 K-1]
      REAL Cmin                 !heat capacity of mineral soil [J m-3 K-1]
      REAL Cpeat                !heat capacity of dry peat (J m-3 K-1)

      PARAMETER (Cair = 1200)   !Bonan Ecol. Clim., p. 188
      PARAMETER (Cwater = 4180000) !Bonan Ecol. Clim., p. 188
c     PARAMETER (Cice = 1700000) !calculated from James 1968
      PARAMETER (Cice = 3117800) !from Bartlett et al. 2004
      PARAMETER (Corg = 2500000) !Bonan Ecol. Clim., p. 188
      PARAMETER (Cmin = 2380000) !Bonan Ecol. Clim., p. 188
      PARAMETER (Cpeat = 580000) !Bonan Ecol. Clim., p. 188, 0% water
c      PARAMETER (Cpeat = 2500000) !Zhuang et al. 2001; max value for
c      thawed upper organic layer
c      PARAMETER (Cpeat = 1900000) !Zhuang et al. 2001; mean value for
c      thawed upper organic layer

c     NOTE: using the Corg values for Cpeat and Korg for Kpeat does not
c     seem to influence the upper soil layer Ts, but increases the range
c     in the lower layers (2m)

      REAL Kair            !thermal conductivity of air [W m-1 K-1]
      REAL Kwater          !thermal conductivity of water [W m-1 K-1]
      REAL Kice            !thermal conductivity of ice [W m-1 K-1]
      REAL Korg            !thermal conductivity of organic matter [W m-1 K-1]
      REAL Kmin            !thermal conductivity of mineral soil [W m-1 K-1]
      REAL Ksnow           !thermal conductivity of snow [W m-1 K-1]
      REAL Kpeat           !thermal conductivity of dry peat [W m-1 K-1]

      REAL lKorg, lKpeat, lKmin, lKwater, lKice, lKair
      PARAMETER (Kair = 0.025)  !Hillel 1982, Table 9.2
      PARAMETER (Kwater = 0.57) !Hillel 1982, Table 9.2
c      PARAMETER (Kice = 1.84)   !a mean value from James 1968
      PARAMETER (Kice = 2.2)    !from Bartlett et al. 2004
      PARAMETER (Korg = 0.25)   !moss has similar value (Zhuang et al., 2001)
      PARAMETER (Kmin = 2.0)    !Cryosols book, Tarnocai 2004, Table 3.3.1.

c     PARAMETER (Kmin = 0.275)   !p188 in Bonan, mean between sandy and
                                !clay soil at 0% saturation - water will
                                !be added extra
      PARAMETER (Kpeat = 0.06) !Bonan's Ecological Climatology, p.188  
c      PARAMETER (Kpeat = 1.2)  !Zhuang et al. 2001; mean value for
c      thawed upper organic layer


c     ________________________________________________________________________

      REAL Lheat                !latent heat of fusion (Granberg)
      PARAMETER (Lheat = 3.34E8) ![J m-3]


c     ARGUMENTS:  
      INTEGER d

      REAL dtemp(1:365)

!     perm. wilt. point In case of organic soil, this is the water,
!     which doesn't freeze until T drops below -10 deg C. NOTE: I need
!     to implement that it'll freeze once T has dropped below -10 deg,
!     but I haven't done that yet!
      REAL permafrost_index     !should be a number between 0 and 1, the
                                !higher, the more continuous is the permafrost
      REAL thaw_top             !thawing depth, where all the ice has
                                !melted [mm]

c     LOCAL VARIABLES:
      INTEGER i                 !index
      INTEGER IDX, LiIDX, SIDX  !indexes for first active layer of soil,
                                !litter and snow 
      INTEGER MIDX, AIDX        !index for the mixed (litter and
                                !standing water in peatlands) and air
                                !layer
      INTEGER START_IDX         ! index
      INTEGER TOP               ! index for top layer
      INTEGER step, j, layer0, layer
      INTEGER SIDX_old          !index for snow layers from previous day
      INTEGER SIDX_diff         !difference in indexes for snow layers
      INTEGER snow_active       !number of active snow layers
      INTEGER mixedl            !number of active mixed layers (should be 1)
      INTEGER ngroundl          !sum of soil layers
      INTEGER day_count         !count the years for the permafrost index
      INTEGER perma_count       !count the years, where soil is not fully thawed
      LOGICAL first             !needed for standing water initialisation

      REAL store(1:6, 1:9)
      REAL Tsoil_365(NLAYERS)   ! soil temperatures on last day [deg. C]
      REAL T(NLAYERS)           !soil temperature in each layer [deg. C]
      REAL T_old(NLAYERS)       !soil temperature from previous time
                                !step [deg C]
      REAL Tmin(NLAYERS)        !current minimum T [deg. C] used to find
                                !freezing conditions
      REAL delta_T(NLAYERS)     !temperature difference (current - previous
                                !time step[deg C]
      REAL surf_temp            ! [deg. C]
      REAL Di(NLAYERS)          ! [mm2 / day]
      REAL por(NLAYERS)         !porosity of each soil layer
      REAL Dz(NLAYERS)          ! [mm]
      REAL Dlitter              !Diffusivity of litter [mm2 d-1]
      REAL litterdepth          !total litter depth [m]
      REAL litter_ag_total      !total aboveground litter summed over PFTs
      REAL bulk_dens(npft)      !bulk density of litter [kgC m-3]
      REAL bulk_dens_ave        !average bulk density of litter [kgC m-3]
      REAL Dz_snow              !depth of each snow layer [mm]
      REAL Csnow                !heat capacity of snow (J m-3 K-1)
      REAL Dsnow                !Diffusivity of snow [mm2 d-1]
      REAL snowpor              ![m3 m-3]


      REAL pad_dz(PAD_LAYERS)   ! [mm]
      REAL pad_temp(PAD_LAYERS) ! [deg. C]

      REAL lKfwater             !log of therm. cond. for the water fraction
      REAL lKfice               !log of therm. cond. for the ice fraction
      REAL lKforg               !log of therm. cond. for the org. fraction
      REAL lKfpeat              !log of therm. cond. for the peat fraction
      REAL lKfmin               !log of therm. cond. for the min. fraction
      REAL lKfair               !log of therm. cond. for the air fraction

c$$$      REAL Kfwater             !therm. cond. for the water fraction
c$$$      REAL Kfice               !therm. cond. for the ice fraction
c$$$      REAL Kforg               !therm. cond. for the org. fraction
c$$$      REAL Kfpeat              !therm. cond. for the peat fraction
c$$$      REAL Kfmin               !therm. cond. for the min. fraction
c$$$      REAL Kfair               !therm. cond. for the air fraction

      REAL Ktotal(NLAYERS)      !total therm. cond.
      REAL Ctotal(NLAYERS)      !total heat capacity

      REAL Ftot                 !total of solid fraction in soil
      REAL Forg(NLAYERS)        !org. soil fraction
      REAL Fpeat(NLAYERS)       !peat fraction
      REAL Fmin(NLAYERS)        !mineral soil fraction

      REAL Fice_old(NLAYERS)    !ice fraction from previous day
      REAL acro_por             !acro_telm porosity minus Fgas
      LOGICAL Freezing          !Indicates if conditions for freezing
                                !are right
      REAL Ffreez               !fraction of water which freezes
      REAL Fthaw                !fraction of ice which thaws
      REAL Fchange              !fraction which freezes or thaws
      REAL energy               !Energy per volume [J m-3]

c     If you wanted to move the calculation of dz_factors outside the CN
c     Subroutine then undelete the following lines.

c      REAL dzhere, dzminus, dzplus
c      REAL Dz_factors1(NLAYERS)
c      REAL Dz_factors2(NLAYERS)


      SAVE                      !Save all the variables so that they are
                                !available on the next time step 

c----------------
c     Initialise
c----------------
      IF (year .EQ. 1 .AND. d .EQ. 1) THEN
         thawdepth(d) = 0.0
         thaw_top = 0.0
         first = .TRUE.
         !The log is used to speed things up. Without the log you need
         !to use an exponential, with the log you just multiply.
         lKorg = LOG(Korg)
         lKpeat = LOG(Kpeat)
         lKmin = LOG(Kmin)
         lKwater = LOG(Kwater)
         lKice = LOG(Kice)
         lKair = LOG(Kair)
         Dz(NLAYERS) = 100.0
         DO i = 1, NLAYERS
            Tsoil_365(i) = 0.0
            Tmin(i) = 0.0
            T_old(i) = 0.0
            delta_T(i) = 0.0
            Di(i) = 0.04 * 1E6
            Ctotal(i) = Cwater
         ENDDO
      ENDIF

      DO i = 1, NLAYERS
         ice_frac(i) = 0.0
      ENDDO

      

c     ------------------------------------------
c     SET count for permafrost index calculation
c     ------------------------------------------

      IF (d .EQ. 1) THEN
         day_count = 0.0
         perma_count = 0.0
      ENDIF

c     -------------------------------------------
c     Allocate yesterday's temperature to today's
c     -------------------------------------------

      DO i = 1, NLAYERS
         IF (d .EQ. 1) THEN
            Tsoil(d,i) = Tsoil_365(i)
         ELSE
            Tsoil(d,i) = Tsoil(d-1,i)
         ENDIF
      ENDDO

c------------------------------------------------------------------
c     LAYER SETUP
c------------------------------------------------------------------

c     -------------------------------------
c     SNOW LAYER
c     Use one snow layer with varying depth
c     -------------------------------------

      snow_active = 1
      snowpor = 1 - snowdens(d) / 1000.0 

      ! Csnow from Fukusako, it's the same as for ice
      ! ---------------------------------------------
      Csnow = (0.185 + 0.689 * (273 + dtemp(d)) * 0.01) * 1000 *
     $     snowdens(d) 


c      Ksnow = 0.0209 + 2.93 * 1E-6 * snowdens(d) ** 2 !Devaux 1933 in
                                                      !Fukusako 1990

      ! From Sturm et al. 1997, and in Ling and Zhang, 2006
      IF (snowdens(d) .GT. 156) THEN
         Ksnow = 0.138 - 1.01 * snowdens(d) / 1000 + 3.233
     $        * (snowdens(d) / 1000) ** 2
      ELSE
         Ksnow = 0.023 + 0.234 * snowdens(d) / 1000
      ENDIF

      Dsnow = Ksnow / Csnow * 86400 * 1E6

c     -------
c     LAYERS
c     -------

c     -----------------------------------------------------------------
c     After 20 years, calculate the litter depth.
c     Multiply bulk_dens(pft) by 1000 to get from [kgC/m3] to [gC/m3]
c     For peatlands, assume that the litter gets incorporated into the
c     peat instead of accumulating on top of the soil
c     -----------------------------------------------------------------

      IF (year .LT. 20 .OR. peathydr) THEN
         litterdepth = 50       !initialise litterdepth at the beginning
                                !of the simulation (this avoids problems
                                !with dt issues that can lead to the
                                !soil T going wild) and keep it constant
                                !for peatlands
      ELSE
         ! Convert bulk density from kg biomass /m3 to gCm3
         DO pft = 1, npft
            bulk_dens(pft) = pftpar(pft,43) * biom_to_C * 1000 
         ENDDO

         litter_ag_total = 0.0 
         bulk_dens_ave = 0.0

         DO pft = 1, npft
            IF (present(pft)) 
     $           litter_ag_total = litter_ag_total + litter_ag(pft)
         ENDDO
         IF (litter_ag_total .LT. 0) litter_ag_total = 0.0

         ! In case there is no biomass production and hence litter_ag,
         ! set bulk density to a fixed value to avoid division by 0.
         IF (litter_ag_total .GT. 0) THEN
            DO pft = 1, npft
               IF (present(pft)) 
     $              bulk_dens_ave = bulk_dens_ave + bulk_dens(pft) *
     $              litter_ag(pft)/ litter_ag_total
            ENDDO
         ELSE
            bulk_dens_ave = 10000 
         ENDIF
         litterdepth = litter_ag_total / bulk_dens_ave * 1000 ![mm]
      ENDIF


c     Avoid having no litterdepth at all as this will lead to problems
c     as the layer thickness will be set to that value
c     -----------------------------------------------------------------
      IF (litterdepth .LT. 50) THEN
         litterdepth = 50       ![mm]
      ENDIF
 
c     -------------------------------------------------------------------
c     MIXED LAYER
c     There is a mixed layer, which is filled with litter, air or water,
c     depending on the WTP.
c     -------------------------------------------------------------------
      mixedl = 1 

c     -----------------------------------------------------------------
c     All of the layers exist all of the time, the snow layer is either
c     filled with snow or air if there is no snow.  

c     AIDX = air layer index
c     SIDX = snow layer index
c     MIDX = mixed layer index, i.e. litter and water (in peatlands)
c     IDX = soil layers index
c     -----------------------------------------------------------------
      IDX   = NLAYERS - NSOILL + 1
      MIDX  = NLAYERS - NSOILL - mixedl + 1
      SIDX  = NLAYERS - NSOILL - mixedl - snow_active + 1 
      AIDX  = NLAYERS - NSOILL - mixedl - snow_active - 1 + 1 


c     ----------------------------------------
c     Set soil parameters for each soil layer
c     ----------------------------------------
      j = 0
      DO i = IDX, NLAYERS
         j = j + 1
         Dz(i) = Dz_soil(j)
      ENDDO

c     -------------------------------------
c     ALLOCATE values to various variables.
c     -------------------------------------

      IF (peathydr) THEN

         ! ACROTELM
         ! --------
         DO i = IDX, NLAYERS - NCATOTELM
            por(i)   = acrotelm_por - Fgas 
            Forg(i)  = 0.0
            Fmin(i)  = soilpar(5) !is set to 0
            Fpeat(i) = 1 - acrotelm_por
c        If you want to have super-cooled water, then you should set
c        Fpwp to a fraction, which does not freeze even when the T drops
c        below 0. At the moment, this is not activated.
c$$$            IF (Tsoil(d,i) .GE. -10.0) THEN
c$$$               Fpwp(i) = 0.0
c$$$c              Fpwp(i) = soilpar(7)! + 0.159 !add water - see Nakano & Brown
c$$$                                            !and notebook entry from the
c$$$                                            !27 May 2006
c$$$            ELSE
c$$$               Fpwp(i) = 0.0
c$$$            ENDIF
         ENDDO

         ! CATOTELM
         ! --------
         DO i = IDX + NACROTELM, NLAYERS
            por(i)   = soilpar(8) - Fgas
            Forg(i)  = 0.0
            Fmin(i) = soilpar(5)
            Fpeat(i) = soilpar(6)
c     ------------------------------------------------------------------
c     NOTES:
c        If you want to have super-cooled water, then you should set
c        Fpwp to a fraction, which does not freeze even when the T drops
c        below 0. At the moment, this is not activated.  The problem I
c        encountered was this: I found data which showed that you find
c        liquid water in frozen soil down to a temperature of about
c        -10degC. However, there were no data showing, how the liquid
c        water content decreased thereafter and how fast the water froze
c        or if it froze at all.
c     ------------------------------------------------------------------
c$$$            IF (Tsoil(d,i) .GE. -10.0) THEN
c$$$               Fpwp(i) = 0.0
c$$$c              Fpwp(i) = soilpar(7) ! + 0.159 !add water see Nakano & Brown
c$$$                                            !and notebook entry from the
c$$$                                            !27 May 2006
c$$$            ELSE
c$$$               Fpwp(i) = 0.0
c$$$            ENDIF
         ENDDO


         ! MIXED LAYER
         ! consists of litter and water and/or ice
         ! ---------------------------------------
         Dz(MIDX) = litterdepth + Fwater(d,MIDX) * maxh + Fice(d,MIDX) *
     $        maxh
         !-------------------------------------------------
         !multiply by 86400 to get from m2 s-1 to m2 d-1
         !multiply by 1E6 to get from m2 d-1 to mm2 d-1
         !-------------------------------------------------
         ! Note: Di(MIDX) should take the water and ice content into
         ! account, but when I included it into the loop below where Di
         ! gets updated, the model produced NaN in a few places.
         Di(MIDX) = Korg / Corg * 86400.0 * 1E6
         Forg(MIDX) = litterdepth / Dz(MIDX)
         por(MIDX) = 0.0 ! as it's completely filled with litter, water or ice
         Fmin(MIDX) = 0.0
         Fpeat(MIDX) = 0.0
         Fpwp(MIDX) = 0.0
      ELSE

      ! NON-WETLAND SOILS
      ! -----------------
         DO i = IDX, NLAYERS
            por(i)  = soilpar(8)
            Forg(i) = soilpar(6)
            Fmin(i) = MAX(0.0, soilpar(5))
c$$$            IF (Tsoil(d,i) .GE. -10.0) THEN
c$$$c               Fpwp(i) = soilpar(7) 
c$$$               Fpwp(i) = 0.0
c$$$            ELSE
c$$$               Fpwp(i) = 0.0
c$$$            ENDIF
            Fpeat(i) = 0.0
         ENDDO
         ! MIXED LAYER
         ! -----------
         Dz(MIDX) = litterdepth
         !-------------------------------------------------
         !multiply by 86400 to get from m2 s-1 to m2 d-1
         !multiply by 1E6 to get from m2 d-1 to mm2 d-1
         !-------------------------------------------------
         Di(MIDX) = Korg / Corg * 86400.0 * 1E6
         Forg(MIDX) = 1 
         por(MIDX) = 1.0 - Forg(MIDX)
         Fmin(MIDX) = 0.0
         Fpeat(MIDX) = 0.0
         Fpwp(MIDX) = 0.0
         Fwater(d,MIDX) = 0.0 
         Fice(d,MIDX) = 0.0
      ENDIF
    

c     -----------------------------------------------------------------
c     Set thickness of 'empty' snow layer and air layer to a high value
c     to avoid numerical instabilities
c     -----------------------------------------------------------------

      ! SNOW LAYER
      ! ----------
      IF (snowdepth(d) .GT. 0) THEN
         Di(SIDX) = Dsnow
         Dz(SIDX) = snowdepth(d)
      ELSE !set snow layer to air values
         Di(SIDX) = Kair / Cair * 86400 * 1E6
         Dz(SIDX) = 1000
      ENDIF

      ! AIR LAYER
      ! ---------
      Dz(AIDX) = 1000
      por(AIDX) = 1
      Forg(AIDX) = 0.0
      Fmin(AIDX) = 0.0
      Fpwp(AIDX) = 0.0
      Fpeat(AIDX) = 0.0
      Di(AIDX) = Kair / Cair * 86400 * 1E6

c     -------------
c     PADDING SETUP
c     -------------
      ! Set up padding - use calc_padding to calculate the thickness
      ! of the padding layers, then initialise the padding layer
      ! temperatures to zero.
      IF (first) THEN
         CALL calc_padding(PAD_DEPTH, PAD_LAYERS, Dz(NLAYERS), pad_dz)
         DO layer = 1, PAD_LAYERS
            pad_temp(layer) = 0.0
         END DO
         first = .FALSE.
      ENDIF


      
c     Assign daily temperature to surface temperature
c     -----------------------------------------------
      surf_temp = dtemp(d) 

c^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
c     If you wanted to move the calculation of dz_factors outside the CN
c     Subroutine then undelete the following lines. The reason why you
c     might want to do that is to speed the code up. As it does not make
c     a big difference though, it's not active at the moment.

c$$$      DO layer = 1, nlayers - layer0 + 1 + pad_layers
c$$$         ! Extract sensible values to use for temperature and diffusivity
c$$$         ! of the current layer, the layer above and the layer below,
c$$$         ! taking account of padding layers and end cases.
c$$$         IF (layer .LE. nlayers) THEN
c$$$            dzhere = Dz(layer)
c$$$         ELSE
c$$$            dzhere = pad_dz(layer - nlayers)
c$$$         END IF
c$$$         IF (layer .LE. nlayers - 1) THEN
c$$$            dzplus = Dz(layer + 1)
c$$$         ELSE IF (layer .LT. nlayers + pad_layers) THEN
c$$$            dzplus = pad_dz(layer - nlayers + 1)
c$$$         ELSE
c$$$            dzplus = pad_dz(pad_layers)
c$$$         END IF
c$$$         IF (layer .EQ. 1) THEN
c$$$            dzminus = Dz(layer)
c$$$         ELSE IF (layer .LE. nlayers + 1) THEN
c$$$            dzminus = Dz(layer - 1)
c$$$         ELSE
c$$$            dzminus = pad_dz(layer - nlayers - 1)
c$$$         END IF
c$$$         Dz_factors1(layer) = 0.25 * (dzplus + 2 * dzhere + dzminus) *
c$$$     $        (dzplus + dzhere)
c$$$         Dz_factors2(layer) = 0.25 * (dzplus + 2 * dzhere + dzminus) *
c$$$     $        (dzhere + dzminus)
c$$$      END DO
c$$$
c^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


c     ------------------------------------------
c     Allocate surface temperature to air layer
c     ------------------------------------------
c      Tsoil(d,AIDX) = surf_temp
      layer0 = AIDX

c     --------------------------------------------------------------------
c     Do one timestep of the Crank-Nicholson method.
c     --------------------------------------------------------------------

c^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
c     If you wanted to move the calculation of dz_factors outside the CN
c     Subroutine then undelete the following lines. 

c      CALL cnstep(NLAYERS, layer0, T, Di, Dz, Dz_factors1, Dz_factors2,
c     $     surf_temp,Dt, PAD_LAYERS, pad_dz, pad_temp,d,year)
c^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

      DO i = AIDX, NLAYERS
         T(i) = Tsoil(d,i)
      ENDDO

      CALL cnstep(layer0, T, Di, Dz, surf_temp,Dt, PAD_LAYERS, pad_dz,
     $     pad_temp,d,year)

      DO i = AIDX, NLAYERS
         Tsoil(d,i) = T(i)
       ENDDO


c     --------------------------------------------------------------------
c     Calculate WATER AND ICE fraction in each layer
c     --------------------------------------------------------------------
c     Note: To know how much ice is formed or melted at each time step,
c     I first calculate the temperature change from the previous to this
c     time step. If not all of the available water is frozen, then the
c     temperature will be held at 0 deg C until everything is frozen (and
c     vice versa for the melting). You need to find the amount of heat
c     released when freezing or absorbed when thawing. To do this you
c     multiply the total heat capacity of the soil times delta_T. Since
c     we are working in m-3 here, we don't need to multiply by the volume
c     or mass, because we want to get the fraction of water freezing or
c     ice thawing, not the absolute value. 
c     --------------------------------------------------------------------
      

      !Run through layers
      !------------------

      DO i = IDX, NLAYERS
         ! Subtract the non-freezable (water that does not freeze until
         ! the temperature drops below -10degC) water from Fwater

         Fwater(d,i) = MAX(0.0, Fwater(d,i) - Fpwp(i))

         !Initialise temperatures on first day
         !-------------------------------------
         IF (d .EQ. 1 .AND. year .EQ. 1) T_old(i) = Tsoil(d,i)

         delta_T(i) = Tsoil(d,i) - T_old(i)

         ! Determine if conditions are right for freezing
         ! ----------------------------------------------

         ! -----------------------------------------------------------
         ! FREEZING
         ! Freezing is only allowed under the following conditions:
         ! 1. Tsoil is < zero
         ! 2. Tsoil is < T_old to ensure falling temperature
         ! 3. Freezing can happen only during the second half of the
         ! year, this avoids problems with refreezing in spring, which
         ! we can't account for
         ! -----------------------------------------------------------

         IF (Tsoil(d,i) .LT. 0 .AND. Tsoil(d,i) .LT. T_old(i) .AND.
     $        Fwater(d,i) .GT. 0.0) THEN
            Freezing = .TRUE.
            Tmin(i) = Tsoil(d,i)
         ELSE
            Freezing = .FALSE.
         ENDIF

         IF (Freezing) THEN
            Fthaw = 0.0
            IF (Fwater(d,i) .GT. 0.0) THEN 
c               energy = Ctotal(i) * ABS(delta_T(i)) !energy in J m-3
               energy = Cwater * ABS(delta_T(i)) !energy in J m-3
               Ffreez = energy / Lheat !unitless fraction

               IF (Fwater(d,i) .GT. Ffreez) THEN
                  Fice(d,i) = Fice(d,i) + Ffreez
                  Fwater(d,i) = Fwater(d,i) - Ffreez
                  Tsoil(d,i) = 0.0
                  IF (Fwater(d,i) .LT. 0.000001) Fwater(d,i) = 0.0
                  IF (Fice(d,i) .LT. 0.000001) Fice(d,i) = 0.0                  
               ELSE
               ! ------------------------------------------------
               ! It is cold enough to freeze all the water and
               ! decrease the temperature below 0, since the rest
               ! of the water gets frozen now.
               ! ------------------------------------------------
                  Fice(d,i) = Fice(d,i) + Fwater(d,i)
                  Tsoil(d,i) = -(Ffreez - Fwater(d,i)) *
     $                 Lheat / Cwater 
                  IF (Fice(d,i) .LT. 0.000001) Fice(d,i) = 0.0                  
                  Fwater(d,i) = 0.0
               ENDIF
            ELSE
               Ffreez = 0.0
            ENDIF

         ! --------
         ! THAWING
         ! --------


         ELSE IF (Tsoil(d,i) .GE. 0.0) THEN
            Ffreez = 0.0
            If (Fice(d,i) .GT. 0.0) THEN
c               energy = Ctotal(i) * ABS(delta_T(i)) !energy in J m-3
               energy = Cwater * ABS(delta_T(i)) !energy in J m-3
               Fthaw = energy / Lheat !unitless fraction
               IF (Fice(d,i) .GT. Fthaw) THEN
                  Fwater(d,i) = Fwater(d,i) + Fthaw
                  Fice(d,i) = Fice(d,i) - Fthaw 
                  Tsoil(d,i) = 0.0
                  IF (Fice(d,i) .LT. 0.000001) Fice(d,i) = 0.0
                  IF (Fwater(d,i) .LT. 0.000001) Fwater(d,i) = 0.0
               ELSE
                ! ------------------------------------------------
                ! There is more energy available than required for
                ! melting the rest of the ice. Therefore the
                ! left-over energy will increase the temperature.
                ! ------------------------------------------------
                  Fwater(d,i) = Fwater(d,i) + Fice(d,i)
                  Tsoil(d,i) = (Fthaw - Fice(d,i)) * Lheat / Cwater 
                  Fice(d,i) = 0.0
                  IF (Fwater(d,i) .LT. 0.000001) Fwater(d,i) = 0.0
               ENDIF
            ELSE
               Fthaw = 0.0
               Fice(d,i) = 0.0
            ENDIF

         ENDIF           
         T_old(i) = Tsoil(d,i)
 
         IF (Tsoil(d,i) .GE. 0.0) Tmin(i) = 0.0

         ! Add the non-freezable water to Fwater again
         !---------------------------------------------
         Fwater(d,i) = Fwater(d,i) + Fpwp(i)

c     -------------------------------------------------------------------
c     Update HEAT CAPACITY for each layer
c     -------------------------------------------------------------------

         IF (i .EQ. MIDX) THEN
            Fair(d,i) = 0.0
         ELSE
            Fair(d,i) = MAX(0.0,Fgas + por(i) - Fwater(d,i) - Fice(d,i))
         ENDIF

         Ctotal(i) = Fmin(i) * Cmin + Forg(i) * Corg + Fice(d,i) * Cice
     $        +Fwater(d,i) * Cwater + Fpeat(i) * Cpeat + Fair(d,i) *
     $        Cair


c     -------------------------------------------------------------------
c     Update the THERMAL CONDUCTIVITY K - which follows
c     Granberg et al. 1999, who refers to Farouki, 1986
c     -------------------------------------------------------------------

         Ftot = Fwater(d,i) + Fice(d,i) + Forg(i) + Fmin(i) +
     $        Fpeat(i)

c$$$         Kforg = Korg ** (Forg(i) / Ftot)
c$$$         Kfpeat = Kpeat ** (Fpeat(i) / Ftot)
c$$$         Kfmin = Kmin ** (Fmin(i) / Ftot)
c$$$         Kfwater = Kwater ** ((Fwater(d,i) + Fpwp(i)) / Ftot)
c$$$         Kfice = Kice ** (Fice(d,i) / Ftot)

c$$$         Ktotal(i) =  (Fair(d,i) + Fgas) * Kair +
c$$$     $        Ftot * Kfwater * Kfice * Kforg * Kfmin * Kfpeat


         ! Computational faster way to do the calculations of Ktotal:
         !------------------------------------------------------------
         lKforg = (Forg(i) / Ftot) * lKorg
         lKfpeat = (Fpeat(i) / Ftot) * lKpeat
         lKfmin = (Fmin(i) / Ftot) * lKmin
         lKfwater = (Fwater(d,i)/ Ftot) * lKwater
         lKfice = (Fice(d,i) / Ftot) * lKice
         lKfair = (Fair(d,i) / Ftot) * lKair

         Ktotal(i) = Fair(d,i) * Kair + 
     $        Ftot * EXP(lKfwater + lKfice + lKforg + lKfmin + lKfpeat)


c     -------------------------------------------------------------------
c     Update THERMAL DIFFUSIVITY for each SOIL layer
c     -------------------------------------------------------------------
         !-------------------------------------------------
         !multiply by 86400 to get from m2 s-1 to m2 d-1
         !multiply by 1E6 to get from m2 d-1 to mm2 d-1
         !-------------------------------------------------

         Di(i) = Ktotal(i) / Ctotal(i) * 86400.0 * 1E6

c     ----------------------------------------------------------------
c     Ice_frac is for output purposes only
c     ----------------------------------------------------------------
         IF (Fice(d,i) .GT. 0.01) THEN
            ice_frac(i) = Fice(d,i) / (Fice(d,i) + Fwater(d,i)) 
            !ERROR message:
            !--------------
           IF (ice_frac(i) .GT. 1.0) THEN 
              print*, 'ERROR: ice fraction',  i, ice_frac(i), '> 1!'
               print*, year, d, Fice(d,i), Fwater(d,i)
            ENDIF
         ELSE
            ice_frac(i) = 0.0
         ENDIF
         
      ENDDO


c     ----------------------------------------------------------------
c     SOIL TEMPERATURE for output purposes. Only Tsoil_25 is further
c     needed within the code.
c     ----------------------------------------------------------------

         Tsurf(d) = surf_temp

         Tsoil_0(d)  = Tsoil(d,MIDX) * (Dz(MIDX) / (Dz(MIDX) + Dz(IDX)))
     $        + Tsoil(d,IDX) * (Dz(IDX) / (Dz(MIDX) + Dz(IDX)))

         Tsoil_10(d)  = Tsoil(d,IDX) * (Dz(IDX) / (Dz(IDX) + Dz(IDX +
     $        1)))+ Tsoil(d,IDX + 1) * (Dz(IDX + 1) / (Dz(IDX) + Dz(IDX
     $        + 1)))

         Tsoil_20(d)  = Tsoil(d,IDX + 1) * (Dz(IDX + 1) / (Dz(IDX + 1) +
     $        Dz(IDX + 2)))+ Tsoil(d,IDX + 2) * (Dz(IDX + 2) / (Dz(IDX +
     $        1) + Dz(IDX + 2)))

         Tsoil_25(d) = Tsoil(d, IDX + 2)

         Tsoil_50(d)  = Tsoil(d,IDX + 3) * (Dz(IDX + 3) / (Dz(IDX + 3) +
     $        Dz(IDX + 4)))+ Tsoil(d,IDX + 4) * (Dz(IDX + 4) / (Dz(IDX +
     $        3) + Dz(IDX + 4)))

         Tsoil_100(d)  = Tsoil(d,IDX + 5) * (Dz(IDX + 5) / (Dz(IDX + 5)
     $        + Dz(IDX + 6)))+ Tsoil(d,IDX + 6) * (Dz(IDX + 6) / (Dz(IDX
     $        + 5) + Dz(IDX + 6)))

         Tsoil_150(d)  = Tsoil(d,IDX + 6) * (Dz(IDX + 6) / (Dz(IDX + 6)
     $        + Dz(IDX + 7)))+ Tsoil(d,IDX + 7) * (Dz(IDX + 7) / (Dz(IDX
     $        + 6) + Dz(IDX + 7)))

         Tsoil_200(d)  = Tsoil(d,IDX + 7) * (Dz(IDX + 7) / (Dz(IDX + 7)
     $        + pad_dz(1))) + pad_temp(1) * (pad_dz(1) / (Dz(IDX + 7) +
     $        pad_dz(1)))

c     ------------------------------------------------------------------
c     THAW DEPTH: Define the thaw depth as the 0degC isotherm (Muller
c     1947, Burn 1998), which equals the depth, down to which liquid
c     water exists continuously from the top down.
c     The thaw depth is also defined as the 0degC isotherm.
c     ------------------------------------------------------------------
         
      thawdepth(d) = 0.0
      i = IDX 
      DO WHILE (i .LE. NLAYERS .AND. Tsoil(d,i) .GE. 0.0)
         thawdepth(d) = thawdepth(d) + Dz(i)
         i = i + 1
      ENDDO
      thawdepth(d) = MIN(2000.0, thawdepth(d))

c     Calculate also at which depth there is no ice anymore. This could
c     be used in the waterbalance SR to estimate drainage or
c     runoff. (Not implemented at the moment.)

      thaw_top = 0.0
      i = IDX
      DO WHILE (i .LE. NLAYERS .AND. Fice(d,i) .LT. 0.001)
         thaw_top = thaw_top + Dz(i)
         i = i + 1
      ENDDO

c     Check if each soil layer thaws at least for one day of the
c     year. This is needed to know if roots will occur in this layer or
c     not. This information is used in the methane SR.
      DO i = IDX, NLAYERS
         IF (Fwater(d,i) .LE. Fpwp(i) .AND. Fice(d,i) .GT. 0.001) THEN
            frozen_day(d,i) = .TRUE.
         ELSE
            frozen_day(d,i) = .FALSE.
         ENDIF
      ENDDO
      

      IF (d .eq. 365) THEN
         DO i = 1, NLAYERS
            Tsoil_365(i) = Tsoil(365,i)
            Fice_365(i) = Fice(d,i)
            Fwater_365(i) = Fwater(d,i)
         ENDDO
      ENDIF


      RETURN
      END SUBROUTINE soiltemp 


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
!----------------------------------------------------------------------
!
!     SUBROUTINE cnstep
!
!     Crank-Nicholson timestepper for temperature diffusion equation.
!     This routine performs a single timestep (of length dt) of the heat
!     diffusion equation
!
!                        6 T    6  /      6 T \
!                        --- = --- | D(z) --- |
!                        6 t   6 z \      6 z /
!
!     where here, T(z, t) is the temperature, the '6's represent partial
!     differentiation, and D(z) is the (depth-dependent) diffusion
!     constant.  The boundary conditions are a prescribed surface
!     temperature, and no heat flow at the bottom of the solution domain.
!
!     INPUT PARAMETERS:
!
!      nlayers      Total number of layers represented in temp, d and
!                   dz arrays.  Not all of the layers have to be active
!                   at one time, but this parameter is used to constrain
!                   the sizes of the depth-based arrays.!
!
!      layer0       Index of the first active layer.  The diffusion
!                   equation will be solved for all layers in the range
!                   layer0 - nlayers inclusive, and the surface temperature
!                   boundary condition will be applied to the top of layer
!                   index layer0.
!
!      temp         The temperatures in each layer at the start of the
!                   timestep.
!
!      d            The thermal diffusivities in each layer.
!
!      dz           The thickness of each layer.
!
!      surf_temp    The surface temperature boundary condition.
!
!      dt           The timestep.
!
!      pad_layers   Number of layers to use below computation domain to
!                   deal with bottom boundary condition.
!
!      pad_dz       Padding layer thicknesses (derived from calc_padding).
!
!      pad_temp     Temperature in padding layers.
!
!     NOTE ON UNITS: no units are specified here for any of these
!     inputs; the only constraint on units is that the layer
!     thicknesses, the diffusion constant and the timestep should be
!     measured in consistent units.  For instance, if the layer
!     thicknesses are measured in metres, and the timestep in days, then
!     the diffusion constants should have units of m2 / day.
!
!     OUTPUT PARAMETERS
!
!      temp         The temperatures in each layer at the end of the
!                   timestep.  Only layers layer0 - nlayers will have
!                   valid temperature values after a call to cnstep.  All
!                   layers with index less than layer0 will have missing
!                   value temperatures set (a value of -1.0E35).
!
!      pad_temp     Temperature in padding layers at the end of the
!                   timestep.
!
!----------------------------------------------------------------------
c     If you wanted to move the calculation of dz_factors outside the CN
c     Subroutine then undelete the following lines.

c$$$      SUBROUTINE cnstep(nlayers, layer0, temp, Di, dz,
c$$$     $     dz_factors1, dz_factors2,
c$$$     $     surf_temp, dt, pad_layers, pad_dz, pad_temp,d,year)
      SUBROUTINE cnstep(layer0, temp, Di, dz,
     $     surf_temp, dt, pad_layers, pad_dz, pad_temp,d,year)

      IMPLICIT NONE

      include "params.cb"

      ! --- PARAMETERS ---

      INTEGER d,year
      INTEGER layer0            ! Index of first active layer.
      REAL temp(NLAYERS)        ! Layer temperatures.
      REAL Di(NLAYERS)          ! Layer diffusion constants.
      REAL dz(NLAYERS)          ! Layer thicknesses.
c      REAL dz_factors1(NLAYERS)
c      REAL dz_factors2(NLAYERS)
      REAL surf_temp            ! Surface temperature forcing.
      REAL dt                   ! Timestep.
      INTEGER pad_layers
      REAL pad_dz(pad_layers)
      REAL pad_temp(pad_layers)
      

      ! --- LOCAL VARIABLES ---


      ! Layer counters: note that there are two different layer counting
      ! schemes used, one for the input and output parameters (vectors of
      ! length NLAYERS) and one for the values used in the Crank-Nicholson
      ! solver (vectors of length active_layers)
      INTEGER layer, lidx, active_layers

      ! Diffusion constants averaged over adjacent layers.
      REAL dplus, dminus

      ! Layer-dependent weighting factors in Crank-Nicholson scheme.
      REAL dz_factor, Cplus, Cminus
      REAL dzhere, dzminus, dzplus
      REAL there, tminus, tplus

      ! Leading diagonal, left and right subdiagonals for Crank-Nicholson
      ! matrix.
      REAL diag(NLAYERS - layer0 + 1 + pad_layers)
      REAL left(NLAYERS - layer0 + 1 + pad_layers)
      REAL right(NLAYERS - layer0 + 1 + pad_layers)

      ! Right hand side vector for Crank-Nicholson scheme equations.
      REAL rhs(NLAYERS - layer0 + 1 + pad_layers)

      ! Solution vector for Crank-Nicholson scheme equations.
      REAL solution(NLAYERS - layer0 + 1 + pad_layers)


      ! --- CODE STARTS HERE ---

      ! Not all layers in the input arrays are actually used.  Calculate
      ! how many layers we have to deal with here.

      active_layers = NLAYERS - layer0 + 1 + pad_layers


      !------------------------------------------------------------
      !
      !   BUILD TRIDIAGONAL MATRIX AND KNOWN RIGHT HAND SIDE
      !

      ! End members for off-diagonal elements.
      left(1) = 0.0
      right(active_layers) = 0.0

      ! Process the active layers.
      DO lidx = 1, active_layers
         ! Deal with different layer counting schemes.
         layer = lidx + layer0 - 1

         ! Calculate diffusion constants averaged over adjacent layers.
         ! The diffusion constant at the bottom layer is clamped to zero
         ! to enforce the no heat flow boundary condition there.  Diffusion
         ! constants in the padding layers are calculated in a sensible way.
         IF (layer .EQ. NLAYERS + pad_layers) THEN
            dplus = 0.0
         ELSE IF (layer .GE. NLAYERS) THEN
            dplus = Di(NLAYERS)
         ELSE
            dplus = 0.5 * (Di(layer) + Di(layer + 1))
         END IF
         IF (layer .EQ. layer0) THEN
            dminus = Di(layer)
         ELSE IF (layer .GT. NLAYERS) THEN
            dminus = Di(NLAYERS)
         ELSE
            dminus = 0.5 * (Di(layer) + Di(layer - 1))
         END IF

         ! Extract sensible values to use for temperature and diffusivity
         ! of the current layer, the layer above and the layer below,
         ! taking account of padding layers and end cases.
         IF (layer .LE. NLAYERS) THEN
            dzhere = dz(layer)
            there = temp(layer)
         ELSE
            dzhere = pad_dz(layer - NLAYERS)
            there = pad_temp(layer - NLAYERS)
         END IF
         IF (layer .LE. NLAYERS - 1) THEN
            dzplus = dz(layer + 1)
            tplus = temp(layer + 1)
         ELSE IF (layer .LT. NLAYERS + pad_layers) THEN
            dzplus = pad_dz(layer - NLAYERS + 1)
            tplus = pad_temp(layer - NLAYERS + 1)
         ELSE
            dzplus = pad_dz(pad_layers)
            tplus = pad_temp(pad_layers)
         END IF
c         IF (layer .EQ. 1) THEN
         IF (layer .EQ. layer0) THEN
            dzminus = dz(layer)
            tminus = temp(layer)
         ELSE IF (layer .LE. NLAYERS + 1) THEN
            dzminus = dz(layer - 1)
            tminus = temp(layer - 1)
         ELSE
            dzminus = pad_dz(layer - NLAYERS - 1)
            tminus = pad_temp(layer - NLAYERS - 1)
         END IF


c     If you wanted to move the calculation of dz_factors outside the CN
c     Subroutine then undelete the following lines.

c$$$         IF (layer .LE. NLAYERS) THEN
c$$$            there = temp(layer)
c$$$         ELSE
c$$$            there = pad_temp(layer - NLAYERS)
c$$$         END IF
c$$$         IF (layer .LE. NLAYERS - 1) THEN
c$$$            tplus = temp(layer + 1)
c$$$         ELSE IF (layer .LT. NLAYERS + pad_layers) THEN
c$$$            tplus = pad_temp(layer - NLAYERS + 1)
c$$$         ELSE
c$$$            tplus = pad_temp(pad_layers)
c$$$         END IF
c$$$         IF (layer .EQ. 1) THEN
c$$$            tminus = temp(layer)
c$$$         ELSE IF (layer .LE. NLAYERS + 1) THEN
c$$$            tminus = temp(layer - 1)
c$$$         ELSE
c$$$            tminus = pad_temp(layer - NLAYERS - 1)
c$$$         END IF


         ! Calculate layer-dependent weighting factors.
c         Cplus = dplus * dt / dz_factors1(layer)
c         Cminus = dminus * dt / dz_factors2(layer)

         dz_factor = 0.25 * (dzplus + 2 * dzhere + dzminus)
         Cplus = dplus * dt / dz_factor / (dzplus + dzhere)
         Cminus = dminus * dt / dz_factor / (dzhere + dzminus)

         ! Fill in matrix diagonal and off-diagonal elements.
         IF (lidx .EQ. 1) THEN
            diag(lidx) = 1.0
         ELSE
            diag(lidx) = 1.0 + Cplus + Cminus
         END IF
         IF (lidx .GT. 1) left(lidx) = -Cminus
         IF (lidx .LT. active_layers) THEN
            IF (lidx .GT. 1) THEN
               right(lidx) = -Cplus
            ELSE
               right(lidx) = 0.0
            END IF
         END IF

         ! Calculate right hand side vector values.
         IF (lidx .EQ. 1) THEN
            rhs(lidx) = surf_temp
         ELSE IF (lidx .EQ. active_layers) THEN
            rhs(lidx) = (1.0 - Cminus) * there + Cminus * tminus
         ELSE
            rhs(lidx) = (1.0 - Cplus - Cminus) * there +
     $           Cplus * tplus + Cminus * tminus
         END IF
      END DO

      !------------------------------------------------------------
      !
      !   SOLVE TRIDIAGONAL SYSTEM
      !

      CALL tridag(active_layers, left, diag, right, rhs, solution)


      !------------------------------------------------------------
      !
      !   FORMAT OUTPUT PARAMETERS
      !

      ! Transfer the solution to the temperature array.  Also update
      ! the padding layer temperatures so they can be used for the
      ! next timestep.
      DO lidx = 1, NLAYERS
         IF (lidx .LT. layer0) THEN
            temp(lidx) = MISSING_VALUE
         ELSE
            temp(lidx) = solution(lidx - layer0 + 1)
         END IF

      END DO
      DO lidx = 1, pad_layers
         pad_temp(lidx) = solution(lidx + NLAYERS - layer0 + 1)
      END DO

      END SUBROUTINE cnstep

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
!-------------------------------------------------------------------------
!
!     SUBROUTINE tridag
!
!     Tridiagonal system solver from Numerical Recipes. Press et al. 1986.
!
!-------------------------------------------------------------------------

      SUBROUTINE tridag(n, a, b, c, r, u)

      IMPLICIT NONE

      INTEGER n, i
      REAL a(n), b(n), c(n), r(n), u(n)

      REAL gam(n), bet
      INTEGER j

      IF (b(1) .EQ. 0.0) THEN
         PRINT *, 'ERROR in tridag: degenerate system!'
         STOP
      END IF

      bet = b(1)
      u(1) = r(1) / bet
      DO j = 2, n
         gam(j) = c(j - 1) / bet
         bet = b(j) - a(j) * gam(j)
         IF (bet .EQ. 0.0) THEN
            PRINT *, 'ERROR in tridag: zero pivot!'
            STOP
         END IF
         u(j) = (r(j) - a(j) * u(j - 1)) / bet
      END DO

      DO j = n - 1, 1, -1
         u(j) = u(j) - gam(j + 1) * u(j + 1)
      END DO

      END SUBROUTINE tridag

c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
!----------------------------------------------------------------------
!
!     SUBROUTINE calc_padding
!
!     Calculate padding layer depths for Crank-Nicholson bottom boundary
!     condition trick.  The padding layer thicknesses are calculated as
!     a geometric sequence with a given total padding thickness and
!     number of layers.  This gives a smooth increase in layer depth
!     away from the calculation domain.
!
!----------------------------------------------------------------------

      SUBROUTINE calc_padding(pad_depth, pad_layers, dz, pad_dz)

      IMPLICIT NONE

      REAL pad_depth            ! Total depth of padding to use [mm].
      INTEGER pad_layers        ! Number of padding layers.
      REAL dz                   ! Thickness of last real layer.
      REAL pad_dz(pad_layers)   ! Output padding layer thicknesses [mm].

      ! Function, derivative and parameter for geometric factor
      ! calculation.
      REAL func, dfunc, k

      ! Iterative improvements of geometric factor
      REAL k_guess, k_last

      ! Layer counter.
      INTEGER layer

      ! Statement functions for function and derivative for Newton
      ! solver for geometric sequence factor.
      func(k) = k ** (pad_layers + 1) -
     $     k * (1.0 + pad_depth / dz) + pad_depth / dz
      dfunc(k) = (pad_layers + 1) * k ** pad_layers -
     $     (1.0 + pad_depth / dz)

      ! Initial state - don't choose 1.0 as the initial value for
      ! k_guess, since it's a trivial solution!
      k_guess = 10.0
      k_last = 0.0

      ! Do Newton-Raphson solve to find geometric sequence factor.
      DO WHILE (ABS(k_guess - k_last) .GT. 1E-2)
         k_last = k_guess
         k_guess = k_guess - func(k_guess) / dfunc(k_guess)
      END DO

      ! Build geometric sequence of padding layer depths.
      DO layer = 1, pad_layers
         pad_dz(layer) = dz * k_guess ** layer
      END DO
      

      RETURN
      END SUBROUTINE calc_padding


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE NPP
c     Calculation of maintenance and growth respiration and NPP

      subroutine npp(soilcode,pftpar,senspar,sens_test,methane_mod,dtemp
     $     ,Tsoil_25,tree,dphen,nind,lm_ind,sm_ind,rm_ind,mgpp,anpp
     $     ,mnpp,anpp_red,mnpp_red,bm_inc,present,mlresp,sresp,rresp
     $     ,gresp,aresp,year ,grass,moss,lm_ind_PFT11,exudates,aaresp
     $     ,xalresp)

      IMPLICIT NONE

      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"
      include "ch4vars.cb"

c     PARAMETERS
      real k
c     OLD RESPIRATION COEFFICIENT
c      parameter (k=0.0548)
c     NEW RESPIRATION COEFFICIENT
      parameter (k=0.095218)    !Factor was changed due to
                                !recommendation by Annett Wolf in 2008

      REAL exu_frac             !Fraction of exudates
c      PARAMETER (exu_frac = 0.175)
c     ARGUMENTS
      real dphen(1:365,1:npft)
      real dtemp(1:365)
      real aaresp               !annual autotrophic respiration (gC/m2)
      real xalresp(npft)        !annual leaf respiration per PFT(gC/m2)

c     LOCAL VARIABLES
      real l_cton     ! leaf tissue C:N mass ratio
      real s_cton     ! sapwood tissue C:N mass ratio
      real r_cton     ! fineroot tissue C:N mass ratio
      real respcoeff  ! coefficient in respiration equations
      real lresp_m    ! monthly total leaf respiration
      real sresp_m    ! monthly total sapwood respiration
      real rresp_m    ! monthly total root respiration
      real lresp_d    ! daily total leaf respiration
      real sresp_d    ! daily sapwood respiration
      real rresp_d    ! daily root respiration
      real gtemp_air  ! value of temperature response function
                      !given air temperature
      real gtemp_soil ! value of temperature response function
                      !given soil temperature 
      real mresp      ! total monthly maintenance respiration
      real gresp_m      ! monthly growth respiration
      integer d,dm
      real gresp(1:12,1:npft)  ! monthly growth respiration
      real mlresp(1:12,1:npft)  ! monthly leaf respiration
      real lresp(1:12,1:npft)  ! monthly leaf respiration
      real sresp(1:12,1:npft)  ! monthly sapwood respiration
      real rresp(1:12,1:npft)  ! monthly root respiration
      real aresp(1:12,1:npft)  ! monthly autotrophic respiration
      real nleaf(1:npft)        !Rita: leaf N content
      real nleafper(1:npft)     !Rita: N content in %
      real test(12)




c     Initialise

      ! LPJ-WHy
      ! -------
      DO m = 1, 12
         exudates(m) = 0.0
         DO pft = 1, npft
            mnpp_red(m,pft) = 0.0
         ENDDO
      ENDDO
      DO pft = 1, npft
         anpp_red(pft) = 0.0
      ENDDO


      IF (sens_test) THEN
         exu_frac = senspar(7)
      ELSE
         exu_frac = 0.15
      ENDIF


c     Calculation of maintenance respiration components for each
c     living tissue.

c     Based on the relations

c     (A) Tissue respiration response to temperature
c         (Sprugel et al, in press, Eqn 7)

c         (A1) Rm = 7.4e-7 * N * f(T)
c         (A2) f(T) = EXP (beta * T) 

c           where Rm   = tissue maintenance respiration rate in mol C/sec
c                 N    = tissue nitrogen in mol N
c                 f(T) = temperature response function
c                 beta = ln Q10 / 10 
c                 Q10  = change in respiration rate with a 10 K change 
c                        in temperature
c                 T    = tissue absolute temperature in K

c     (B) Temperature response of soil respiration across ecosystems
c         incorporating damping of Q10 response due to temperature acclimation
c         (Lloyd & Taylor 1994, Eqn 11)  

c         (B1) R = R10 * g(T)
c         (B2) g(T) = EXP [308.56 * (1 / 56.02 - 1 / (T - 227.13))]

c           where R    = respiration rate
c                 R10  = respiration rate at 10 K
c                 g(T) = temperature response function at 10 deg C
c                 T    = soil absolute temperature in K

c     Mathematical derivation:

c     For a tissue with C:N mass ratio cton, and C mass, c_mass, N concentration
c     in mol given by
c       (1) N = c_mass / cton / atomic_mass_N
c     Tissue respiration in gC/day given by
c       (2) R = Rm * atomic_mass_C * seconds_per_day
c     From (A1), (1) and (2),
c       (3) R = 7.4e-7 * c_mass / cton / atomic_mass_N * atomic_mass_C
c             * seconds_per_day * f(T)
c     Let  
c       (4) k = 7.4e-7 * atomic_mass_C / atomic_mass_N * seconds_per_day
c             = 0.0548
c     from (3), (4)
c       (5) R = k * c_mass / cton * f(T)
c     substituting ecosystem temperature response function g(T) for f(T)
c     (Eqn B2),
c       (6) R = k * c_mass / cton * EXP [308.56 * (1 / 56.02 - 1 /
c             (T - 227.13))]	
c     incorporate PFT-specific respiration coefficient to model acclimation
c     of respiration rates to average (temperature) conditions for PFT (Ryan
c     1991)
c       (7) R_pft = respcoeff_pft * k * c_mass / cton
c             * EXP [308.56 * (1 / 56.02 - 1 / (T - 227.13))]



      do pft=1,npft
         if (present(pft)) then

            l_cton=pftpar(pft,13)
            s_cton=pftpar(pft,14)
            r_cton=pftpar(pft,15)
            
            respcoeff=pftpar(pft,5)
          
            anpp(pft)=0.0

            d=0                 !day of year
          
            do m=1,12

               lresp_m=0.0
               sresp_m=0.0
               rresp_m=0.0
            

               do dm=1,ndaymonth(m)

                  d=d+1         !next day of year

                  if (dtemp(d).ge.-40.0) then
 
                     gtemp_air=
     *                    exp(308.56*(1.0/56.02-1.0/(dtemp(d)+46.02)))

                     gtemp_soil=
     *                 exp(308.56*(1.0/56.02-1.0/(Tsoil_25(d)+46.02)))

                    ! Eqn (7)
                    ! 46.02 = -227.13 + 273.15; conversion from deg C to K

                  else
            
                     gtemp_air=0.0
                     gtemp_soil=0.0

                  endif

c             Calculate tissue maintenance respiration values today [Eqn (7)]
c             incorporating daily phenology

c     THIS IS USING THE NEW LEAF RESPIRATION METHOD

                  if (tree(pft)) then
                     sresp_d=respcoeff*k*sm_ind(pft)*nind(pft)/
     *                    s_cton*gtemp_air
                     rresp_d=respcoeff*k*rm_ind(pft)*nind(pft)/
     *                    r_cton*gtemp_soil*dphen(d,pft)
                     sresp_m=sresp_m+sresp_d
                     rresp_m=rresp_m+rresp_d

                  else if (grass(pft)) then !grass
                     rresp_d=respcoeff*k*rm_ind(pft)*nind(pft)/
     *                    r_cton*gtemp_soil*dphen(d,pft)

                     rresp_m=rresp_m+rresp_d
                  endif
               enddo            !day of month



c     THIS IS USING THE OLD LEAF RESPIRATION METHOD
c$$$                  if (tree(pft)) then
c$$$                     lresp_d=respcoeff*k*lm_ind(pft)*nind(pft)/ changed
c$$$     *                    l_cton*gtemp_air*dphen(d,pft) changed
c$$$                     sresp_d=respcoeff*k*sm_ind(pft)*nind(pft)/
c$$$     *                    s_cton*gtemp_air
c$$$                     rresp_d=respcoeff*k*rm_ind(pft)*nind(pft)/
c$$$     *                    r_cton*gtemp_soil*dphen(d,pft)
c$$$
c$$$                     lresp_m=lresp_m+lresp_d changed
c$$$                     sresp_m=sresp_m+sresp_d
c$$$                     rresp_m=rresp_m+rresp_d
c$$$
c$$$                  else if (grass(pft)) then grass
c$$$                     lresp_d=respcoeff*k*lm_ind(pft)*nind(pft)/ changed
c$$$     *                    l_cton*gtemp_air*dphen(d,pft) changed
c$$$                     rresp_d=respcoeff*k*rm_ind(pft)*nind(pft)/
c$$$     *                    r_cton*gtemp_soil*dphen(d,pft)
c$$$
c$$$                     lresp_m=lresp_m+lresp_d changed
c$$$                     rresp_m=rresp_m+rresp_d
c$$$                  else          moss !changed
c$$$                     lresp_d=respcoeff*k*lm_ind(pft)*nind(pft)/ changed
c$$$     *                    l_cton*gtemp_air*dphen(d,pft) changed
c$$$                     lresp_m=lresp_m+lresp_d changed
c$$$                  endif
c$$$               enddo            day of month

c     --------------------------------------------------------
c     Rita: find leaf N content of each pft. This is just for
c     information purposes.
c     --------------------------------------------------------
c     absolute N content in g
               nleaf(pft) = lm_ind(pft) * nind(pft) / l_cton
c     N content in %
               nleafper(pft) = 100 * nleaf(pft) / (lm_ind(pft) *
     $              nind(pft))

c     Incorporate growth respiration = 25% of
c     (GPP - maintenance respiration) in monthly NPP calculation
c     Mosses growth respiration = 50% of (GPP - Rm) (Yurova et al.
c     subm.)

c     THIS IS USING THE NEW LEAF RESPIRATION METHOD
               if (tree(pft)) then
                  mresp=mlresp(m,pft)+sresp_m+rresp_m
                  gresp_m=max((mgpp(m,pft)-mresp)*0.25,0.0)
                         !growth resp always non-negativ
               else if (grass(pft)) then !grass
                  mresp=mlresp(m,pft)+rresp_m
                  gresp_m=max((mgpp(m,pft)-mresp)*0.25,0.0)
                         !growth resp always non-negativ
               else             !moss
                  mresp=mlresp(m,pft)
                  gresp_m=max((mgpp(m,pft)-mresp)*0.50,0.0)
                         !growth resp always non-negativ
               endif


c     THIS IS USING THE OLD LEAF RESPIRATION METHOD
c$$$               if (tree(pft)) then
c$$$                  mresp=lresp_m+sresp_m+rresp_m
c$$$                  gresp_m=max((mgpp(m,pft)-mresp)*0.25,0.0)
c$$$                         !growth resp always non-negativ
c$$$               else if (grass(pft)) then !grass
c$$$                  mresp=lresp_m+rresp_m
c$$$                  gresp_m=max((mgpp(m,pft)-mresp)*0.25,0.0)
c$$$                         !growth resp always non-negativ
c$$$               else             !moss
c$$$                  mresp=lresp_m
c$$$                  gresp_m=max((mgpp(m,pft)-mresp)*0.50,0.0)
c$$$                         !growth resp always non-negativ
c$$$               endif

               mnpp(m,pft)=mgpp(m,pft)-mresp-gresp_m 


c     Avoid negative MNPP
c               IF (mnpp(m,pft) .LT. 0.0) mnpp(m,pft) = 0.0 

c     LPJ-WHy: CALCULATE THE EXUDATES POOL: The exudates are taken as a
c     fraction of mnpp. mnpp_red is the exudates-reduced mnpp. However,
c     mnpp is the variable which should be put out in order to get the
c     full npp and to calculate NEE/NEP. 
c     -----------------------------------------------------------------
               IF (mnpp(m,pft) .GT. 0) THEN
                  exudates(m) = exudates(m) + exu_frac * mnpp(m,pft)
                  mnpp_red(m,pft) = mnpp(m,pft) - exu_frac * mnpp(m,pft)
               ELSE
                  exudates(m) = 0.0
                  mnpp_red(m,pft) = mnpp(m,pft)
               ENDIF


c     Increment annual NPP

               anpp(pft) = anpp(pft) + mnpp(m,pft)
               anpp_red(pft) = anpp_red(pft) + mnpp_red(m,pft)


c     Copy NPP to bm_inc will store running total of
c     C production remaining for growth

               bm_inc(pft)=anpp_red(pft)
c               bm_inc(pft)=anpp(pft)
               sresp(m,pft)=sresp_m
               rresp(m,pft)=rresp_m
               gresp(m,pft)=gresp_m

c     THIS IS USING THE OLD LEAF RESPIRATION METHOD
c               aresp(m,pft)=lresp_m+sresp_m+rresp_m+gresp_m

c     THIS IS USING THE NEW LEAF RESPIRATION METHOD
               aresp(m,pft)=mlresp(m,pft)+sresp_m+rresp_m+gresp_m

            enddo               !month     
         endif !if pft present
c     Don't allow negative anpp. 
         if (anpp(pft).lt.0.0) then
            anpp(pft)=0.0
            do m=1,12  
               mnpp(m,pft)=0.0
            enddo               ! month
         endif 

      enddo                     !pft

      aaresp = 0.0
      
      DO pft = 1, 12
         xalresp(pft) = 0.0
      ENDDO

      DO m = 1, 12
         DO pft = 1, 12
            test(m) = test(m) + mnpp(m,pft)
            aaresp = aaresp + aresp(m,pft)
c     THIS IS USING THE OLD LEAF RESPIRATION METHOD
c            xalresp(pft) = xalresp(pft) + lresp(m,pft)
c     THIS IS USING THE NEW LEAF RESPIRATION METHOD
            xalresp(pft) = xalresp(pft) + mlresp(m,pft)
         ENDDO
      ENDDO

      lm_ind_PFT11 = lm_ind(11)
      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE REPRODUCTION
c     Deduction of reproduction costs from annual biomass increment

      subroutine reproduction(bm_inc,litter_ag,present
     $     ,litter_monthly)

      IMPLICIT NONE

      include "params.cb"
      include "vegvars.cb"

c     PARAMETERS
      real reprod_cost              !proportion of NPP lost to reproduction
        parameter (reprod_cost=0.1) !Harper 1977


c     LOCAL VARIABLES
      real reprod        !allocation to reproduction (gC/m2)

c     INITIALISATION

      do pft=1,npft
        if (present(pft)) then

c         Calculate allocation to reproduction
c         Reproduction costs taken simply as a constant fraction of annual NPP

          reprod=max(bm_inc(pft)*reprod_cost,0.0)

c         assume the costs go to reproductive structures which will
c         eventually enter the litter pool


          litter_ag(pft)=litter_ag(pft)+reprod

c         Reduce biomass increment by reproductive cost

          bm_inc(pft)=bm_inc(pft)-reprod

        endif

      enddo

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE TURNOVER
c     Turnover of PFT-specific fraction from each living C pool
c     Leaf and root C transferred to litter, sapwood C to heartwood

      subroutine turnover(pftpar,present,lm_ind,sm_ind, hm_ind,rm_ind
     $     ,litter_ag,litter_bg,nind,turnover_ind
     $     ,moss,litter_monthly)

      IMPLICIT NONE

      include "params.cb"
      include "vegvars.cb"


c     LOCAL VARIABLES:
      real l_torate             !leaf turn over rate (year-1)
      real s_torate             !sap wood turn over rate (year-1)
      real r_torate             !root turn over rate (year-1)
      real lm_turn
      real sm_turn
      real rm_turn

c     INITIALISATION


      do pft=1,npft

c       Turnover rates are reciprocals of tissue longevity

        l_torate=1.0/pftpar(pft,9)
        s_torate=1.0/pftpar(pft,11)
        r_torate=1.0/pftpar(pft,12)

        if (present(pft)) then

c         Calculate the biomass turnover in this year

          lm_turn=lm_ind(pft)*l_torate
          sm_turn=sm_ind(pft)*s_torate
          rm_turn=rm_ind(pft)*r_torate

c         Update the pools

          lm_ind(pft)=lm_ind(pft)-lm_turn
          sm_ind(pft)=sm_ind(pft)-sm_turn
          rm_ind(pft)=rm_ind(pft)-rm_turn 
          
c         Convert sapwood to heartwood

          hm_ind(pft)=hm_ind(pft)+sm_turn

          
c         Transfer to litter pools

          litter_ag(pft)=litter_ag(pft)+lm_turn*nind(pft)
          litter_bg(pft)=litter_bg(pft)+rm_turn*nind(pft)

          IF (moss(pft)) litter_bg(pft) = MISSING_VALUE

c         Record total turnover

          turnover_ind(pft)=lm_turn+sm_turn+rm_turn

        endif

      enddo  !pft

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE LITTERSOM
c     Litter and soil decomposition
c     Incorporates analytical solution for soil pool sizes once litter inputs
c     are (assumed to be) at equilibrium, reducing spin-up time for carbon
c     fluxes due to soil respiration.

      subroutine littersom(senspar, sens_test, peathydr, litter_ag
     $     ,litter_bg ,mw1,mtemp_soil,cpool_fast,cpool_slow, arh,mrh
     $     ,year,k_fast_ave,k_slow_ave,litter_decom_ave,moss ,exudates
     $     ,litter_monthly,litter_ag_old,litter_bg_old,exu_pool)

      IMPLICIT NONE 
      
      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"
      include "ch4vars.cb"

c     PARAMETERS
      REAL k_exu10              ! exudates decomposition rate at
c      PARAMETER (k_exu10 = 13)  !10 deg C (/year); I assumed a turnover
                                !time of 2 weeks, which gives 26 as
                                !turnover rate. 

      real k_litter10           !litter decomposition rate at
      parameter (k_litter10=0.35) !10 deg C (/year) k_litter10 changed
                                  !from 0.5 on 6/2000

      real k_soil_fast10        !fast pool decomposition rate at
      parameter (k_soil_fast10=0.03) !10 deg C (/year)

      real k_soil_slow10        !slow pool decomposition rate at
      parameter (k_soil_slow10=0.001) !10 deg C (/year)

      real fastfrac             !fraction of litter entering fast soil
      parameter (fastfrac=0.985) !decomposition pool

      real atmfrac              !fraction of litter decomposition
      parameter (atmfrac=0.7)   !going directly into the atmosphere

      real Rmoist               !moisture response under inundation
c      parameter (Rmoist=0.35)  

c     ARGUMENTS
      real k_fast_ave
      real k_slow_ave
      real litter_decom_ave
   
c     LOCAL VARIABLES 
      real k_exu                !monthly exudates decomposition rate (/month)
      real k_litter             !monthly litter decomposition rate (/month)
      real k_fast(1:12)         !monthly fast pool decomposition rate (/month)
      real k_slow(1:12)         !monthly slow pool decomposition rate (/month)
      real temp_resp            !monthly temperature response of decomposition
      real moist_resp           !monthly moisture response of decomposition
      real cflux_litter_soil    !litter decomposition flux to soil
      real cflux_litter_atmos   !litter decomposition flux to atmosphere
      real cflux_fast_atmos     !soil fast pool decomposition flux to atmosphere
      real cflux_slow_atmos     !soil slow pool decomposition flux to atmosphere
      real exu_decom            !decomposed exudates
      real litter_decom(1:12)   !monthly litter decomposition
      real litter_decom_ag      !above-ground component of litter decomposition
      real litter_decom_bg      !below-ground component of litter decomposition
      real litter_ag_new(npft)  !above-ground litter which has been
                                !added to the litter pool since the last
                                !call of the littersom SR
      real litter_bg_new(npft)  !below-ground litter which has been
                                !added to the litter pool since the last
                                !call of the littersom SR
      real litter_ag_old(npft)  !above-ground litter after this years'
                                !decomposition (gC/m2)
      real litter_bg_old(npft)  !below-ground litter after this years'
                                !decomposition (gC/m2)
      real soilfrac             !fraction of litter decomposition going
                                !to soil C pools
      real acro_to_cato         !transfer of C from the fast to the slow
                                !C pool which reflects the transfer from
                                !acrotelm to catotelm
      SAVE

      soilfrac=1.0-atmfrac
      arh=0.0

      IF (sens_test) THEN
         k_exu10 = senspar(3)
         Rmoist = senspar(4)
      ELSE
         k_exu10 = 13
         Rmoist = 0.4
      ENDIF

      DO m=1,12
         mrh(m) = 0.0 
      ENDDO
c     ------------------------------------------------------------------
c     LPJ-WHy: Calculate how much litter was added between the last call
c     of the Littersom SR and the end of Establishment SR.
c     ------------------------------------------------------------------

      DO pft = 1, npft
         litter_ag_new(pft) = litter_ag(pft) - litter_ag_old(pft)
         litter_bg_new(pft) = litter_bg(pft) - litter_bg_old(pft)
      ENDDO

      do m=1,12
         
c       Temperature response function is a modified Q10 relationship
c       (Lloyd & Taylor 1994)

        if (mtemp_soil(m).le.-40.0) then !avoid division by zero
c        if (mtemp_soil(m).lt.0.0) then !avoid division by zero TEST TEST
           temp_resp=0.0                                    
        else
           temp_resp=exp(308.56*((1.0/56.02)-
     *       (1.0/(mtemp_soil(m)+273.0-227.13))))  !Lloyd & Taylor 1994
        endif

c       ---------------------------------------------------------------------
c       Moisture response based on soil layer 1 moisture content (Foley 1995)
c        moist_resp=(0.25+(0.75*mw1(m)))

c       Rita: I don't know where the new Eqn below comes from. Compared
c       to Foley's eqn, it slows down the moisture response at low water
c       contents, which seems reasonable.
c       --------------------------------------------------------------------
        IF (peathydr) THEN
           moist_resp = Rmoist
        ELSE
           moist_resp=(1.0-exp(-1.0*mw1(m)))/(1.0-exp(-1.0))
        ENDIF

c       -----------------------------------------------------------------
c       Calculate monthly decomposition rates (k, /month) as a function
c       of temperature and moisture

c       k = k_10 * temp_resp * moist_resp
c       -----------------------------------------------------------------

        k_exu=(k_exu10*temp_resp*moist_resp)/12.0
        k_litter=(k_litter10*temp_resp*moist_resp)/12.0
        k_fast(m)=(k_soil_fast10*temp_resp*moist_resp)/12.0
        k_slow(m)=(k_soil_slow10*temp_resp*moist_resp)/12.0


c       ---------------------------------------------------------------------
c       Calculate monthly litter decomposition using equation  
c         (1) dc/dt = -kc     where c=pool size, t=time, k=decomposition rate
c       from (1),
c         (2) c = c0*exp(-kt) where c0=initial pool size
c       from (2), decomposition in any month given by
c         (3) delta_c = c0 - c0*exp(-k)
c       from (4)
c         (4) delta_c = c0*(1.0-exp(-k))
c       ---------------------------------------------------------------------

        litter_decom(m)=0.0

c       LPJ-WHy: Calculate the exudates
c       -------------------------------
        exu_pool = exu_pool + exudates(m)
        exu_decom = exu_pool * (1.0 - exp(-k_exu))
        exu_pool = exu_pool - exu_decom

c       LPJ-WHy: Increase litter pools by a monthly fraction
c       ----------------------------------------------------
        IF (litter_monthly) THEN
           DO pft=1,npft
              litter_ag(pft) = litter_ag_old(pft) + litter_ag_new(pft)
     $             / 12
              litter_bg(pft) = litter_bg_old(pft) + litter_bg_new(pft)
     $             / 12
           ENDDO        
        ENDIF

c       Note: mosses do not have a litter_bg part as that one consists
c       of roots, which mosses lack.
c       ---------------------------------------------------------------
        DO pft=1,npft
          litter_decom_ag=litter_ag(pft)*(1.0-exp(-k_litter))  !eqn 4
          IF (.not.moss(pft)) THEN
             litter_decom_bg=litter_bg(pft)*(1.0-exp(-k_litter))
          ELSE
             litter_decom_bg = 0.0
          ENDIF
          litter_decom(m)=litter_decom(m)+litter_decom_ag+
     *      litter_decom_bg

c     ERROR message
c     -------------
          IF (litter_ag(pft) .NE. litter_ag(pft).OR. litter_bg(pft) .NE.
     $         litter_bg(pft)) THEN
             print*, 'ERROR: litter_ag or litter_bg are NAN', pft,
     $            litter_ag(pft), litter_bg(pft)
          ENDIF

c         Update the litter pools
          litter_ag(pft)=litter_ag(pft)-litter_decom_ag
          litter_bg(pft)=litter_bg(pft)-litter_decom_bg
          IF (moss(pft)) litter_bg(pft) = MISSING_VALUE
          litter_ag_old(pft) = litter_ag(pft)
          litter_bg_old(pft) = litter_bg(pft)
        ENDDO
        litter_decom(m) = litter_decom(m)  + exu_decom

c       Calculate carbon flux to atmosphere and soil

        cflux_litter_atmos=atmfrac*litter_decom(m)
        cflux_litter_soil=soilfrac*litter_decom(m) 

c       Further subdivide soil fraction between fast and slow soil pools

        cpool_fast=cpool_fast+fastfrac*cflux_litter_soil
        
c       ----------------------------------------------------------------
c       PEATLAND C TRANSFER: In peatlands, C input happens at the top of
c       the peat column. The organic material gets compressed over time
c       as more material accumulates on the top. At some point, the
c       material reaches the boundary between the partially oxic,
c       partially anoxic layer (acrotelm) and the continuously anoxic
c       layer (catotelm).  Decomposition rates are much slower in the
c       catotelm then in the acrotelm because most of the quickly
c       decomposable material has been decomposed already and because of
c       the lack of oxygen.

c       Here, I assume that cpool_fast represents the acrotelm and
c       cpool_slow the catotelm.  To represent the process of C
c       transfer, a constant fraction of the cpool_fast is transferred
c       to the cpool_slow, once enough organic material has accumulated.
c       The value 7500 comes from assuming a bulk density of 0.05 g/cm3,
c       which is 0.025g C/cm3, which is 25kg C/m3 soil. The acrotelm is
c       0.3m thick, hence 25*0.3 is 7.5kg C is 7500g C/m2 in the
c       acrotelm.  Assume that a fraction of the acrotelm is transfered
c       to the catotelm through the growth of the acrotelm. See Clymo
c       1984.
c       ----------------------------------------------------------------
        IF (peathydr .AND. cpool_fast .GT. 7500) THEN
           acro_to_cato = 12 / 12    !12 g C/m2/yr Clymo 1984, p.617 (tunable)
        ELSE
           acro_to_cato = 0.0
        ENDIF
        cpool_fast = cpool_fast - acro_to_cato
        cpool_slow = cpool_slow + (1.0 - fastfrac) * cflux_litter_soil +
     $       acro_to_cato

c       Calculate monthly soil decomposition to the atmosphere

        cflux_fast_atmos=cpool_fast*(1.0-exp(-k_fast(m)))  !eqn 4
        cflux_slow_atmos=cpool_slow*(1.0-exp(-k_slow(m)))  !eqn 4

c       Update the soil pools

        cpool_fast=cpool_fast-cflux_fast_atmos
        cpool_slow=cpool_slow-cflux_slow_atmos   

c       Calculate monthly heterotrophic respiration

        mrh(m)=cflux_litter_atmos+cflux_fast_atmos+cflux_slow_atmos
        arh=arh+mrh(m)

c       Empty soil pools below a minimum threshold

        if (cpool_fast.lt.1.0e-5) cpool_fast=0.0
        if (cpool_slow.lt.1.0e-5) cpool_slow=0.0
      enddo   !month

c     SOIL DECOMPOSITION EQUILIBRIUM CALCULATION
c     Analytical solution of differential flux equations for fast and slow
c     soil carbon pools.  Implemented after (soil_equil_year) simulation
c     years, when annual litter inputs should be close to equilibrium.  Assumes
c     average climate (temperature and soil moisture) from all years up to
c     soil_equil_year.


      if (year.eq.soil_equil_year+1) then

c       Analytically calculate pool sizes this year only

c       Rate of change of soil pool size = litter input - decomposition
c         (5) dc/dt = litter_decom - kc
c       At equilibrium,
c         (6) dc/dt = 0
c       From (5) & (6),
c         (7) c = litter_decom / k

c       ---------------------------------------------------------------
c       In dry areas, soil moisture can be 0.0 in which case, the
c       moisture response is 0.0 and with it k_fast_ave and
c       k_slow_ave. That will lead to NaN for the cpools. Hence, the if
c       statement below which sets the cpools to zero of the
c       decomposition rates are 0.0.
c       ---------------------------------------------------------------

         IF (k_fast_ave .EQ. 0.0 .OR. k_slow_ave .EQ. 0.0) THEN
            cpool_fast = 0
            cpool_slow = 0
         ELSE
            cpool_fast=(soilfrac*fastfrac*litter_decom_ave)/k_fast_ave !eqn 7
            cpool_slow=(soilfrac*(1.0-fastfrac)*litter_decom_ave)
     *           /k_slow_ave    !eqn 7
         ENDIF
      else

        if (year.le.soil_equil_year) then

c         Update running average respiration rates and litter input
 
          do m=1,12
            k_fast_ave=k_fast_ave+k_fast(m)/real(soil_equil_year)
            k_slow_ave=k_slow_ave+k_slow(m)/real(soil_equil_year)
            litter_decom_ave=litter_decom_ave+litter_decom(m)/
     *        real(soil_equil_year)
          enddo

        endif

      endif

      return
      end


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************

      subroutine methane(year, soilpar, senspar, sens_test, soilcode,
     $     peathydr,anpp_red, drh, mrh,Tsoil , Fair, Fpwp, wtp,dphen,CH4
     $     , CH4_diss, CH4_diss_mol, CH4_gas , CO2_soil, O2 , CH4_diff
     $     ,CO2_diff, O2_diff ,CH4_plant, CH4_oxid, CO2_plant ,O2_plant,
     $     CH4_ebull ,CH4_prod, CO2_prod, lm_ind_PFT11 ,Total_water
     $     ,Fwater ,Fice ,mCH4_plant, mCH4_diff, mCH4_ebull ,frozen
     $     ,CH4_plant_out)

      IMPLICIT NONE

      include "params.cb"
      include "soilvars.cb"
      include "vegvars.cb"
      include "ch4vars.cb"

      !! PARAMETERS:

      REAL Dt                   ! time step [day]
      PARAMETER (Dt = 0.01)

      REAL CH4toCO2             ! CH4:CO2 ratio - a tuning parameter
c      PARAMETER (CH4toCO2 = 0.30)  !set below

      REAL rho_H20              !density of water [kg m-3]
      PARAMETER (rho_H20 = 1000.)

      REAL g                    ! acceleration due to gravity
      PARAMETER (g = 9.81)      ! [m s-2]

      REAL mr_CH4               !Molecular mass of CH4 [g mol-1]
      PARAMETER (mr_CH4 = 16)

      REAL mr_C                 !Molecular mass of carbon [g mol-1]
      PARAMETER (mr_C = 12)

      REAL mr_h2o               ! molecular weight of water
      PARAMETER (mr_h2o = 18)

      REAL CH4_atm              ! CH4 concentration in the atmosphere
      PARAMETER(CH4_atm = 1.774)! in [ppmv or micromol L-1] data from
                                ! 2005 (IPCC AR4, Ch.1)

      REAL CO2_atm              ! CO2 concentration in the atmosphere
      PARAMETER(CO2_atm = 379)  ! in [ppmv or micromol L-1] data from
                                ! 2005 (IPCC AR4, Ch.1)

      REAL D_CH4_air_const      ! diffusivity of CH4 in air at 5degC
      PARAMETER (D_CH4_air_const = 0.194) ! [cm2 s-1] (in Potter et al. 1996
                                    ! #002102)

      REAL acten_CH4            ! activation energy for diffusion in
                                ! water (Jaehne et al. 1987, #2149)
      PARAMETER (acten_CH4 = 18360) ! [J mol-1]

      REAL acten_CO2            ! activation energy for diffusion in
                                ! water (Jaehne et al. 1987, #2149)
      PARAMETER (acten_CO2 = 19510) ! [J mol-1]

      REAL A_CH4                ! constant in diffusion equation
      PARAMETER (A_CH4 = 0.03047) ! [cm2 s-1] (Jaehne et al. 1987, #2149)

      REAL A_CO2                ! constant in diffusion equation
      PARAMETER (A_CO2 = 0.05019) ! [cm2 s-1] (Jaehne et al. 1987, #2149)

      REAL R                    ! universal gas constant
      PARAMETER (R = 8.314472)  ! [J mol-1 K-1]

      REAL atm_press            ! standard atmospheric pressure (if DEM
                                ! is used in a later version, one can
                                ! correct for altitudinal effects)
      PARAMETER (atm_press = 101325) ! [Pa]

      REAL n                    ! coefficient for the calculation of the
                                ! gas transport velocity
      PARAMETER (n = -0.5)      ! given in Riera et al. 1999

      REAL U10                  ! wind speed at 10m height
      PARAMETER (U10 = 0)       ! [m s-1]
      
      REAL henry_k_CO2          ! Henry's constant at 298K (Sander's
      REAL henry_k_CH4          ! Henry's Law constant compilation)
      REAL henry_k_O2
      PARAMETER (henry_k_CO2 = 29.4)  ! [L atm mol-1]
      PARAMETER (henry_k_CH4 = 714.29)! [L atm mol-1]
      PARAMETER (henry_k_O2 = 769.23) ! [L atm mol-1]

      REAL const_CO2            ! Coefficients in Henry's law
      REAL const_CH4            ! (Sander 1999)
      REAL const_O2             ! 
      PARAMETER (const_CO2 = 2400.0)! [K]
      PARAMETER (const_CH4 = 1600.0)! [K]
      PARAMETER (const_O2 = 1500.0) ! [K]

      REAL pp_CO2               ! partial pressure of CO2 above water
      REAL pp_CH4               ! partial pressure of CH4 above water
      REAL pp_O2                ! partial pressure of O2 above water
      PARAMETER (pp_CO2 = 380.0)! [micro-atm] - change later to vary
      PARAMETER (pp_CH4 = 1.7)  ! with year
      PARAMETER (pp_O2 = 210000)   


      REAL bubble_init          ! bubble initialisation threshold
      PARAMETER (bubble_init = 0.05) ! = 5% of saturation (Baird et
                                     ! al. 2004 found bubble formation
                                     ! starting at 3.5% saturation)
      REAL vgc_low              ! when ebullition occurs, the volumetric
                                ! gas content (VGC) will drop to this
                                ! level
c      PARAMETER (vgc_low = 0.145) ! [unitless]

      REAL vgc_high             ! ebullition occurs, when the volumetric
                                ! gas content (VGC) exceeds this level
      PARAMETER (vgc_high = 0.15) ! [unitless]  

      REAL bubble_CH4_frac      ! CH4 fraction of gas bubbles (Kellner
                                ! et al. 2006)
      PARAMETER (bubble_CH4_frac = 0.57) ! [unitless]

      REAL oxid_frac            ! Fraction of oxygen used to oxidise CH4
c      PARAMETER (oxid_frac = 0.1)

      REAL ag_frac              ! Fraction of ANPP used to calculate number
c      PARAMETER (ag_frac = 0.5) ! of tillers

      REAL tiller_radius        ! Radius of an average tiller [m]
c      PARAMETER (tiller_radius = 0.0029) ! Schimel 1995. Average over
c      PARAMETER (tiller_radius = 0.004) ! Schimel 1995. Average over
                                         ! E. angustifolium (diam=7.9mm)
                                         ! and C. aquatilis(diam=3.8mm)

      REAL tiller_por           ! Tiller porosity
c      PARAMETER (tiller_por = 0.6) ! Wetland plants book, eds. Cronk and
c      PARAMETER (tiller_por = 0.5) ! Wetland plants book, eds. Cronk and
                                   ! Fennessy, p.90, values for 2
                                   ! Erioph. spp.

      REAL c_content            ! C content of biomass
      PARAMETER (c_content = 0.45) ! assumed value

      REAL cmass                ! atomic mass of carbon
      PARAMETER (cmass = 12)    ! [g/mol]

      REAL tiller_weight        ! Individual tiller weight
      PARAMETER (tiller_weight = 0.22) ! [g C]

      REAL water_min            ! a threshold factor for a minimum water
                                ! content in the layer [unitless]
      PARAMETER (water_min = 0.1) 
c      PARAMETER (water_min = 0.01) 

c     ARGUMENTS:
c     ----------
      REAL dphen(365,npft)      ! daily phenology


c     LOCAL VARIABLES:
c     ----------------
      LOGICAL NEWEBULL          ! Uses new way to calculate ebullition
                                ! based on maximum dissolved methane
      INTEGER i, j, d           ! indexes
      INTEGER IDX, WIDX         ! indexes
      INTEGER layer0            ! start layer for CN solver
      INTEGER ngroundl          ! number of soil layers
      INTEGER stand             ! indicator for standing water ON/OFF 

      REAL a, b, corr           ! parameters used in root biomass distr.
      REAL anoxic               ! degree of anoxia
      REAL C(NLAYERS)           ! concentration in [units?]
      REAL c_soil               ! total C concentration in soil
      REAL Ceq                  ! Equilibrium concentration of a gas
                                ! [mol L-1]
      REAL CH4_diss_max         ! maximal dissolved CH4 concentration
                                ! [ml CH4 mlH2O]
      REAL CH4_diss_max_m3      ! maximal dissolved CH4 concentration
                                ! [m3 layer-1]
      REAL CH4_diss_max_mol     ! maximal dissolved CH4 concentration
                                ! [mol layer-1]
      REAL CH4_diss_max_g       ! maximal dissolved CH4 concentration [g
                                ! CH4-C layer-1]
      REAL CH4_ebull_ind(NLAYERS) ! ...
                                ! [g CH4-C layer-1]
      REAL CH4_ebull_vol(NLAYERS) ! Volume of CH4 which bubbles out [m3]
      REAL CH4_flux(365)        ! daily total CH4 flux [g C m-2 d-1]
      REAL CH4_gas_vol(NLAYERS) ! gaseous CH4 volume in each layer [m3]
      REAL CH4_total            ! total CH4 content and annual flux
      REAL CH4_vgc(NLAYERS)     ! volumetric CH4 content [unitless]
      REAL CO2_cato             ! CO2 concentration in catotelm
      REAL CO2_flux(365)        ! daily total CO2 flux [g C m-2 d-1]
      REAL CO2_total            ! total CO2 content and annual flux

      REAL corr_factor          ! a correction factor to conserve C and
                                ! correct inaccuracies originating in
                                ! the CN solver
      REAL D_CH4(NLAYERS)       ! diffusivity of CH4 [cm2/s]
      REAL D_CO2(NLAYERS)       ! diffusivity of CO2 [cm2/s]
      REAL D_O2(NLAYERS)        ! diffusivity of O2 [cm2/s]
      REAL D_CH4_air(NLAYERS)   ! diffusivity of CH4 in air [cm2/s]
      REAL D_CO2_air(NLAYERS)   ! diffusivity of CO2 in air [cm2/s]
      REAL D_O2_air(NLAYERS)    ! diffusivity of O2 in air [cm2/s]
      REAL D_CH4_water(NLAYERS) ! diffusivity of CH4 in water [cm2/s]
      REAL D_CO2_water(NLAYERS) ! diffusivity of CO2 in water [cm2/s]
      REAL D_O2_water(NLAYERS)  ! diffusivity of O2 in water [cm2/s]
      REAL daycount             ! index for days within a month
      REAL delta_c_soil         ! change in soil C storage
      REAL depth                ! depth of soil layer (negative numbers)
                                ! [cm]
      REAL Di(NLAYERS)          ! diffusivity of  in each layer [cm2/s] -dm2/d
      REAL dnpp(365)            ! daily NPP [g C m-2 d-1]
      REAL difflux(NLAYERS)     ! net diffusive flux from anoxic
                                ! acrotelm layer (g CH4-C/m2/h)
      REAL drh(365)             ! daily C available for methanogens [gC
                                ! m-2 d-1]
      REAL Dz(NLAYERS)          ! layer thickness [mm]
      REAL flux_CH4             ! CH4 flux to the atmosphere
      REAL flux_CO2             ! CO2 flux to the atmosphere
      REAL flux_O2              ! O2 flux to the atmosphere
      REAL henry_coeff          ! Henry's T dependent coefficient
      REAL henry_k_cc_CH4       ! Henry's constant as dimensionless
                                ! ratio between the aqueous and the
                                ! gaseous phase
      REAL hydro_press          ! hydrostatic pressure caused by water
                                ! above layer [Pa]
      REAL kCH4                 ! gas transport velocity for CH4 [cm h-1]
      REAL kCO2                 ! gas transport velocity for CO2 [cm h-1]
      REAL kO2                  ! gas transport velocity for O2 [cm h-1]
      REAL k_600                ! gas transfer velocity of SF6 [cm h-1]
                                ! normalised to a Schmidt number of 600
      REAL plant_trans(NLAYERS) ! CH4 transport through plants [...]
      REAL plant_trans_gas(NLAYERS) ! CH4 transport through plants [...]
      REAL por(NLAYERS)         ! porosity of each layer
      REAL root(NLAYERS)        ! fraction of roots in each layer
      REAL rm(NLAYERS)          ! root biomass in [gC layer-1]
      REAL root_no(NLAYERS)     ! number of roots in each layer
      REAL root_length(NLAYERS) ! root length in each layer [cm]
      REAL root_area(NLAYERS)   ! root cross-sectional area [m2]
      REAL root_porous(NLAYERS) ! root porous cross-sectional area [m2]
      REAL roottrans            ! root fraction transferred to other
                                ! layer because a layer is frozen
      REAL ScCH4                ! Schmidt number for CH4
      REAL ScCO2                ! Schmidt number for CO2
      REAL ScO2                 ! Schmidt number for O2
      REAL surf_conc            ! gas concentration at surface [units?]
      REAL tillers              ! number of tillers
      REAL tiller_frac          ! fraction of tillersw eighted by root
                                ! biomass in each layer
      REAL tiller_area(NLAYERS) ! area of tillers corresponding to the
                                ! root fraction in each layer
      REAL waterheight          ! height of water at current layer
      REAL wcorrfac(NLAYERS)    ! water correction factor, takes account
                                ! of how much water is in liquid form
      REAL V_water(NLAYERS)     ! volumetric water content (liquid only)
                                ! not including water below permanent
                                ! wilting point - this is assumed to be
                                ! "inactive"
      REAL test
      REAL Ctest
      REAL Cnew
      SAVE

      !! INITIALISE
      !! -----------
      c_soil = 0.0
      CH4_total = 0.0
      CO2_total = 0.0
      flux_CO2 = 0.0
      flux_CH4 = 0.0
      flux_O2 = 0.0
      hydro_press = 0.0
      ngroundl = 0
      IDX = 0

      DO i = 1, NLAYERS
         Dz(i) = 0.0
         Di(i) = 0.0
         D_CH4(i) = 0.0
         D_CO2(i) = 0.0
         D_O2(i) = 0.0
         CH4_ebull_ind(i) = 0.0
         CH4_ebull_vol(i) = 0.0
      ENDDO

      DO d = 1, 365
         CH4_diff(d) = 0.0
         CH4_plant(d) = 0.0
         CH4_ebull(d) = 0.0
         CO2_diff(d) = 0.0
         CO2_plant(d) = 0.0
         dnpp(d) = 0.0
         O2_diff(d) = 0.0
         DO i = 1, NLAYERS
            CH4_diss(d,i) = 0.0
            CH4_diss_mol(d,i) = 0.0
            CH4_gas(d,i) = 0.0
            O2_plant(d,i) = 0.0
            CH4_plant_out(d,i) = 0.0
         ENDDO
      ENDDO

      DO m = 1, 12
         mCH4_plant(m) = 0.0
         mCH4_diff(m) = 0.0
         mCH4_ebull(m) = 0.0
      ENDDO

c     SET SOME OF THE PARAMETERS
c     --------------------------
      IF (sens_test) THEN
         oxid_frac = senspar(1)
         CH4toCO2 = senspar(2)
         tiller_radius = senspar(5)
         tiller_por = senspar(6)
c         vgc_low = senspar(8)
      ELSE
         oxid_frac = 0.5
         CH4toCO2 = 0.1
         tiller_radius = 0.003
         tiller_por = 0.7
      ENDIF
      vgc_low = 0.145 
      ag_frac   = 0.4


c     INTERPOLATE monthly heterotrophic respiration and NPP
c     -----------------------------------------------------
      call daily(mrh,drh)
      DO d = 1, 365
         drh(d) = drh(d) / 30 ! It would be nicer to divide by the exact
                              ! day in each month
      ENDDO


c ------------------------------------------------------------------
c
c     LAYER SETUP
c
c ------------------------------------------------------------------

      !! The number of soil layers vary between organic and
      !! non-organic soils
      !! ---------------------------------------------------
      IF (peathydr) THEN
         ngroundl = NACROTELM + NCATOTELM
      ELSE
         ngroundl = NSOILL
      ENDIF

      !! Set the index for the soil layers
      !! ---------------------------------
      IDX = NLAYERS - ngroundl + 1 

      !! --------------------------------------------------------
      !! Set porosity for each layer
      !! Subtract a constant gas fraction from the total porosity 
      !! --------------------------------------------------------

      DO i = IDX, NLAYERS - NCATOTELM
         por(i) = acrotelm_por - Fgas
      ENDDO
      DO i = IDX + NACROTELM, NLAYERS
         por(i) = soilpar(8) - Fgas
      ENDDO

      !! --------------
      !! START DAY LOOP
      !! --------------
      daycount = 0
      m = 1
      DO d = 1, 365
         IF (d .eq. 1 .and. year .eq. 11) THEN
            total_C_flux = 0.0
            c_input = 0.0
            DO i = 1, NLAYERS
               CO2_365(i) = 0.0
               CH4_365(i) = 0.0
               O2_365(i) = 0.0
               CH4_diss_365(i) = 0.0
            ENDDO
         ENDIF


         daycount = daycount + 1
         DO i = IDX, NLAYERS
            IF (d .eq. 1 .and. year .gt. 10) THEN
               CO2_soil(1,i) = CO2_365(i)
               CH4(1,i) = CH4_365(i)
               O2(1,i) = O2_365(i)
            ENDIF
         ENDDO
         

         !! ----------------------------------------------------------
         !! SET LAYER DEPTHS AND DIFFUSIVITIES
         !! Calculate the diffusivity of methane in air in dependence
         !! of temperature in each soil layer
         !! ----------------------------------------------------------
            
         !! Calculate the diffusivities for each gas and for water
         !! and air once and then allocate it as necessary
      
         !! FOR ALL LAYERS
         !! ---------------

         DO i = IDX, NLAYERS

            !! From Lerman (1979)
            D_CH4_air(i) =  0.1875 + 0.0013 * Tsoil(d,i)
            D_CO2_air(i) = 0.1325 + 0.0009 * Tsoil(d,i)
            D_O2_air(i) = 0.1759 + 0.00117 * Tsoil(d,i)


            !! Method 1 gives slightly different results from the one
            !! following (Method 2).
            !! Method 1:
c               D_CH4_water(i) = A_CH4 * exp(-acten_CH4 / (R * (273.15 +
c     $              T(WIDX))))
            !! Method 2 following Broecker and Peng (1974)
            !! Units are [cm2 s-1]
            D_CH4_water(i) = (0.9798 + 0.02986 * Tsoil(d,i) +
     $           0.0004381 *Tsoil(d,i) ** 2) * 1E-5
            D_CO2_water(i) = (0.939 + 0.02671 * Tsoil(d,i) +
     $           0.0004095 *Tsoil(d,i) ** 2) * 1E-5
            D_O2_water(i) = (1.172 + 0.03443 * Tsoil(d,i) + 0.0005048
     $           * Tsoil(d,i)** 2) * 1E-5
         ENDDO

c     Set fractions of soil parameters for each soil layer
c     ----------------------------------------------------
         j = 0
         DO i = IDX, NLAYERS
            j = j + 1
            Dz(i) = Dz_soil(j)
         ENDDO


         !! ACROTELM
         !! --------
         DO i = IDX, NLAYERS - NCATOTELM
            !! Use Millington and Quirk 1961 (MQ) Model tested in
            !! Iiyama and Hasegawa 2005.
            !! When Fair is less then 0.05, then use the diffusivity
            !! of water.  The cut-off of 0.05 is where Dwater is
            !! about equal D using the MQ model.

            IF (Fair(d,i) .GT. 0.05) THEN 
               D_CH4(i) = Fair(d,i) ** (10 / 3) / acrotelm_por ** 2 *
     $              D_CH4_air(i)
               D_CO2(i) = Fair(d,i) ** (10 / 3) / acrotelm_por ** 2 *
     $              D_CO2_air(i) ! find a way to make D_CO2_air
                                 ! dependent on T
               D_O2(i) = Fair(d,i) ** (10 / 3) / acrotelm_por ** 2 *
     $              D_O2_air(i)
            ELSE                ! if Fair .lt. 0.05, the slow
                                ! diffusivity of water dominates the
                                ! overall diffusivity
               D_CH4(i) = D_CH4_water(i)
               D_CO2(i) = D_CO2_water(i)
               D_O2(i) =  D_O2_water(i)

               !! STANDING WATER LAYER: add the water table position to
               !! the upper most soil depth
               !! -----------------------------------------------------
               IF (wtp(d) .gt. 0) THEN
                  Dz(IDX) = Dz(IDX) + wtp(d) 
               ENDIF
            ENDIF
         ENDDO 

         !! CATOTELM
         !! --------
         DO i = NLAYERS - NCATOTELM + 1, NLAYERS
            D_CH4(i) = D_CH4_water(i)
            D_CO2(i) = D_CO2_water(i)
            D_O2(i) =  D_O2_water(i)
         ENDDO
         
         !! Convert diffusivity from cm2 s-1 to m2 d-1
         !! s-1 * 60 * 60 * 24 = d-1
         !! cm2 / 10000 = m2
         !! Convert depths Dz(i) from mm to m
         !! -------------------------------------------
         DO i = IDX, NLAYERS
            D_CH4(i) = D_CH4(i) * 86400 / 10000
            D_CO2(i) = D_CO2(i) * 86400 / 10000
            D_O2(i) = D_O2(i) * 86400 / 10000
            Dz(i) = Dz(i) / 1000 
         ENDDO

         !! Calculate the volume of water and ice in [m3]
         !! (Fwater(d,i) + Fice(d,i)) * Dz(i) is in [m], but when *1m2
         !! you get [m3]
         !! ----------------------------------------------------------

         DO i = IDX, NLAYERS
            Total_water(d,i) = (Fwater(d,i) + Fice(d,i)) * Dz(i)
            ! It's not quite clear to me, what I should use here, either
            ! just the water fraction or the ice and water fraction?

c            V_water(i) =  Fwater(d,i) * Dz(i)

c ****************************
c           TESTING MODE:
            V_water(i) =  Total_water(d,i) * Dz(i)
c            V_water(i) =  0.82 * Dz(i)
c ****************************

            wcorrfac(i) = Fwater(d,i) / (Fwater(d,i) + Fice(d,i))
         ENDDO

c------------------------------------------------------------------------
c
c     METHANE and CO2 PRODUCTION
c
c------------------------------------------------------------------------

         !! ACROTELM
         !! Put an equivalent fraction of the C substrate in each
         !! acrotelm and the catotelm layer
         !! -----------------------------------------------------
         
         ! ROOT DISTRIBUTION
         ! -----------------
         !! The root distribution is taken from the combination of
         !! Angela Gallego-sala (pers. comm.) and Bernard & Fiala 1986
         !! data. The curve fitting
         !! was done in gnuplot.
         !! f(depth) = exp(depth/b) * corr_factor [unitless fraction]

         !! The data come from the top 1m of peat. I used the
         !! function to go down to 2m. Only the decay length (b) was
         !! derived from the data.  The corr_factor calculates then how
         !! much biomass is in each layer to add up to 100% within 2 m.
         !! NOTE: b and corr change if the layer thicknesses change!
         !! Currently, the values are valid for 8 soil layers with 100,
         !! 100, 100, 200, 200, 300, 500, 500mm depths.

         j = 1
         DO i = IDX, NLAYERS
            depth = soil_middepth(j) / 10 !convert from [mm] to [cm]
            b = 25.1695
            corr = 2.09624062
            root(i) = exp(depth / b) / corr
            j = j + 1
            IF (depth .LT. -200) root(i) = 0.0 
         ENDDO
c        If a layer is frozen, take the allocated root fraction and put
c        it in the next unfrozen layer above.

         test = 0
         roottrans = 0.0
         DO i =  NLAYERS, IDX, -1
            IF (frozen(i)) THEN
               roottrans = roottrans + root(i)
               root(i) = 0.0
            ELSE
               root(i) = root(i) + roottrans
               roottrans = 0.0
            ENDIF
            test = test + root(i)
         ENDDO


         DO i = IDX, NLAYERS

           !! Decide if ANAEROBIC CONDITIONS prevail, apply ON/OFF
           !! switch.  Instead of an ON/OFF switch, I could use a
           !! gradient, assuming that there will be enough anoxic
           !! microzones even when the layer is partially oxic.
           !! Work on the Fair(d,i) term so that it reads in the right
           !! layers
           !! ------------------------------------------------------

           anoxic = 1 - Fair(d,i) - Fgas

           !! Calculate how much CH4 is produced
           !! CH4_prod [g CH4-C layer-1 d-1]
           !! When there is more ice than water, don't allow CH4
           !! production
           !! --------------------------------------------------
           ! Limitation by Tsoil is a TEST and used for Inke Forbrich's paper
           IF (Fwater(d,i) .LT. water_min .OR. Tsoil(d,i) .LE. 0.5 ) 
     $          THEN
              CH4_prod(d,i) = 0.0
           ELSE
              CH4_prod(d,i) = drh(d) * root(i) * CH4toCO2 * anoxic
              ! c_input is used to check the C balance of the Methane SR
              c_input = c_input + drh(d) * root(i) * CH4toCO2 * anoxic
           ENDIF

           !! Add CH4 production to CH4 pool [g CH4-C layer-1]
           !! --------------------------------------------------------

           IF (d .EQ. 1) THEN
              CH4(d,i) = CH4_365(i) + CH4_prod(d,i)
           ELSE
              CH4(d,i) = CH4(d - 1,i) + CH4_prod(d,i)
           ENDIF

           !! ----------------------
           !! CO2 PRODUCTION
           !! [g CO2-C layer-1 d-1]
           !! ----------------------
           IF (Fwater(d,i) .LT. water_min ) THEN
              CO2_prod(d,i) = 0.0
           ELSE
              CO2_prod(d,i) = (drh(d) * root(i) - CH4_prod(d,i)) 
              c_input =c_input + (drh(d) * root(i) - CH4_prod(d,i)) 
           ENDIF

           !! CO2 CONCENTRATION
           !! [g CO2-C layer-1]
           !! ------------------  
           IF (d .EQ. 1) THEN
              CO2_soil(d,i) = CO2_365(i) + CO2_prod(d,i) 
           ELSE
              CO2_soil(d,i) = CO2_soil(d-1,i) + CO2_prod(d,i) 
           ENDIF
        ENDDO

c -----------------------------------------------------------------------
c
c     GAS TRANSPORT
c
c -----------------------------------------------------------------------

        !! -------------------
        !! DIFFUSION OF OXYGEN
        !! -------------------

        !! Caculate Schmidt number and gas transfer velocity in the
        !! top soil layer. 
        !! For another way to calculate k for CH4 look at Happell
        !! et al. (1995). They used -2/3 for n.
        !! ScO2 is derived from Wanninkhof 1992
        !! --------------------------------------------------------
        ScO2 = 1800.6 - 120.1 * Tsoil(d,IDX) + 3.7818 * Tsoil(d,IDX)
     $       ** 2 -0.047608 * Tsoil(d,IDX) ** 3
            
        k_600 = 2.07 + 0.215 * U10 ** 1.7
        
        kO2 = k_600 * (ScO2 / 600) ** n

        !! Go from cm h-1 to m d-1
        kO2 =kO2 / 100 * 24

        !! Calculate concentration of O2 in equilibrium with the
        !! atmosphere using Henry's law
        !! Henry's constant/coefficient depends on T
        !! UNITS: henry_k [L atm mol-1], const_O2 [K], T [K]
        !! henry_coeff [L atm mol-1]
        !! -------------------------------------------------------
        henry_coeff = henry_k_O2 * exp(-const_O2 * (1.0 / (273.15
     $       + Tsoil(d,IDX)) - 1.0 / 298.15))
 
        !! pp_O2 [micro-atm] / 1E6 to get [atm]
        !! Ceq is in [mol C L-1] 
        !! -------------------------------------
        Ceq = pp_O2 / 1E6 / henry_coeff

        !! Convert Ceq from [mol C L-1] to [mmol C m-3]
        !! * 1000 to get to [mmol]
        !! * 1000 to get from [L-1] to [m-3]
        !! --------------------------------------------
        Ceq = Ceq * 1E6

        !! convert concentration from [mol O2 layer-1] to [mmol m-3]
        !! / V_water to get per m-3
        !! * 1000 to get mmol
        !! * wcorrfac(i) to use only the fraction in liquid water
        !! ------------------------------------------------------

        DO i = IDX, NLAYERS
           C(i) = O2(d,i) / V_water(i) * 1000
           Di(i) = D_O2(i)
        ENDDO

        !! -------------------------------------------------
        !! Set the boundary condition on the top of the soil
        !! -------------------------------------------------

        !! F = k (Cm - Ce) in 
        !! UNITS: k [m d-1], surf_conc and Ceq [mmol m-3]
        !! [mmol m-3] multiplied by [m d-1] gives
        !! flux units of [mmol m-2 d-1]
        !!
        !! Here, an analytical solution is used to calculate in one step
        !! how much gas diffuses in or out from a layer within 1 day.
        !! C(t+1) = Ceq + (C(t) - Ceq) * e(-k/dz)
        !! dz is replaced by V_water[m3]/Area[m2], where Area is 1m2
        !! --------------------------------------------------------------

        !! ---------------------------------------------------------------
        !! If the water content is smaller than a threshold (water_min)
        !! then do not allow oxygen diffusion into the top layer.
        !! ---------------------------------------------------------------

        IF (Fwater(d,IDX) .LT. water_min) THEN
           Cnew = C(IDX)
           O2_diff(d) = 0.0
        ELSE
           Cnew = Ceq + (C(IDX) - Ceq) * exp(-kO2 / V_water(IDX)) ![mmol m-3]
           O2_diff(d) = (C(IDX) - Cnew) * V_water(IDX) /  1000 ![mol
                                                               !layer-1
                                                               !d-1]
        ENDIF
        C(IDX) = Cnew ! [mmol m-3]

        ! The surface concentration equals the concentration in the top
        ! layer
        ! -------------------------------------------------------------

        layer0 = IDX
        surf_conc = C(IDX)
        !! ----------------------------------------------------------
        !! Do one timestep of the Crank-Nicholson method.
        !! The CN solver is an approximation and doesn't give exact
        !! results in terms of conservation, i.e. when having a
        !! total C content summed over all layers of e.g. 248.149216
        !! before calling cnstepgas, I end up with a total C content of
        !! 248.149368 after calling it.
        !! ----------------------------------------------------------
        DO i = IDX, NLAYERS
           IF (Fwater(d,i) .LT. water_min) THEN
              Di(i) = 1e-9
           ENDIF
        ENDDO

        CALL cnstepgas(layer0, C, Di, Dz, surf_conc,Dt)

        !! Allocate concentrations C to O2 
        !! units [mol layer-1]
        DO i = IDX, NLAYERS
           O2(d,i) = C(i) * V_water(i) / 1000
        ENDDO

c                           END OF O2 DIFFUSION
c -----------------------------------------------------------------------

        !! -------------------------
        !! PLANT TRANSPORT OF OXYGEN
        !! -------------------------

        !! TILLER SETUP for all gases
        !! --------------------------
        tillers = lm_ind_PFT11 * dphen(d,11) / tiller_weight
      
        DO i = IDX, NLAYERS 
           tiller_frac = tillers  * root(i) 
           tiller_area(i) = MAX(0.0001, tiller_radius ** 2 * pi *
     $          tiller_frac * tiller_por) 
        ENDDO

        DO i = IDX, NLAYERS 

           !! convert concentration from [mol layer-1] to [mmol m-3]
           !! / V_water to get per m-3
           !! * 1000 to get mmol
           !! ------------------------------------------------------

           !! Weight plant transport by the area of porous root
           !! cross-sections. plant_trans [mmol layer-1 d-1]
           !! root_porous is in m2 but the units disappear because it
           !! is practically divided by 1m2 to get a unitless
           !! fraction.


           IF (Fwater(d,i) .GT. water_min) THEN
              O2(d,i) = O2(d,i) / V_water(i) * 1000
              Cnew = Ceq + (O2(d,i) - Ceq) * exp(-kO2 / (V_water(i)
     $             / tiller_area(i)))
              O2_plant(d,i) = (O2(d,i) - Cnew)  * V_water(i) /
     $             1000 
              O2(d,i) = Cnew * V_water(i) / 1000
           ENDIF
        ENDDO

c                       END OF O2 PLANT TRANSPORT
c -----------------------------------------------------------------------

        !! --------------------
        !! DIFFUSION OF METHANE 
        !! --------------------

        !! Caculate Schmidt number and gas transfer velocity in the
        !! top soil layer.
        !! For another way to calculate k for CH4 look at Happell
        !! et al. (1995). They used -2/3 for n.
        !! --------------------------------------------------------
        ScCH4 = 1898 - 110.1 * Tsoil(d,IDX) + 2.834 * Tsoil(d,IDX)**2
     $       -0.02791 * Tsoil(d,IDX)**3
        
        k_600 = 2.07 + 0.215 * U10 ** 1.7

        kCH4 = k_600 * (ScCH4 / 600) ** n

        !! Go from cm h-1 to m d-1
        kCH4 =kCH4 / 100 * 24

        !! Calculate concentration of CH4 in equilibrium with the
        !! atmosphere using Henry's law
        !! Henry's constant/coefficient depends on T
        !! UNITS: henry_k [L atm mol-1], const_CH4 [K], T [K]
        !! henry_coeff [L atm mol-1]
        !! -------------------------------------------------------
        henry_coeff = henry_k_CH4 * exp(-const_CH4 * (1.0 / (273.15
     $       + Tsoil(d,IDX)) - 1.0 / 298.15))
 
        !! pp_CH4 [micro-atm] / 1E6 to get [atm]
        !! Ceq is in [mol C L-1] 
        !! ---------------------------------------
        Ceq = pp_CH4 / 1E6 / henry_coeff

        !! Convert Ceq from [mol C L-1] to [mmol m-3]
        !! * 1000 to get to [mmol]
        !! * 1000 to get from [L-1] to [m-3]
        !! Typical values for Ceq are around 3.6nmol/L
        !! --------------------------------------------
        Ceq = Ceq * 1E6

        !! convert concentration from [g C layer-1] to [mmol m-3]
        !! / cmass to get mol
        !! / V_water to get per m-3
        !! * 1000 to get mmol
        !! ------------------------------------------------------

        DO i = IDX, NLAYERS
           C(i) = CH4(d,i) / cmass / V_water(i) * 1000
           Di(i) = D_CH4(i) 
        ENDDO

        !! Set the boundary condition on the top of the soil
        !! -------------------------------------------------
        layer0 = IDX
        
        !! When the water table rises above the surface, then the CH4
        !! concentration in the surface layer is suddenly very low as it
        !! didn't have a concentration allocated. Therefore, I'll add
        !! the water volume of the standing water to the top layer and
        !! spread the methane concentration over 2 layers now. This will
        !! mean that you'll see a drop in CH4 concentration in the top
        !! soil layer when the wtp goes above the surface.
        surf_conc = C(IDX)

        !! ----------------------------------------------------------
        !! Do one timestep of the Crank-Nicholson method.
        !! The CN solver is an approximation and doesn't give exact
        !! results in terms of conservation, i.e. when having a
        !! total C content summed over all layers of e.g. 248.149216
        !! before calling cnstepgas, I end up with a total C content of
        !! 248.149368 after calling it.
        !! ----------------------------------------------------------
        DO i = IDX, NLAYERS
           IF (Fwater(d,i) .LT. water_min) THEN
              Di(i) = 1e-9
           ENDIF
        ENDDO

        CALL cnstepgas(layer0, C, Di, Dz, surf_conc,Dt)

        DO i = IDX, NLAYERS
           CH4(d,i) = C(i) * cmass * V_water(i) / 1000 
           test = test + CH4(d,i)+ CO2_soil(d,i)
        ENDDO

c                           END OF CH4 DIFFUSION
c -----------------------------------------------------------------------

        !! -------------
        !! CH4 OXIDATION
        !! -------------
        !! Assume that 1/2 of the O2 is utilized by other electron
        !! acceptors and you need 2 moles of O2 to oxidise 1 mole of
        !! CH4.
        test = 0
        DO i = IDX, NLAYERS
           ! O2 is in [mol layer-1]
           O2(d,i) = 0.5 * O2(d,i) ! 2 moles of O2 per 1 mol of CH4 are needed
           ! convert CH4 from [g C layer-1] to [mol layer-1]
           CH4(d,i) = CH4(d,i) / cmass
           CH4_oxid(d,i) = MIN(CH4(d,i), oxid_frac * O2(d,i))
           CH4(d,i) = (CH4(d,i) - CH4_oxid(d,i)) * cmass
           O2(d,i) = O2(d,i) - CH4_oxid(d,i)
           CO2_soil(d,i) = CO2_soil(d,i) + CH4_oxid(d,i) * cmass
           test = test + CH4(d,i)+ CO2_soil(d,i)
        ENDDO

        C(IDX) = CH4(d,IDX) / cmass / V_water(IDX) * 1000 

        !! -------------------------------------------------------------
        !! F = k (Cm - Ce) in 
        !! UNITS: k [m d-1], surf_conc and Ceq [mmol m-3]
        !! [mmol m-3] multiplied by [m d-1] gives
        !! flux_CH4 units of [mmol m-2 d-1]

        !! Here, an analytical solution is used to calculate in one step
        !! how much gas diffuses in or out from a layer within 1 day.
        !! C(t+1) = Ceq + (C(t) - Ceq) * e(-k/dz)
        !! dz is replaced by V_water[m3]/Area[m2], where Area is 1m2
        !! -------------------------------------------------------------

        IF (Fwater(d,IDX) .GT. water_min) THEN
           Cnew = Ceq + (C(IDX) - Ceq) * exp(-kCH4 /V_water(IDX)) !![mmol m-3]
           CH4_diff(d) = (C(IDX) - Cnew) * cmass * V_water(IDX) /
     $          1000 ![g layer-1 d-1]
           CH4(d,IDX) = Cnew * cmass * V_water(IDX) / 1000 
        ELSE
           Cnew = C(IDX)
           CH4_diff(d) = 0
           CH4(d,IDX) = Cnew * cmass * V_water(IDX) / 1000 
        ENDIF

c                           END OF CH4 OXIDATION
c -----------------------------------------------------------------------

        !! ----------------------
        !! PLANT TRANSPORT OF CH4
        !! ----------------------

        !! Root biomass distribtution of the flood-tolerant
        !! graminoids
        !! ------------------------------------------------

        ! use the dphen(d,pft) to have a phenology in the roots as
        ! they'll have to be built up in spring and plant transport
        ! will depend on existence of stems and leaves.

        ! I think that the limiting factor to plant transport are
        ! the number of tillers as they represent the
        ! cross-sectional area available to gas transport. I relate
        ! the biomass to tiller density based on Schimel 1995. The
        ! average biomass was 185g/m2 and the average number of
        ! tillers was 380/m2. This gives us 2.05 tillers per g
        ! biomass and 0.48 g per tiller, which corresponds to 0.22 g
        ! C per tiller

        ! add explanation: k_600 is fixed as U10 is assumed to be zero.
        k_600 = 2.07 

        kCH4 = k_600 * (ScCH4 / 600) ** n

        !! Go from cm h-1 to m d-1
        kCH4 =kCH4 / 100 * 24
        test = 0
        DO i = IDX, NLAYERS 
           !! convert concentration from [g C layer-1] to [mmol m-3]
           !! / cmass to get mol
           !! / V_water to get per m-3
           !! * 1000 to get mmol
           !! ------------------------------------------------------
           CH4(d,i) = CH4(d,i) / cmass / V_water(i) * 1000 ![mmol m-3]

           !! Weight plant transport by the area of porous root
           !! cross-sections. plant_trans(i) [mmol layer-1 d-1]
           ! the 2.07 are from the equation above, when U10 is zero.

           IF (Fwater(d,i) .GT. water_min) THEN
              Cnew = Ceq + (CH4(d,i) - Ceq) * exp(-kCH4 / (V_water(i)
     $             / tiller_area(i)))
              CH4_plant(d) = CH4_plant(d) + (CH4(d,i) - Cnew) * cmass *
     $              V_water(i) / 1000 
              CH4_plant_out(d,i) = (CH4(d,i) - Cnew) * cmass *
     $             V_water(i) / 1000 
              CH4(d,i) = Cnew * cmass * V_water(i) / 1000 ![g CH4-C layer-1]
           ELSE
              Cnew = CH4(d,i)
              CH4_plant(d) = CH4_plant(d) + 0
              CH4(d,i) = Cnew * cmass * V_water(i) / 1000
           ENDIF
              test = test + CH4(d,i) + CO2_soil(d,i)
        ENDDO

c                        END OF CH4 PLANT TRANSPORT
c -----------------------------------------------------------------------
        !! -----------------
        !! EBULLITION OF CH4
        !! -----------------
        
        !! SOLUBILITY OF CH4
        !! -----------------
        !! Estimate how much methane can be dissolved at current T
        !! from Yamamoto et al. 1976. Yamamoto's relationship
        !! holds only up to about 37 degrees, after which the 
        !! dissolubility increases again when using the eqn below.
        !! This is in ml CH4 per ml H20. 
        !! ---------------------------------------------------------
        waterheight = 0.0
        test = 0

        DO i = IDX, NLAYERS
           CH4_diss_max =  0.05708 - 0.001545 * max(0.0, Tsoil(d,i)) +
     $          0.00002069 * (max(0.0, Tsoil(d,i)) ** 2) ![ml CH4 dissolved ml-1 H2O]

           !! Calculate hydrostatic pressure
           !! ------------------------------
           
           waterheight = waterheight + Total_water(d,i) ![m]
           hydro_press = rho_H20 * g * waterheight ![kg m s-2 or N]

           !! [mL CH4 mL-1 H2O] is equivalent to [m3 CH4 m-3 H2O in soil] 
           !! To calculate the volume of CH4 per layer, multiply by the
           !! the layer's water content V_water.
           !! -----------------------------------------------------------

c           CH4_diss_max_m3 = CH4_diss_max * V_water(i) ![m3 CH4 layer-1]
           CH4_diss_max_m3 = CH4_diss_max * Total_water(d,i) ![m3 CH4 layer-1]

           !! Use the Ideal Gas Law to convert [m3 CH4 layer-1] to
           !! [mol CH4 layer-1]
           !! Add hydrostatic pressure to atmospheric pressure.
           !! atm_press is in [Pa], which is [N m-2], so hydro_press
           !! has to be divided by 1m2 to have the units [N m-2] or [Pa]
           !! ----------------------------------------------------------

           CH4_diss_max_mol = CH4_diss_max_m3 * (atm_press +
     $          hydro_press) / (R * (Tsoil(d,i) + 273.15)) ![mol CH4 layer-1]

           !! * molecular weight of C
           !! CH4_diss_max [g CH4-C layer-1]
           !! -----------------------------------------------------
           CH4_diss_max_g = CH4_diss_max_mol* mr_C ![g CH4-C layer-1] 

           !! If the total amount of CH4 is higher than what can be
           !! dissolved in the given water volume times the
           !! threshold for bubble initiation, then increase
           !! the gaseous CH4 pool.
           !! Units for concentrations are in [g CH4-C layer-1]
           !! Units for volume is in [m3]
           !! ------------------------------------------------------
           
           test = test + CH4(d,i) + CO2_soil(d,i)

           CH4_diss(d,i) = MIN(CH4(d,i), CH4_diss_max_g) ![g CH4-C layer-1]
           CH4_diss_mol(d,i) = CH4_diss(d,i) / Total_water(d,i) / mr_C
     $          / 1000          ![mol CH4-C L-1]
c           CH4_diss_mol(d,i) = CH4_diss(d,i) / V_water(i) / mr_C
c     $          / 1000          ![mol CH4-C L-1]

! Test the sensitivity of ebullition to the amount of max. dissolved CH4
! If you reduce CH4_diss, more will be gaseous.
!           CH4_diss(d,i) = CH4_diss(d,i) * 0.5
           CH4_gas(d,i) = CH4(d,i) - CH4_diss(d,i) ![g CH4-C m-2]

           NEWEBULL = .TRUE.

           IF (NEWEBULL) THEN
              CH4_ebull_ind(i) = CH4_gas(d,i) ! [g CH4-C layer-1 d-1] 
              CH4_ebull(d) = CH4_ebull(d) + CH4_ebull_ind(i) ! [g CH4-C m-2 d-1] 
              CH4_gas(d,i) = 0.0
              CH4(d,i) = CH4_diss(d,i)

           ELSE


c -----------------------------------------------------------------------
c                         OLD WAY TO CALCULATE EBULLITION
c -----------------------------------------------------------------------

c$$$
              IF (Fwater(d,i) .GT. water_min) THEN
              !! ----------------------------------------------------------
              !! kHcc = ca/cg from Sander's compilation -> ca = kHcc * cg
              !! kHcc = T * kH [mol/atm]/ 12.2 
              !! henry_coeff is 1/kH, hence you get the following equation:
              !! kHcc = T * (1/henry_coeff) / 12.2 = T / (henry_coeff * 12.2)

              !! Note on henry_k_CH4 = 1/kH: I defined henry_k_CH4 to be
              !! in the units of [L atm mol-1] so that henry_coeff would
              !! be in the same units. This was by choice. You could
              !! also use henry_k_CH4 in units of Mol/atm and then
              !! change the calculation of Ceq above. 

              !! Note: this approach will lead to a slight overestimation
              !! of the dissolved CH4, as I would need a stepwise
              !! approximation of how much diss. CH4 is in equilibrium
              !! with the gas above.
              !! ----------------------------------------------------------
c$$$              henry_k_cc_CH4 = (273.15 + Tsoil(d,i)) / (henry_coeff *
c$$$     $             12.2) 
c$$$              CH4_diss(d,i) = MIN(CH4_diss_max_g, henry_k_cc_CH4 *
c$$$  $             CH4(d,i))
c$$$              CH4_gas(d,i) = CH4(d,i) - CH4_diss(d,i)

c -----------------------------------------------------------------------
c                   END OF OLD WAY TO CALCULATE EBULLITION
c -----------------------------------------------------------------------

                 CH4_gas_vol(i) = CH4_gas(d,i) / cmass * R * (273.15 +
     $                Tsoil(d,i))/ (atm_press + hydro_press) ! [m3]
                 CH4_vgc(i) = CH4_gas_vol(i) / (Dz(i) * 1) ! [m3/m3]

              !! -------------------------------------------------------   
              !! Kellner et al. 2006 gave the average CH4 content in
              !! gas bubbles at 57 percent. Hence I lower the
              !! threshold above which gas will bubble out from vgc_high
              !! to vgc_high * 0.57
              !! IF the ebullitive flux is too small, you can increase
              !! the gap between the vgc_low and vgc_high limits.
              !! -------------------------------------------------------   

                 IF (CH4_vgc(i) .GT. vgc_high * bubble_CH4_frac) THEN
                    CH4_ebull_vol(i) = (CH4_vgc(i) - vgc_low *
     $                   bubble_CH4_frac) * Dz(i) * 1 
                 
              ! ----------------------------------------------------
              ! Let VGC drop down to a lower boundary, so that it'll
              ! take a few days, before more ebullition can
              ! occur.
              ! ----------------------------------------------------
                    CH4_vgc(i) = vgc_low * bubble_CH4_frac ! [unitless]

                    CH4_ebull_ind(i) = CH4_ebull_vol(i) * (atm_press +
     $                   hydro_press) / (R * (273.15 + Tsoil(d,i))) ! [mol]
                    CH4_gas(d,i) = CH4_vgc(i) * Dz(i) * 1 * (atm_press +
     $                   hydro_press) / (R * (273.15 + Tsoil(d,i))) ! [mol] 
                    CH4_ebull_ind(i) = CH4_ebull_ind(i) * cmass ! mol to g
                    CH4_ebull(d) = CH4_ebull(d) + CH4_ebull_ind(i) ! [g m-2 d-1] 
                    CH4_gas(d,i) = CH4_gas(d,i) * cmass ! mol to g
                 ENDIF
                 CH4(d,i) = CH4_diss(d,i) + CH4_gas(d,i)
              ELSE
                 IF (d .EQ. 1) THEN
                    CH4_diss(d,i) = CH4_diss_365(i)
                 ELSE
                    CH4_diss(d,i) = CH4_diss(d-1, i)
                 ENDIF
              ENDIF


           ENDIF

        ENDDO

c                           END OF CH4 EBULLITION
c -----------------------------------------------------------------------

        !! ----------------
        !! DIFFUSION OF CO2
        !! ----------------

        !! Calculate Schmidt number and gas transfer velocity in the
        !! layer below the air layer, i.e. either standing water or
        !! the top soil layer (= IDX)
        !! For another way to calculate k for CH4 look at Happell
        !! et al. (1995). They used -2/3 for n.
        !! --------------------------------------------------------
        ScCO2 = 1911 - 113.7 * Tsoil(d,IDX) + 2.967 * Tsoil(d,IDX) **
     $       2 -0.02943 * Tsoil(d,IDX) ** 3
        k_600 = 2.07 + 0.215 * U10 ** 1.7
        kCO2 = k_600 * (ScCO2 / 600) ** n
        !! Go from cm h-1 to m d-1
        kCO2 =kCO2 / 100 * 24

        !! Calculate concentration of CO2 in equilibrium with the
        !! atmosphere using Henry's law
        !! Henry's constant/coefficient depends on T
        !! UNITS: henry_k [L atm mol-1], const_CO2 [K], T [K]
        !! henry_coeff [L atm mol-1]
        !! -------------------------------------------------------
        henry_coeff = henry_k_CO2 * exp(-const_CO2 * (1.0 / (273.15
     $       + Tsoil(d,IDX)) - 1.0 / 298.15))
 
        !! pp_CO2 [micro-atm] / 1E6 to get [atm]
        !! Ceq is in [mol C L-1] 
        !! -------------------------------------
        Ceq = pp_CO2 / 1E6 / henry_coeff

        !! Convert Ceq from [mol C L-1] to [mmol C m-3]
        !! * 1000 to get to [mmol]
        !! * 1000 to get from [L-1] to [m-3]
        !! --------------------------------------------
        Ceq = Ceq * 1E6

        !! convert concentration from [g C layer-1] to [mmol m-3]
        !! / cmass to get mol
        !! / V_water to get per m-3
        !! * 1000 to get mmol
        !! ------------------------------------------------------

        DO i = IDX, NLAYERS
           C(i) = CO2_soil(d,i) / cmass / V_water(i) * 1000
           Di(i) = D_CO2(i) 
        ENDDO

        !! Set the boundary condition on the top of the soil
        !! -------------------------------------------------
        surf_conc = C(IDX)

        !! ----------------------------------------------------------
        !! Do one timestep of the Crank-Nicholson method.
        !! The CN solver is an approximation and doesn't give exact
        !! results in terms of conservation, i.e. when having a
        !! total C content summed over all layers of e.g. 248.149216
        !! before calling cnstep, I end up with a total C content of
        !! 248.149368 after calling it.
        !! ----------------------------------------------------------

        DO i = IDX, NLAYERS
           IF (Fwater(d,i) .LT. water_min) THEN
              Di(i) = 1e-9
           ENDIF
        ENDDO

        CALL cnstepgas(layer0, C, Di, Dz, surf_conc,Dt)


        !! F = k (Cm - Ce) in 
        !! UNITS: k [m d-1], surf_conc and Ceq [mmol m-3]
        !! [mmol m-3] multiplied by [m d-1] gives
        !! flux units of [mmol m-2 d-1]
        !! ----------------------------------------------

        IF (Fwater(d,IDX) .GT. water_min) THEN
           Ctest = Ceq + (C(IDX) - Ceq) * exp(-kCO2 /V_water(IDX)) ![mmol
                                                                         !m-3]
           CO2_diff(d) = (C(IDX) - Ctest) * cmass * V_water(IDX) /
     $          1000  ![g C m-2 d-1]
        ELSE
           Ctest = C(IDX)
           CO2_diff(d) = 0
        ENDIF
        C(IDX) = Ctest


        !! Reallocate concentrations to CO2_soil(i)
        !! * cmass to get from [mmol C] to [mg C]
        !! / 1000 to get from [mg] to [g]
        !! * V_water to get from [m-3] to [layer-1]
        !! 
        !! ----------------------------------------
        test = 0
        DO i = IDX, NLAYERS
           CO2_soil(d,i) = C(i) * cmass / 1000 * V_water(i) 
           test = test + CH4(d,i)+CO2_soil(d,i)!+CH4_gas(d,i)
        ENDDO 

c                           END OF CO2 DIFFUSION
c -----------------------------------------------------------------------
        !! ----------------------
        !! PLANT TRANSPORT OF CO2
        !! ----------------------

        !! Root biomass distribtution of the flood-tolerant
        !! graminoids
        !! ------------------------------------------------


        ! use the dphen(d,pft) to have a phenology in the roots as
        ! they'll have to be built up in spring and plant transport
        ! will depend on existence of stems and leaves.

        ! I think that the limiting factor to plant transport are
        ! the number of tillers as they represent the
        ! cross-sectional area available to gas transport. I relate
        ! the biomass to tiller density based on Schimel 1995. The
        ! average biomass was 185g/m2 and the average number of
        ! tillers was 380/m2. This gives us 2.05 tillers per g
        ! biomass and 0.48 g per tiller, which corresponds to 0.22 g
        ! C per tiller
        CO2_plant(d) = 0.0
        test = 0
        DO i = IDX, NLAYERS 
           !! convert concentration from [g C layer-1] to [mmol m-3]
           !! / cmass to get mol
           !! / V_water to get per m-3
           !! * 1000 to get mmol
           !! ------------------------------------------------------
           CO2_soil(d,i) = CO2_soil(d,i) / cmass / V_water(i) *
     $          1000

           !! Weight plant transport by the area of porous root
           !! cross-sections. plant_trans [mmol layer-1 d-1]
           !! root_porous is in m2 but the units disappear because it
           !! is practically divided by 1m2 to get a unitless
           !! fraction.
           !! --------------------------------------------------------
           IF (Fwater(d,i) .GT. water_min) THEN
              Cnew = Ceq + (CO2_soil(d,i) - Ceq) * exp(-kCO2 /
     $             (V_water(i) / tiller_area(i)))
              CO2_plant(d) = CO2_plant(d) + (CO2_soil(d,i) - Cnew) *
     $             cmass * V_water(i) / 1000 
              CO2_soil(d,i) = Cnew * cmass * V_water(i) / 1000
           ELSE
              Cnew = CO2_soil(d,i)
              CO2_plant(d) = CO2_plant(d) + 0
              CO2_soil(d,i) = Cnew * cmass * V_water(i) / 1000
           ENDIF
           test = test + CH4(d,i)+CO2_soil(d,i)!+CH4_gas(d,i)
        ENDDO

c                        END OF CO2 PLANT TRANSPORT
c -----------------------------------------------------------------------

        !! ----------------------------------------
        !! CORRECTION OF CH4 AND CO2 CONCENTRATIONS
        !! ----------------------------------------

        !! Find a correction factor to conserve amounts of C.
        !! The c_soil and the total_c_flux should balance out the
        !! C input into the soil. If you don't use a correction
        !! factor, then c_input would be bigger than the
        !! flux to the atmosphere plus the current C contained in
        !! the soil. So, what you do is calculate how much C should
        !! be in the soil (c_input - total_c_flux) and divide
        !! this by the total concentration which is there c_soil.
        !! The corr_factor is usually greater than 1. You increase
        !! the concentration in each layer by the corr_factor.
        !! ----------------------------------------------------------

        !! Add up the total C flux out of the soil within a month
        !! total_c_flux is in g C m-2 d-1
        !! ------------------------------------------------------
        total_c_flux = total_c_flux + CO2_diff(d) + CH4_diff(d) +
     $       CH4_ebull(d) + CH4_plant(d) + CO2_plant(d)

        CO2_flux(d) =  CO2_diff(d) + CO2_plant(d)
        CH4_flux(d) =  CH4_diff(d) + CH4_plant(d) + CH4_ebull(d)

        !! Calculate the total C content in the soil
        !! -----------------------------------------
        c_soil = 0.0
        DO i = IDX, NLAYERS
           c_soil = c_soil + CO2_soil(d,i) + CH4(d,i)! + CH4_gas(d,i)
        ENDDO
 
        !! Get a correction factor and correct concentrations
        !! --------------------------------------------------
        IF (c_soil .NE. 0.0) THEN
           corr_factor = (c_input - total_c_flux) / c_soil
        ELSE
           corr_factor = 1.0
        ENDIF

        !! ERROR message:
        IF (1 - ABS(corr_factor) .GT. 0.15) THEN
           PRINT*, 'ERROR: corr_factor is too big/small', year, d,
     $          corr_factor
        ENDIF

        c_soil = 0.0
        DO i = IDX, NLAYERS
           CO2_soil(d,i) = CO2_soil(d,i) * corr_factor
           CH4(d,i) = CH4(d,i) * corr_factor

           !! Update C content in soil
           !! ------------------------
           c_soil = c_soil + CO2_soil(d,i) + CH4(d,i)! + CH4_gas(d,i)
           
           IF (d .eq. 365) THEN
              CO2_365(i)     = CO2_soil(365,i)
              CH4_365(i)     = CH4(365,i)
              CH4_diss_365(i) = CH4_diss(365,i)
              O2_365(i)      = O2(365,i)
c              CH4_total = CH4_total + CH4(365,i)
c              CO2_total = CO2_total + CO2_soil(365,i)
           ENDIF
        ENDDO
c        CH4_total = CH4_total + CH4_flux(d) 
c        CO2_total = CO2_total + CO2_flux(d) 

        ! Calculate monthly totals
        ! ------------------------
        ! Fluxes are in g C d-1 (daily fluxes) and g C month-1 (monthly
        ! fluxes)


        IF (daycount .LE. ndaymonth(m) - 1) THEN
           mCH4_plant(m) = mCH4_plant(m) + CH4_plant(d)
           mCH4_diff(m)  = mCH4_diff(m)  + CH4_diff(d)
           mCH4_ebull(m) = mCH4_ebull(m) + CH4_ebull(d)
        ELSE
           mCH4_plant(m) = mCH4_plant(m) + CH4_plant(d)
           mCH4_diff(m)  = mCH4_diff(m)  + CH4_diff(d)
           mCH4_ebull(m) = mCH4_ebull(m) + CH4_ebull(d)
           daycount = 0
           m = m + 1
        ENDIF
      ENDDO ! day loop

      RETURN
      END SUBROUTINE methane


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
!----------------------------------------------------------------------
!
!     SUBROUTINE cnstepgas
!
!     Crank-Nicholson timestepper for temperature diffusion equation.
!     This routine performs a single timestep (of length dt) of the heat
!     diffusion equation
!
!                        6 T    6  /      6 T \
!                        --- = --- | D(z) --- |
!                        6 t   6 z \      6 z /
!
!     where here, T(z, t) is the temperature, the '6's represent partial
!     differentiation, and D(z) is the (depth-dependent) diffusion
!     constant.  The boundary conditions are a prescribed surface
!     temperature, and no heat flow at the bottom of the solution domain.
!
!     INPUT PARAMETERS:
!
!      nlayers      Total number of layers represented in temp, d and
!                   dz arrays.  Not all of the layers have to be active
!                   at one time, but this parameter is used to constrain
!                   the sizes of the depth-based arrays.
!
!      layer0       Index of the first active layer.  The diffusion
!                   equation will be solved for all layers in the range
!                   layer0 - nlayers inclusive, and the surface temperature
!                   boundary condition will be applied to the top of layer
!                   index layer0.
!
!      temp         The temperatures in each layer at the start of the
!                   timestep.
!
!      d            The thermal diffusivities in each layer.
!
!      dz           The thickness of each layer.
!
!      surf_conc    The surface concentration boundary condition.
!
!      dt           The timestep.
!
!
!     NOTE ON UNITS: no units are specified here for any of these
!     inputs; the only constraint on units is that the layer
!     thicknesses, the diffusion constant and the timestep should be
!     measured in consistent units.  For instance, if the layer
!     thicknesses are measured in metres, and the timestep in days, then
!     the diffusion constants should have units of m2 / day.
!
!     OUTPUT PARAMETERS
!
!      conc         The concentrations in each layer at the end of the
!                   timestep.  Only layers layer0 - nlayers will have
!                   valid concentrations values after a call to cnstep. 
!                   All layers with index less than layer0 will have 
!                   missing value temperatures set (a value of -1.0E35).
!
!----------------------------------------------------------------------

      SUBROUTINE cnstepgas(layer0, conc, Di, dz, surf_conc, dt)

      IMPLICIT NONE

      include "params.cb"

      ! --- PARAMETERS ---

      INTEGER layer0            ! Index of first active layer.
      REAL conc(NLAYERS)        ! Layer concentrations.
      REAL Di(NLAYERS)          ! Layer diffusion constants.
      REAL dz(NLAYERS)          ! Layer thicknesses.
      REAL surf_conc            ! Surface concentration forcing.
      REAL dt                   ! Timestep.


      ! --- LOCAL VARIABLES ---

      ! Layer counters: note that there are two different layer counting
      ! schemes used, one for the input and output parameters (vectors of
      ! length NLAYERS) and one for the values used in the Crank-Nicholson
      ! solver (vectors of length active_layers)
      INTEGER layer, lidx, active_layers

      ! Diffusion constants averaged over adjacent layers.
      REAL dplus, dminus

      ! Layer-dependent weighting factors in Crank-Nicholson scheme.
      REAL dz_factor, Cplus, Cminus
      REAL dzhere, dzminus, dzplus
      REAL chere, cominus, coplus

      ! Leading diagonal, left and right subdiagonals for Crank-Nicholson
      ! matrix.
      REAL diag(NLAYERS - layer0 + 1)
      REAL left(NLAYERS - layer0 + 1)
      REAL right(NLAYERS - layer0 + 1)

      ! Right hand side vector for Crank-Nicholson scheme equations.
      REAL rhs(NLAYERS - layer0 + 1)

      ! Solution vector for Crank-Nicholson scheme equations.
      REAL solution(NLAYERS - layer0 + 1)


      ! --- CODE STARTS HERE ---

      ! Not all layers in the input arrays are actually used.  Calculate
      ! how many layers we have to deal with here.

      active_layers = NLAYERS - layer0 + 1

      !------------------------------------------------------------
      !
      !   BUILD TRIDIAGONAL MATRIX AND KNOWN RIGHT HAND SIDE
      !

      ! End members for off-diagonal elements.
      left(1) = 0.0
      right(active_layers) = 0.0

      ! Process the active layers.
      DO lidx = 1, active_layers
         ! Deal with different layer counting schemes.
         layer = lidx + layer0 - 1

         ! Calculate diffusion constants averaged over adjacent layers.
         ! The diffusion constant at the bottom layer is clamped to zero
         ! to enforce the no heat flow boundary condition there.  
         IF (layer .EQ. NLAYERS) THEN
            dplus = 0.0
         ELSE
            dplus = 0.5 * (Di(layer) + Di(layer + 1))
         END IF
         IF (layer .EQ. layer0) THEN
            dminus = Di(layer)
         ELSE
            dminus = 0.5 * (Di(layer) + Di(layer - 1))
         END IF

         ! Extract sensible values to use for concentration and
         ! thickness of the current layer, the layer above and the layer
         ! below, taking account of padding layers and end cases.
         dzhere = dz(layer)
         chere = conc(layer)
         IF (layer .LE. NLAYERS - 1) THEN
            dzplus = dz(layer + 1)
            coplus = conc(layer + 1)
         END IF


c         IF (layer .EQ. 1) THEN
         IF (layer .EQ. layer0) THEN
            dzminus = dz(layer)
            cominus = conc(layer)
         ELSE IF (layer .LE. NLAYERS + 1) THEN
            dzminus = dz(layer - 1)
            cominus = conc(layer - 1)
         END IF


         ! Calculate layer-dependent weighting factors.
         dz_factor = 0.25 * (dzplus + 2 * dzhere + dzminus)
         Cplus = dplus * dt / dz_factor / (dzplus + dzhere)
         Cminus = dminus * dt / dz_factor / (dzhere + dzminus)

         ! Fill in matrix diagonal and off-diagonal elements.
         ! Setting the diag(1) to 1.0 and right(1) to 0.0 means
         ! that the surface concentration is allocated to the top 
         ! layer.
         IF (lidx .EQ. 1) THEN
            diag(lidx) = 1.0
         ELSE
            diag(lidx) = 1.0 + Cplus + Cminus
         END IF
         IF (lidx .GT. 1) left(lidx) = -Cminus
         IF (lidx .LT. active_layers) THEN
            IF (lidx .GT. 1) THEN
               right(lidx) = -Cplus
            ELSE
               right(lidx) = 0.0
            END IF
         END IF

         ! Calculate right hand side vector values.
         IF (lidx .EQ. 1) THEN
            rhs(lidx) = surf_conc
         ELSE IF (lidx .EQ. active_layers) THEN
            rhs(lidx) = (1.0 - Cminus) * chere + Cminus * cominus
         ELSE
            rhs(lidx) = (1.0 - Cplus - Cminus) * chere +
     $           Cplus * coplus + Cminus * cominus
         END IF
      END DO


      !------------------------------------------------------------
      !
      !   SOLVE TRIDIAGONAL SYSTEM
      !
      CALL tridag(active_layers, left, diag, right, rhs, solution)


      !------------------------------------------------------------
      !
      !   FORMAT OUTPUT PARAMETERS
      !

      ! Transfer the solution to the concentration array. 
      DO lidx = 1, NLAYERS
         IF (lidx .LT. layer0) THEN
            conc(lidx) = MISSING_VALUE
         ELSE
            conc(lidx) = solution(lidx - layer0 + 1)
         END IF

      END DO

      END SUBROUTINE cnstepgas


c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE KILLPLANTS
c     Removal of PFTs with negative annual C increment
c     Note: Killing of PFTs newly beyond their bioclimatic limits is done in
c     subroutine establishment

      subroutine killplants(bm_inc,present,tree,lm_ind,rm_ind,hm_ind,
     *  sm_ind,nind,litter_ag,litter_bg,moss)

      implicit none

      include "params.cb"
      include "vegvars.cb"

c     ARGUMENTS

      do pft=1,npft

        if (present(pft)) then

          if (bm_inc(pft).lt.0.0) then  !negative C increment this year

            present(pft)=.false.   !remove PFT

c           Transfer killed biomass to litter

            if (tree(pft)) then
              litter_ag(pft)=litter_ag(pft)+(lm_ind(pft)+sm_ind(pft)+
     *          hm_ind(pft))*nind(pft)
            else  !grasses or mosses
              litter_ag(pft)=litter_ag(pft)+lm_ind(pft)*nind(pft)
            endif
            litter_bg(pft)=litter_bg(pft)+rm_ind(pft)*nind(pft)
            IF (moss(pft)) litter_bg(pft) = MISSING_VALUE

          endif

        endif
      enddo
      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE ALLOCATION
c     Allocation of annual C increment to leaf, stem and fine root
c     compartments, update of individual structure

      subroutine allocation(pftpar,allom1,allom2,latosa,wooddens,
     $     tree,grass,sla,wscal,nind, bm_inc,lm_ind,sm_ind
     $     ,hm_ind,rm_ind,crownarea,fpc_grid, lai_ind,height,litter_ag
     $     ,litter_bg,fpc_inc,present,year,moss)

      IMPLICIT NONE  

      include "params.cb"
      include "vegvars.cb"

c     PARAMETERS
      real xacc,yacc
      integer nseg
        parameter (xacc=0.1)     !threshold x-axis precision of allocation soln
        parameter (yacc=1.0e-10) !threshold y-axis precision of allocation soln
        parameter (nseg=20)
      integer jmax
        parameter (jmax=40) !max number of iterations in search for alloc soln

c     ARGUMENTS
      real wscal(1:npft)

c     LOCAL VARIABLES
      real lminc_ind ! individual leafmass increment this year
      real sminc_ind ! individual sapwood mass increment this year
      real rminc_ind ! individual fineroot mass increment this year
      real bm_inc_ind ! individual total biomass increment this year
      real sap_xsa ! cross sectional area of sapwood
      real stemdiam ! stem diameter
      real x1,x2,rtbis,dx,xmid,sign !working vars in bisection
      double precision fx1,fmid
      real lminc_ind_min  !min leafmass increment to maintain current sapwood
      real rminc_ind_min  !min rootmass increment to support new leafmass
      integer j  !counter in bisection loop
      real lmtorm ! ratio of leafmass to fine rootmass
      real fpc_grid_old  !previous year's FPC
      real fpc_ind  !individual FPC
      real crownarea_max  !maximum crown area (m2)

c     INITIALISE
c     ----------
      sap_xsa = 0.0


      do pft=1,npft
         
         if (present(pft)) then

            bm_inc_ind=bm_inc(pft)/nind(pft)

c     calculate this year's leaf to fine root mass ratio from mean annual 
c     water scalar and pft specific parameter

            lmtorm=pftpar(pft,18)*wscal(pft)

            if (tree(pft)) then

               crownarea_max=pftpar(pft,20)

c     TREE ALLOCATION

c     Allocation of this year's biomass increment (bm_inc_ind) to the
c     three living carbon pools, such that the basic allometric 
c     relationships (A-C below) are always satisfied.

c     (A) (leaf area) = latosa * (sapwood xs area)
c     (Pipe Model, Shinozaki et al. 1964a,b; Waring et al 1982)
c     (B) (leaf mass) = lmtorm * (root mass)
c     (C) height = allom2 * (stem diameter)**allom3
c     (source?)
c     (D) (crown area) = min (allom1 * (stem diameter)**reinickerp,
c     crownarea_max) 

c     Mathematical derivation:   

c     (1) bm_inc_ind = lminc_ind + sminc_ind + rminc_ind
c     (2) leaf_area_new = latosa * sap_xsa_new   [from (A)]
c     (3) leaf_area_new = (lm_ind + lminc_ind) * sla
c     from (2) & (3),           
c     (4) (lm_ind + lminc_ind) * sla = latosa * sap_xsa_new 
c     from (4),
c     (5) sap_xsa_new = (lm_ind + lminc_ind) * sla / latosa
c     (6) (lm_ind + lminc_ind) = lmtorm * (rm_ind + rminc_ind)
c     [from (B)]
c     (7) height_new = allom2 * stemdiam_new**allom3  [from (C)]
c     from (1),  
c     (8) sminc_ind = bm_inc_ind - lminc_ind - rminc_ind 
c     from (6), 
c     (9) rminc_ind=((lm_ind + lminc_ind) / lmtorm) - rm_ind
c     from (8) & (9),
c     (10) sminc_ind = bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind)  / lmtorm) + rm_ind  
c     (11) wooddens = (sm_ind + sminc_ind + hm_ind) / stemvolume_new 
c     (12) stemvolume_new = height_new * pi * stemdiam_new**2 / 4  
c     from (10), (11) & (12)                   
c     (13) stemdiam_new = [ ((sm_ind + bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind + hm_ind)
c     / wooddens) / (height_new * pi / 4) ]**(1/2)
c     combining (7) and (13),
c     (14) height_new = allom2 * [ ((sm_ind + bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind + hm_ind) 
c     / wooddens) / (height_new * pi / 4) ]**(1/2 * allom3)
c     from (14),
c     (15) height_new**(1 + 2 / allom3) = allom2**(2 / allom3)
c     * ((sm_ind + bm_inc_ind - lminc_ind - ((lm_ind + lminc_ind) 
c     / lmtorm) + rm_ind + hm_ind) / wooddens) / (pi / 4)
c     (16) wooddens = (sm_ind + sminc_ind) / sapvolume_new 
c     from (10) and (16),
c     (17) wooddens = (sm_ind + bm_inc_ind - lminc_ind 
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind) / sapvolume_new 
c     (18) sapvolume_new = height_new * sap_xsa_new
c     from (17) and (18),
c     (19) sap_xsa_new = (sm_ind + bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind)
c     / (height_new * wooddens) 
c     from (19),
c     (20) height_new = (sm_ind + bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind )
c     / (sap_xsa_new * wooddens)
c     from (5) and (20),
c     (21) height_new**(1 + 2 / allom3) = [ (sm_ind + bm_inc_ind 
c     - lminc_ind - ((lm_ind + lminc_ind) / lmtorm) + rm_ind )
c     / ((lm_ind + lminc_ind) * sla * wooddens / latosa) ]
c     **(1 + 2 / allom3)
c     -------------------------------------------------------------------
c     (15) and (21) are two alternative expressions for
c     height_new**(1 + 2 / allom3). Combining these,

c     (22) allom2**(2 / allom3) * ((sm_ind + bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind + hm_ind)
c     / wooddens) / (pi / 4) - [ (sm_ind + bm_inc_ind - lminc_ind
c     - ((lm_ind + lminc_ind) / lmtorm) + rm_ind )
c     / ((lm_ind + lminc_ind) * sla * wooddens / latosa) ]
c     **(1 + 2 / allom3)
c     = 0

c     Equation (22) can be expressed in the form f(lminc_ind)=0.

c     Numerical methods are used to solve the equation for the
c     unknown lminc_ind.
c     -------------------------------------------------------------------

c     Work out minimum leaf production to maintain current sapmass

c     (23) sap_xsa = sm_ind / wooddens / height
c     from (A) and (23),
c     (24) leaf_mass * sla = latosa * sap_mass / wooddens / height
c     from (24),
c     (25) leaf_mass = latosa * sap_mass / (wooddens * height * sla)
c     from (25), assuming sminc_ind=0,
c     (26) lm_ind + lminc_ind_min = latosa * sm_ind
c     / (wooddens * height * sla)
c     from (26),
c     (27) lminc_ind_min = latosa * sm_ind / (wooddens * height * sla)
c     - lm_ind

               lminc_ind_min=latosa(pft)*sm_ind(pft)/
     *              (wooddens(pft)*height(pft)*sla(pft))-lm_ind(pft) !eqn (27)

c     Work out minimum root production to support this leaf mass
c     (i.e. lm_ind + lminc_ind_min)
c     May be negative following a reduction in soil water limitation
c     (increase in lmtorm) relative to last year.

c     from (B) and (25),
c     (28) root_mass = latosa * sap_mass / (wooddens * height * sla)
c     / lmtorm
c     from (28), assuming sminc_ind=0,
c     (29) rm_ind + rminc_ind_min = latosa * sm_ind
c     / (wooddens * height * sla * lmtorm)
c     from (29),
c     (30) rminc_ind_min = latosa * sm_ind
c     / (wooddens * height * sla * lmtorm) - rm_ind

               rminc_ind_min=latosa(pft)*sm_ind(pft)/(wooddens(pft)*
     $              height(pft)*
     *              sla(pft)*lmtorm)-rm_ind(pft) !eqn (30)
               if (rminc_ind_min.gt.0.0.and.lminc_ind_min.gt.0.0.and.
     *              rminc_ind_min+lminc_ind_min.le.bm_inc_ind.or.
     *              bm_inc_ind.le.0.0) then

c     Normal allocation (positive increment to all living C
c     compartments)

c     Calculation of leaf mass increment (lminc_ind) that satisfies
c     Eqn (22) using Bisection Method (Press et al 1986, p 346)

c     Seeking a root for non-negative lminc_ind, rminc_ind and
c     sminc_ind.  There should be exactly one (no proof presented, but
c     Steve has managed one) and it should lie between x1=0 and
c     x2=(bm_inc_ind-(lm_ind/lmtorm-rm_ind))/(1+1/lmtorm).

                  x1=0.0
                  x2=(bm_inc_ind-(lm_ind(pft)/lmtorm-rm_ind(pft)))/
     *                 (1.0+1.0/lmtorm)
                  dx=(x2-x1)/real(nseg)
                  if (lm_ind(pft).eq.0.0) x1=x1+dx !to avoid division by zero

c     evaluate f(x1)=LHS of eqn (22) at x1

                  fx1=allom2(pft)**(2.0/allom3)*((sm_ind(pft)+
     $                 bm_inc_ind-x1
     *                 -((lm_ind(pft)+x1)/lmtorm)+rm_ind(pft)+
     $                 hm_ind(pft))
     *                 /wooddens(pft))/(pi/4.0)-((sm_ind(pft)+
     $                 bm_inc_ind-x1
     *                 -((lm_ind(pft)+x1)/lmtorm)+rm_ind(pft))
     *                 /((lm_ind(pft)+x1)*sla(pft)*wooddens(pft)/
     $                 latosa(pft)))
     *                 **(1.0+2.0/allom3)
                  
c     Find approximate location of leftmost root on the interval
c     (x1,x2).  Subdivide (x1,x2) into nseg equal segments seeking
c     change in sign of f(xmid) relative to f(x1).

                  fmid=fx1
                  xmid=x1

                  do while (fmid*fx1.gt.0.0.and.xmid.lt.x2)

                     xmid=xmid+dx
                     fmid=allom2(pft)**(2.0/allom3)*((sm_ind(pft)
     $                    +bm_inc_ind
     $                    -xmid
     *                    -((lm_ind(pft)+xmid)/lmtorm)+rm_ind(pft)
     $                    +hm_ind(pft))
     *                    /wooddens(pft))/(pi/4.0)-((sm_ind(pft)+
     $                    bm_inc_ind-xmid
     *                    -((lm_ind(pft)+xmid)/lmtorm)+rm_ind(pft))
     *                    /((lm_ind(pft)+xmid)*sla(pft)*wooddens(pft)/
     $                    latosa(pft)))**(1.0+2.0/allom3)

                  enddo


                  x1=xmid-dx
                  x2=xmid

c     Apply bisection method to find root on the new interval (x1,x2)

                  fx1=allom2(pft)**(2.0/allom3)*((sm_ind(pft)+
     $                 bm_inc_ind-x1
     *                 -((lm_ind(pft)+x1)/lmtorm)+rm_ind(pft)+
     $                 hm_ind(pft))
     *                 /wooddens(pft))/(pi/4.0)-((sm_ind(pft)+
     $                 bm_inc_ind-x1
     *                 -((lm_ind(pft)+x1)/lmtorm)+rm_ind(pft))
     *                 /((lm_ind(pft)+x1)*sla(pft)*wooddens(pft)/
     $                 latosa(pft)))
     *                 **(1.0+2.0/allom3)

                  if (fx1.ge.0.0) then
                     sign=-1.0
                  else
                     sign=1.0
                  endif

                   rtbis=x1

                  dx=x2-x1

c     Bisection loop
c     Search iterates on value of xmid until xmid lies within
c     xacc of the root, i.e. until |xmid-x|<xacc where f(x)=0

                  fmid=1.0      !dummy value to guarantee entry to loop
                  j=0.0         !number of iterations so far (maximum tries=jmax)

                  do while (dx.ge.xacc.and.abs(fmid).gt.yacc)

                     dx=dx*0.5
                     xmid=rtbis+dx

c     calculate fmid=f(xmid) [eqn (22)]

                     fmid=allom2(pft)**(2.0/allom3)*((sm_ind(pft)+
     $                    bm_inc_ind
     $                    -xmid
     *                    -((lm_ind(pft)+xmid)/lmtorm)+rm_ind(pft)+
     $                    hm_ind(pft))
     *                    /wooddens(pft))/(pi/4.0)-((sm_ind(pft)+
     $                    bm_inc_ind-xmid
     *                    -((lm_ind(pft)+xmid)/lmtorm)+rm_ind(pft))
     *                    /((lm_ind(pft)+xmid)*sla(pft)*wooddens(pft)/
     $                    latosa(pft)))
     *                    **(1.0+2.0/allom3)

                     if (fmid*sign.le.0.0) rtbis=xmid
                     j=j+1

                  enddo

c     Now rtbis contains numerical solution for lminc_ind given
c     eqn (22)

                  lminc_ind=rtbis
c     Calculate increments in other compartments using allometry
c     relationships

                  rminc_ind=(lm_ind(pft)+lminc_ind)/lmtorm-rm_ind(pft) !eqn (9)
                  sminc_ind=bm_inc_ind-lminc_ind-rminc_ind !eqn (1)
               else

c     Abnormal allocation: reduction in some C compartment(s)
c     to satisfy allometry

c     Attempt to distribute this year's production among leaves and
c     roots only
c     (31) bm_inc_ind = lminc_ind + rminc_ind
c     from (31) and (9),
c     (32) bm_inc_ind = lminc_ind + ((lm_ind + lminc_ind) / lmtorm)
c     - rm_ind
c     from (32)
c     (33) lminc_ind = (bm_inc_ind - lmind / lmtorm + rm_ind) /
c     (1 + 1 / lmtorm)

                  lminc_ind=(bm_inc_ind-lm_ind(pft)/lmtorm+rm_ind(pft))/
     *                 (1.0+1.0/lmtorm) !eqn (33)

                  if (lminc_ind.ge.0.0) then

c     Positive allocation to leafmass

                     rminc_ind=bm_inc_ind-lminc_ind !eqn (31)

c     Add killed roots (if any) to below-ground litter

                     if (rminc_ind.lt.0.0) then
                        lminc_ind=bm_inc_ind
                        rminc_ind=((lm_ind(pft) + lminc_ind) / lmtorm)
     *                       - rm_ind(pft)
                        litter_bg(pft)=litter_bg(pft)+(-rminc_ind)*
     $                       nind(pft)
                     endif
                  else

c     Negative allocation to leaf mass
                     
                     rminc_ind=bm_inc_ind
                     lminc_ind=(rm_ind(pft)+rminc_ind)*lmtorm-
     $                    lm_ind(pft)
!from eqn (9)

c     Add killed leaves to litter

                     litter_ag(pft)=litter_ag(pft)+(-lminc_ind)*
     $                    nind(pft)

                  endif

c     Calculate sminc_ind (must be negative)

c     from (25),
c     (34) lm_ind + lminc_ind = latosa * (sm_ind + sminc_ind)
c     / (wooddens * height * sla)
c     from (34),
c     (35) sminc_ind = (lm_ind + lminc_ind) * wooddens * height * sla
c     / latosa - sm_ind

                  sminc_ind=(lm_ind(pft)+lminc_ind)*wooddens(pft)*
     $                 height(pft)*
     *                 sla(pft)/latosa(pft)-sm_ind(pft) !eqn (35)


c     Convert killed sapwood to heartwood

                  hm_ind(pft)=hm_ind(pft)+(-sminc_ind)

               endif


c     Increment C compartments

               lm_ind(pft)=lm_ind(pft)+lminc_ind
               rm_ind(pft)=rm_ind(pft)+rminc_ind
               sm_ind(pft)=sm_ind(pft)+sminc_ind
               if (lm_ind(pft).le.0.0.or.rm_ind(pft).lt.0.0
     *              .or.sm_ind(pft).lt.0.0) then
                  litter_ag(pft)=litter_ag(pft)+((lm_ind(pft)+
     *                 sm_ind(pft)+hm_ind(pft)+bm_inc_ind)*nind(pft))
                  lm_ind(pft)=0.0
                  litter_bg(pft)=litter_bg(pft)+(rm_ind(pft))*nind(pft)
                  rm_ind(pft)=0.0
                  present(pft)=.false. !remove PFT
               endif 

c     Calculate new height, diameter and crown area

               IF (present(pft)) THEN
                  sap_xsa=lm_ind(pft)*sla(pft)/latosa(pft) !eqn (5)
                  height(pft)=sm_ind(pft)/sap_xsa/wooddens(pft)
                  stemdiam=(height(pft)/allom2(pft))**(1.0/allom3) !eqn (C)
                  crownarea(pft)=min(allom1(pft)*stemdiam**reinickerp,
     *                 crownarea_max) !eqn (D)
               ELSE
                  sap_xsa = 0.0
                  height(pft) = 0.0
                  stemdiam = 0.0
                  crownarea(pft) = 0.0
               ENDIF
            else if (grass(pft)) THEN

c     GRASS ALLOCATION
c     Distribute this year's production among leaves and fine roots
c     according to leaf to rootmass ratio [eqn (33)]
c     Relocation of C from one compartment to the other not allowed:
c     negative increment in either compartment transferred to litter

               lminc_ind=(bm_inc_ind-lm_ind(pft)/lmtorm+rm_ind(pft))/
     *              (1.0+1.0/lmtorm)
               rminc_ind=bm_inc_ind-lminc_ind
               if (lminc_ind.ge.0.0) then

c     Add killed roots (if any) to below-ground litter
c     
c     CHECK: take out if statement because if rminc is negative than
c     root mass has been translocated to the leaves, therefore mass balance
c     problem since this carbon stays in the vegetation but is in addition
c     added to the litter pool. ALLOW translocation from roots to leaves 
c     i.e. assume carbon stores in the roots which can be delivered 
c     to the leaves under times of stress. 
c     

               else

c     Negative allocation to leaf mass

                  rminc_ind=bm_inc_ind
                  lminc_ind=(rm_ind(pft)+rminc_ind)*lmtorm-lm_ind(pft)
!     from eqn (9)

c     Add killed leaves to litter

                  litter_ag(pft)=litter_ag(pft)+(-lminc_ind)*nind(pft)

               endif

c     Increment C compartments

               lm_ind(pft)=lm_ind(pft)+lminc_ind
               rm_ind(pft)=rm_ind(pft)+rminc_ind

            ELSE                !moss pft
               lminc_ind = bm_inc_ind
               IF (lminc_ind .GT. 0.0) THEN
               ELSE
                  litter_ag(pft) = litter_ag(pft) + (-lminc_ind) *
     $                 nind(pft)
               ENDIF
               lm_ind(pft) = lm_ind(pft) + lminc_ind
            ENDIF               ! end of if tree


c     Update LAI and FPC
            if (crownarea(pft).gt.0.0) then
               lai_ind(pft)=(lm_ind(pft)*sla(pft))/crownarea(pft)
            else
               lai_ind(pft)=0.0
            endif

            fpc_ind=(1.0-exp(-0.5*lai_ind(pft)))
            fpc_grid_old=fpc_grid(pft)
            fpc_grid(pft)=crownarea(pft)*nind(pft)*fpc_ind
            fpc_inc(pft)=max(0.0,fpc_grid(pft)-fpc_grid_old)
            if(fpc_grid(pft).eq.0.0) present(pft)=.false.
         else
            height(pft) = 0.0 !added by Rita
            lm_ind(pft) = 0.0
            hm_ind(pft) = 0.0
            rm_ind(pft) = 0.0
            sm_ind(pft) = 0.0
         endif
      enddo                     !pft

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE LIGHT
c     Competition for light among PFTs

      subroutine light(present,tree,grass,lm_ind,sm_ind,hm_ind,rm_ind,
     *  crownarea,fpc_grid,fpc_inc,nind,litter_ag,litter_bg,sla,moss)

      IMPLICIT NONE

      include "params.cb"
      include "vegvars.cb"


c     LOCAL VARIABLES
      integer ntree        !no of tree PFTs currently present
      real fpc_tree_total  !total grid FPC for tree PFTs
      real fpc_grass_total  !total grid FPC for grass PFTs
      real grasscover      !grass PFT proportional cover ("crown area")
      real fpc_moss_total       !total grid FPC for moss PFTs (Rita)
      real mosscover            !moss PFT proportional cover ("crown area")(Rita)
      real fpc_inc_tree    !this years total FPC increment for tree PFTs
      real excess          !tree FPC or grass cover to be reduced
      real nind_kill       !reduction in individual density to reduce tree FPC
                           !to permitted maximum (indiv/m2)
      real lm_kill         !reduction in grass PFT leaf mass to reduce grass
                           !cover to permitted maximum (gC)
      real rm_kill         !reduction in grass PFT root mass to reduce grass
                           !cover to permitted maximum (gC)  
      real fpc_ind
      real lm_old
      real ngrass
      real nmoss                !Rita

c     Calculate total woody FPC, FPC increment and grass cover (= crown area)
 

      fpc_tree_total=0.0
      fpc_inc_tree=0.0 
      fpc_grass_total=0.0
      fpc_moss_total=0.0
      ntree=0 
      ngrass=0
      nmoss=0
      lm_old=0.0      

      do pft=1,npft

        if (present(pft)) then
          if (tree(pft)) then
            ntree=ntree+1    
            fpc_tree_total=fpc_tree_total+fpc_grid(pft)
            fpc_inc_tree=fpc_inc_tree+fpc_inc(pft)
          else if (grass(pft)) then !grasses
            grasscover=crownarea(pft)
            fpc_grass_total=fpc_grass_total+fpc_grid(pft) 
            ngrass=ngrass+1
         else   !mosses (Rita)
            mosscover=crownarea(pft) !??
            fpc_moss_total=fpc_moss_total+fpc_grid(pft)
            nmoss=nmoss+1
          endif        
        endif  

      enddo

c     LIGHT COMPETITION
      do pft=1,npft

        if (present(pft)) then

           if (tree(pft)) then   

              if (fpc_tree_total.gt.fpc_tree_max) then    ! case (1)

c$$$                   if (fpc_inc_tree.gt.0.0) then
c$$$                      excess=(fpc_tree_total-fpc_tree_max)*
c$$$     *                  (fpc_inc(pft)/fpc_inc_tree)
c$$$                   else
c$$$                      excess=(fpc_tree_total-fpc_tree_max)*
c$$$     *                  (1.0/real(ntree))
c$$$                   endif
c     Changed according to Kuno Strassmann, released 18.1.2005, changed
c     on 8.8.2006
                   if (fpc_inc_tree.gt.0.0) then
                      excess=(fpc_tree_total-fpc_tree_max)*
     *                  (fpc_grid(pft)/fpc_tree_total)
                   else
                      excess=(fpc_tree_total-fpc_tree_max)*
     *                  (1.0/real(ntree))
                   endif

c  Reduce individual density (and thereby gridcell-level biomass)
c  so that total tree FPC reduced to 'fpc_tree_max'
                   nind_kill=nind(pft)*(excess/fpc_grid(pft))
                   nind(pft)=nind(pft)-nind_kill

c                 Transfer lost biomass to litter

                  litter_ag(pft)=litter_ag(pft)+nind_kill
     *               *(lm_ind(pft)+sm_ind(pft)+hm_ind(pft))
                  litter_bg(pft)=litter_bg(pft)+nind_kill
     *               *rm_ind(pft)
                              
             endif
         else  ! grass and moss 
             if((fpc_grass_total+fpc_moss_total).gt.
     *            (1.0-min(fpc_tree_total,fpc_tree_max))) then

c   grass competes with itself if total fpc exceeds 1
c  NEEDS COMMENTS !!!!!!????!!!!

                     excess=(min(fpc_tree_total,fpc_tree_max) +
     *               (fpc_grass_total + fpc_moss_total) - 1.0) *
     *               (fpc_grid(pft)/(fpc_grass_total+fpc_moss_total))
                     lm_old=lm_ind(pft)
                     lm_ind(pft) = -2.0 *
     *                 alog(1.0-(fpc_grid(pft)-excess))/sla(pft)
                     lm_kill=lm_old-lm_ind(pft)
                     rm_kill=rm_ind(pft)*(lm_kill/lm_old)
                     rm_ind(pft)=rm_ind(pft)-rm_kill

c               Transfer lost biomass to litter 

                    litter_ag(pft)=litter_ag(pft)+lm_kill
                    litter_bg(pft)=litter_bg(pft)+rm_kill
                    IF (moss(pft)) litter_bg(pft) = MISSING_VALUE
              endif
          endif 

                     
c
c       update fpc (for establishment routine)
c
          if (crownarea(pft).gt.0.0) then
             lai_ind(pft)=(lm_ind(pft)*sla(pft))/crownarea(pft)
          else
             lai_ind(pft)=0.0
          endif

          fpc_ind=(1.0-exp(-0.5*lai_ind(pft)))
          fpc_grid(pft)=crownarea(pft)*nind(pft)*fpc_ind 
          if(fpc_grid(pft).eq.0.0) present(pft)=.false.
       endif                    ! present


      enddo  !pft


      return  
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE MORTALITY
c     Tree background and stress mortality
c     Rita: deleted argument boreal, since it was called in the SR but not
c     used later on. 

      subroutine mortality(pftpar,present,tree,bm_inc, turnover_ind,sla
     $     ,lm_ind,sm_ind,hm_ind,rm_ind,nind, litter_ag,litter_bg,dtemp
     $     ,anpp,mtemp_max,moss)

      IMPLICIT NONE

      include "params.cb"
      include "vegvars.cb"

c     PARAMETERS
      real mort_max
        parameter (mort_max=0.05) !asymptotic maximum mortality rate (/year)
      real k_mort                 !coefficient of growth efficiency in
        parameter (k_mort=0.5)   !mortality equation 
c  ramp for heat damage function. Above 200 growing degree days above the
c  upper limit tw, establishment is zero and mortality 100%  
      real ramp_gddtw  
        parameter (ramp_gddtw=400.0) 
   
c     ARGUMENTS 
      real twmax,greffic(1:npft)
      real dtemp(1:365)
      real mtemp_max
      real gddtw(1:npft)

c     LOCAL VARIABLES
      real bm_delta   !net individual living biomass increment (incorporating
                      !loss through leaf, root and sapwood turnover) (gC)
      real mort       !tree mortality rate
      real nind_kill  !reduction in individual density due to mortality
                      !(indiv/m2)
      integer d
      real heatstress(1:npft)  !reduction in individual density (& establishment) due to heat
                       !induced mortality  (indiv/m2)


      do pft=1,npft
        
c     initialisation
        heatstress(pft)=0.0
        twmax=pftpar(pft,31)  !PFT-specific upper limit of warmest-month
                              !temperature
        
        if (present(pft).and.tree(pft)) then
c         Calculate net individual living biomass increment

           bm_delta=max(0.0,bm_inc(pft)/nind(pft)-turnover_ind(pft))

c          Calculate growth efficiency (net biomass increment per unit leaf area)

           greffic(pft)=bm_delta/lm_ind(pft)/sla(pft)

c          Mortality rate inversely related to growth efficiency
c          (Prentice et al 1993)
           mort=mort_max/(1.0+k_mort*greffic(pft))
**          mort=0.0
c          heat damage mortality in boreal trees

           if (mtemp_max.gt.twmax) then  ! heat damage
c            calculate growing degree days above twmax
              gddtw(pft)=0.0
              do d=1,365
                 gddtw(pft)=gddtw(pft)+max(0.0,dtemp(d)-twmax)
              enddo                   
              heatstress(pft)=min(1.0,gddtw(pft)/ramp_gddtw) 
           else
              heatstress(pft)=0.0
           endif  
          
c         Reduce individual density (and thereby gridcell-level biomass) by
c         mortality rate

           mort=min(1.0,mort+heatstress(pft))
           nind_kill=nind(pft)*mort
           nind(pft)=nind(pft)-nind_kill 
       

c         Transfer lost biomass to litter

           litter_ag(pft)=litter_ag(pft)+nind_kill*(lm_ind(pft)+
     *          sm_ind(pft)+hm_ind(pft))
           litter_bg(pft)=litter_bg(pft)+nind_kill*rm_ind(pft)

           IF (moss(pft)) litter_bg(pft) = MISSING_VALUE
        endif  

        if (nind(pft).eq.0.) present(pft)=.false.  

      enddo    

      return
      end



c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE FIRE
c     Biomass destruction through disturbance by fire

      subroutine fire(pftpar,dtemp,litter_ag,acflux_fire,afire_frac,
     *  lm_ind,rm_ind,sm_ind,hm_ind,nind,dw1,present,tree,fire_length)
         
      IMPLICIT NONE

      include "params.cb"
      include "vegvars.cb"

c     PARAMETERS
      real minfuel
        parameter (minfuel=100.0)  !fuel threshold to carry a fire (gC/m2)

c     ARGUMENTS
      real dtemp(1:365)
      real acflux_fire
      real dw1(1:365)
      real afire_frac
      real fire_length
     
c     LOCAL VARIABLES
      integer d
      real fuel,fire_prob
      real fire_index,disturb
      real resist(1:npft)
      real moistfactor,flam,litter_ag_total
      real fire_term

c     Calculate total above-ground litter

      litter_ag_total=0.0
      do pft=1,npft
        litter_ag_total=litter_ag_total+litter_ag(pft)
      enddo

c     Calculate litter moisture weighting factor

      moistfactor=0.0

      do pft=1,npft
        flam=pftpar(pft,6)
        if (litter_ag_total.gt.0.0) then
          moistfactor=moistfactor+(litter_ag(pft)/litter_ag_total)*flam
        else
          moistfactor=0.0
        endif   
      enddo
c     First calculate the annual fraction of the grid cell affected by fire

c     Initialise

      fire_length=0.0
      fuel=0.0
      acflux_fire=0.0

c     Assign a minimum fire fraction (for presentational purposes)

      afire_frac=0.001

c     Assign PFT resistance to fire

      do pft=1,npft
        resist(pft)=pftpar(pft,8)
      enddo
       
c     Calculate the length of the fire season (units=days)

      do d=1,365

c       Calculate today's fire probability, fire_prob
c       Assume fire is only possible when temperature is above zero

        if (dtemp(d).gt.0.0.and.moistfactor.gt.0.0) then
          fire_prob=EXP((-pi)*(dw1(d)/moistfactor)**2)
        else 
          fire_prob=0.0
        endif

        fire_length=fire_length+fire_prob

      enddo

c     Calculate annual fire index

      fire_index=fire_length/365.0 


c     Calculate the fraction of the grid cell affected by fire

*      afire_frac=1.0-(EXP(-0.2*fire_index**1.4))**1.1 
      
      fire_term=fire_index-1.0
      
c  new version of fractional area burnt function 21.03.2001

      afire_frac=fire_index*EXP( fire_term/(0.45*fire_term**3 
     *    + 2.83*fire_term**2 + 2.96*fire_term + 1.04))
      
      
      
c     Calculate the available fuel (above-ground litter) to carry the fire

      do pft=1,npft
        fuel=fuel+litter_ag(pft)
      enddo
      
c     Reduce fraction of grid cell affected by fire when fuel
c     becomes limiting (reduced carrying capacity)
      
      if (fuel.lt.minfuel) then
*        afire_frac=afire_frac*(1.0/(minfuel**2))*fuel**2   
        afire_frac=0.0
      endif
      
      if (afire_frac.lt.0.001) afire_frac=0.001   

c     Implement the effect of the fire on vegetation structure and litter 
c     in the disturbed fraction.  

c     Each PFT is assigned a resistance to fire, representing the fraction of
c     the PFT which survives a fire. Grasses assumed already to have completed
c     their life cycle and thus are not affected by fire, giving them
c     a competitive advantage against woody PFTs.

      do pft=1,npft

        if (present(pft).and.tree(pft)) then

c         Calculate the fraction of individuals in grid cell which die

          disturb=(1.0-resist(pft))*afire_frac

c         Calculate carbon flux to atmosphere (gC/m2) due to burnt biomass

          acflux_fire=acflux_fire+disturb*(nind(pft)*
     *      (lm_ind(pft)+sm_ind(pft)+hm_ind(pft)+rm_ind(pft)))

c         Update the individual density

          nind(pft)=nind(pft)*(1.0-disturb)

        endif

c       Add combusted litter to carbon flux to atmosphere term

        acflux_fire=acflux_fire+(afire_frac*litter_ag(pft))

c       Update the above ground litter term

        litter_ag(pft)=(1.0-afire_frac)*litter_ag(pft)
      enddo
           
      return
      end


c -----------------------------------------------------------------------------
c//////////////////////////////////////////////////////////////////////////////
c******************************************************************************
c     SUBROUTINE ESTABLISHMENT
c     Establishment of new individuals (saplings) of woody PFTs,
c     grass establishment, removal of PFTs not adapted to current climate,
c     update of individual structure and FPC.

      subroutine establishment(pftpar,present,survive,estab,nind,lm_ind,
     $     sm_ind,rm_ind,hm_ind,lm_sapl,sm_sapl,rm_sapl,hm_sapl,
     $     crownarea,fpc_grid,lai_ind,height,sla,wooddens,latosa,mprec,
     $     litter_ag,litter_bg,tree,allom1,allom2, acflux_estab
     $     ,leafondays,leafoffdays,leafon,mnpp,anpp,mnpp_add, anpp_add
     $     ,year,grass,moss,vegc)

      IMPLICIT NONE

      include "params.cb"
      include "inputvars.cb"
      include "vegvars.cb"

c     PARAMETERS
      real aprec_min_estab                !minimum annual precipitation for
        parameter (aprec_min_estab=100.0) !establishment [mm]
      real estab_max                      !maximum sapling establishment rate
*        parameter (estab_max=0.12)        !(indiv/m2)  
        parameter (estab_max=0.24)
      real nind_min                       !minimum individual density for
        parameter (nind_min=1.0E-10)      !persistence of PFT (indiv/m2)
     
c     ARGUMENTS
      real acflux_estab
      integer leafondays(1:npft),leafoffdays(1:npft)
      logical leafon(1:npft)
      real mnpp_add(1:12),anpp_add        !NPPs of PFTs that are killed

   
c     LOCAL VARIABLES
      integer npft_estab  !number of regenerating tree PFTs
      real aprec          !annual precipitation [mm]
      real fpc_tree_total !total grid FPC for tree PFTs
      real estab_rate     !sapling establishment rate over area
                          !available for establishment (indiv/m2)
      real estab_grid     !grid-level establishment rate (indiv/m2)
      real nind_old       !number of individuals /m2 before establishment
      real stemdiam       !stem diameter (m)
      real sm_ind_temp    !preliminary sapwood mass (g/m2)
      real fpc_ind        !individual FPC
      real crownarea_max  !maximum crown area (m2)
      real fpc_total      !total grid FPC
      real bare1          !gridcell bare ground fraction, used for above
                          !ground calculations
      real bare2          !gridcell bare ground fraction, used for below
                          !ground calculations, since mosses don't have
                          !roots, this includes only grasses
      integer ngrass, nmoss
      real fpc_grass_total
      real fpc_moss_total
      real bare_max
      REAL excess         !excess fpc of mosses and grasses
      REAL lm_old
      REAL lm_kill
      REAL rm_kill


c     Kill PFTs not adapted to current climate, introduce newly "adapted" PFTs
      
c     Initialize buffers for npps of pfts that go not present
      excess = 0.0
      anpp_add=0.
      bare1 = 0.0
      bare2 = 0.0
      lm_kill = 0.0
      rm_kill = 0.0
      lm_old = 0.0
      do m=1,12
         mnpp_add(m)=0.
      enddo

c     Calculate annual precipitation
      bare_max=0.0
      aprec=0.0
      do m=1,12
         aprec=aprec+mprec(m)
      enddo


      do pft=1,npft    
         
         if (present(pft).and.
     *        (.not.survive(pft).or.nind(pft).lt.nind_min)) then !kill PFT
            present(pft)=.false.
c     Add up NPP of PFTs that are killed in extra balance
            do m=1,12
               mnpp_add(m)=mnpp_add(m)+mnpp(m,pft)
               anpp_add=anpp_add+mnpp(m,pft)
            enddo

c     Add killed biomass to litter

            if (tree(pft)) then
               litter_ag(pft)=litter_ag(pft)+(lm_ind(pft)+sm_ind(pft)+
     *              hm_ind(pft))*nind(pft)
            else                !grasses or mosses
               litter_ag(pft)=litter_ag(pft)+lm_ind(pft)*nind(pft)
            endif
            litter_bg(pft)=litter_bg(pft)+rm_ind(pft)*nind(pft)
            IF (moss(pft)) litter_bg(pft) = MISSING_VALUE
            
            lm_ind(pft)=0.0
            fpc_grid(pft) = 0.0 !added by Rita
            height(pft) = 0.0   !added by Rita
         elseif (.not.present(pft).and.survive(pft).and.estab(pft)
     *           .and.aprec.ge.aprec_min_estab) then

c     Introduce PFT if conditions suitable for establishment
            
            present(pft)=.true.
            if (tree(pft)) then
               nind(pft)=0.0
            else
               nind(pft)=1.0    !each grass or moss PFT = 1 "individual"
            endif

            lm_ind(pft)=0.0
            sm_ind(pft)=0.0
            rm_ind(pft)=0.0
            hm_ind(pft)=0.0
            fpc_grid(pft)=0.0

            if(.not.tree(pft)) crownarea(pft)=1.0
            leafon(pft)=.true.
            leafondays(pft)=0.0
            leafoffdays(pft)=0.0
            
         endif

      enddo


c     SAPLING AND GRASS ESTABLISHMENT

c     Calculate total woody FPC and number of woody PFTs present and
c     able to establish
      
      fpc_tree_total=0.0
      fpc_grass_total=0.0
      fpc_moss_total=0.0
      npft_estab=0
      fpc_total=0.0
      ngrass=0  
      nmoss=0
      

      do pft=1,npft 
         if (present(pft)) then
            if (tree(pft)) then       
               fpc_tree_total=fpc_tree_total+fpc_grid(pft) 
               if (estab(pft)) npft_estab=npft_estab+1
            else if (grass(pft)) then
               ngrass=ngrass+1 
               fpc_grass_total=fpc_grass_total+fpc_grid(pft)
            else
               nmoss=nmoss+1
               fpc_moss_total=fpc_moss_total+fpc_grid(pft)
            endif           
            fpc_total=fpc_total+fpc_grid(pft)
         endif
      enddo                     ! pft 


      acflux_estab=0.0
c     Prohibit establishment under extreme temperature or water stress.
      
      do pft=1,npft                              

         if (aprec.ge.aprec_min_estab.and.npft_estab.gt.0) then

c     Calculate establishment rate over available space, per tree PFT
c     Maximum establishment rate reduced by shading as tree FPC approaches 1
c     Total establishment rate partitioned equally among regenerating woody
c     PFTs

            estab_rate=estab_max*(1.0-exp(5.0*(fpc_tree_total-1.0)))/
     *           real(npft_estab) 


c     Calculate grid-level establishment rate per woody PFT
c     Space available for woody PFT establishment is proportion of grid cell
c     not currently occupied by woody PFTs.

            estab_grid=estab_rate*(1.0-fpc_tree_total)

         else                   !unsuitable climate for establishment

            estab_grid=0.0

         endif


         if (present(pft).and.tree(pft).and.estab(pft)) then
            crownarea_max=pftpar(pft,20)

c     Add new saplings to current population

            nind_old=nind(pft)
            nind(pft)=nind_old+estab_grid   

            lm_ind(pft)=(lm_ind(pft)*nind_old+lm_sapl(pft)*estab_grid)/
     *           nind(pft)
            sm_ind_temp=(sm_ind(pft)*nind_old+sm_sapl(pft)*estab_grid)/
     *           nind(pft)
            hm_ind(pft)=(hm_ind(pft)*nind_old+hm_sapl(pft)*estab_grid)/
     *           nind(pft)
            rm_ind(pft)=(rm_ind(pft)*nind_old+rm_sapl(pft)*estab_grid)/
     *           nind(pft)


c     Accumulate biomass increment due to sapling establishment

            acflux_estab=acflux_estab+(lm_sapl(pft)+sm_sapl(pft)+
     *           hm_sapl(pft)+rm_sapl(pft))*estab_grid

c     Calculate height, diameter and crown area for new average
c     individual such that the basic allometric relationships (A-C below)
c     are satisfied.

c     (A) (leaf area) = latosa * (sapwood xs area)
c     (Pipe Model, Shinozaki et al. 1964a,b; Waring et al 1982)
c     (B) (leaf mass) = lmtorm * (root mass)
c     (C) height = allom2 * (stem diameter)**allom3
c     (source?)
c     (D) (crown area) = min (allom1 * (stem diameter)**reinickerp,
c     crownarea_max)

c     From (A),
c     (1) sap_xsa = lm_ind * sla / latosa
c     (2) wooddens = (sm_ind + hm_ind) / stemvolume
c     (3) stemvolume = stem_xsa * height
c     From (1), (2) & (3),
c     (4) stem_xsa = (sm_ind + hm_ind) / wooddens / height
c     (5) stem_xsa = pi * (stemdiam**2) / 4
c     From (5),
c     (6) stemdiam = ( 4 * stem_xsa / pi )**0.5
c     From (4) & (6),
c     (7) stemdiam = ( 4 * (sm_ind + hm_ind) / wooddens / height /
c     pi )**0.5
c     From (C) & (7),
c     (8) stemdiam = ( 4 * (sm_ind + hm_ind) / wooddens /
c     ( allom2 * stemdiam**allom3 ) / pi )**0.5
c     From (8),
c     (9) stemdiam = ( 4 * (sm_ind + hm_ind ) / wooddens / pi /
c     allom2 )**( 1 / (2 + allom3) )

            stemdiam=(4.0*(sm_ind_temp+hm_ind(pft))/wooddens(pft)/pi/
     $           allom2(pft))**(1.0/(2.0+allom3)) !Eqn 9
            height(pft)=allom2(pft)*stemdiam**allom3 !Eqn C
            crownarea(pft)=min(crownarea_max,
     *           allom1(pft)*stemdiam**reinickerp) !Eqn D

c     Recalculate sapwood mass, transferring excess sapwood to heartwood
c     compartment, if necessary to satisfy Eqn A

            sm_ind(pft)=lm_ind(pft)*height(pft)*wooddens(pft)*sla(pft)/
     $           latosa(pft) 
            hm_ind(pft)=hm_ind(pft)+(sm_ind_temp-sm_ind(pft))

c     Update LAI and FPC
            
            if (crownarea(pft).gt.0.0) then
               lai_ind(pft)=(lm_ind(pft)*sla(pft))/crownarea(pft)
            else
               lai_ind(pft)=0.0
            endif

            fpc_ind=(1.0-exp(-0.5*lai_ind(pft)))
            fpc_grid(pft)=crownarea(pft)*nind(pft)*fpc_ind    

            IF (fpc_grid(pft) .LT. 0.0) fpc_grid(pft) = 0.0
         endif
      enddo                     !pft
      
      fpc_tree_total=0.0
      fpc_grass_total=0.0
      fpc_moss_total=0.0
      fpc_total=0.0

      do pft=1,npft 
         if (present(pft)) then
            if (tree(pft)) then       
               fpc_tree_total=fpc_tree_total+fpc_grid(pft) 
            else if (grass(pft)) then
               fpc_grass_total=fpc_grass_total+fpc_grid(pft)
            else
               fpc_moss_total=fpc_moss_total+fpc_grid(pft)
            endif           
            fpc_total=fpc_total+fpc_grid(pft)
         endif
      enddo                     ! pft


c     SECTION FOR TREES

      if (fpc_tree_total.gt.0.95) then
         do pft=1,npft 
            if(tree(pft).and.present(pft)) then
               nind_old=nind(pft)
               nind(pft)=nind(pft)/(fpc_tree_total/0.95)
               fpc_grid(pft)=fpc_grid(pft)/(fpc_tree_total/0.95)
               litter_ag(pft)=litter_ag(pft)+(lm_ind(pft)+sm_ind(pft)+
     *              hm_ind(pft))*(nind_old-nind(pft))
               litter_bg(pft)=litter_bg(pft)+rm_ind(pft)*(nind_old-
     *              nind(pft))
            endif
         enddo
c     The next 2 lines were re-introduced after I ran all my runs for
c     paper1. I don't think it'll make a difference as fpc_total > 0 can
c     be/is caused by non-tree PFTs
         fpc_total = fpc_total - (fpc_tree_total - 0.95)
         if (fpc_total .GT. 1.0) fpc_total = 1.0
         fpc_tree_total=0.95
      endif

c     SECTION FOR GRASSES AND MOSSES
c     LPJ-WHy: Calculation of bare ground has changed from Sitch et al. (2003).
      DO pft = 1, npft
         IF (present(pft) .AND. .NOT. tree(pft)) THEN
            IF (estab(pft)) THEN
               bare1 = (1.0 - fpc_total) / (real(ngrass) +
     $              real(nmoss))
               bare2 = (1.0 - fpc_total) / real(ngrass)
               IF (fpc_total .LT. 1.0) THEN
                  lm_ind(pft) = lm_ind(pft) + bare1 * lm_sapl(pft)
                  rm_ind(pft) = rm_ind(pft) + bare2 * rm_sapl(pft)    
                              
! The following is taken from the LIGHT SUBROUTINE
!-------------------------------------------------   
               ELSE IF (fpc_total .GT. 1.0) THEN
                  excess = (min(fpc_tree_total, fpc_tree_max) +
     *                 (fpc_grass_total + fpc_moss_total) - 1.0) *
     *                 (fpc_grid(pft) / (fpc_grass_total +
     $                 fpc_moss_total))
                  IF (lm_ind(pft) .GT. 0.0) THEN
                     lm_old = lm_ind(pft)
                     lm_ind(pft) = -2.0 * alog(1.0 -
     $                    (fpc_grid(pft) - excess)) / sla(pft)
                     lm_kill = lm_old - lm_ind(pft)
                     rm_kill = rm_ind(pft) * (lm_kill/lm_old)
                  ELSE
                     lm_old = 0.0
                     lm_ind(pft) = 0.0
                     lm_kill = 0.0
                     rm_kill = 0.0
                  ENDIF

                  rm_ind(pft) = rm_ind(pft) - rm_kill

! Transfer lost biomass to litter 
!--------------------------------
                  litter_ag(pft) = litter_ag(pft) + lm_kill
                  litter_bg(pft) = litter_bg(pft) + rm_kill
                  IF (moss(pft)) litter_bg(pft) = MISSING_VALUE
                  IF (moss(pft)) rm_ind(pft) = MISSING_VALUE 
               ENDIF
            ENDIF
         ENDIF
c     Accumulate biomass increment due to grass establishment
c     NB: acflux_estab is different from acflux_est in LPJ-GUESS, where
c     it is a negative flux. 
         
         acflux_estab=acflux_estab+(bare1*lm_sapl(pft)+bare2*
     *        rm_sapl(pft))*crownarea(pft)
         
         IF (lm_ind(pft) .LE. 0.0) THEN
            present(pft) = .false.
            DO m = 1, 12
               mnpp_add(m) = mnpp_add(m) + mnpp(m,pft)
               anpp_add = anpp_add + mnpp(m,pft)
            ENDDO

            litter_bg(pft) = litter_bg(pft) + rm_ind(pft) * nind(pft)

            ! ERROR message:
            ! --------------
            IF (litter_bg(pft) .NE. litter_bg(pft))
     $           PRINT*, 'ERROR: litter_bg is NAN, pft=', pft,
     $           ' litter_bg=', litter_bg(pft)

            IF (moss(pft)) litter_bg(pft) = MISSING_VALUE
         ENDIF
      ENDDO                     !grasses & mosses

c     RECALCULATE FPC's

      fpc_total = 0.0
      DO pft = 1, npft
         IF (crownarea(pft) .GT. 0.0) THEN
            lai_ind(pft) = (lm_ind(pft) * sla(pft)) / crownarea(pft)
         ELSE
            lai_ind(pft) = 0.0
         ENDIF

         fpc_ind = (1.0 - exp(-0.5 * lai_ind(pft)))
         fpc_grid(pft) = crownarea(pft) * nind(pft) * fpc_ind 
         fpc_total = fpc_total + fpc_grid(pft)


      ENDDO

c     CALCULATE VEGETATION CARBON CONTENT

      vegc = 0.0                                                             
      DO pft = 1, npft                                                        
         IF (present(pft)) vegc = vegc + (lm_ind(pft) + rm_ind(pft) +
     $        sm_ind(pft) + hm_ind(pft)) * nind(pft)      
      ENDDO                                                               


      RETURN
      END

c -----------------------------------------------------------------------------
c                                 REFERENCES
c -----------------------------------------------------------------------------

c AGRMET Handbook 2002: www.rap.ucar.edu/projects/land/LSM/gribdoc/GRIB_
c   HANDBOOK_FOR_LSMvariables.doc
c Baird AJ, Beckwith CW, Waldron S, and Waddington JM 2004 Ebullition of
c   methane-containing gas bubbles from near-surface Sphagnum
c   peat. Geophys. Res. Lett., 31(21):299–322
c Bartlett MG, Chapman DS, and Harris RN 2004 Snow and the ground
c   temperature record of climate change. J. Geophys. Res., 109,
c   art. no. F04008, doi:10.1029/2004JF000224.
c Bernard JM and Fiala K 1986 Distribution and standing crop of living
c   and dead roots in three wetland Carex species. Bull. Torrey Bot. Club,
c   113(1):1–5
c Bonan GB 2002 Ecological climatology: Concepts and
c   applications. Cambridge University Press
c Broecker WS and Peng T-H 1974 Gas exchange rates between air and
c   sea. Tellus, 26(1-2):21–35,
c Burn 1998 The active layer: Two contrasting definitions, Permafrost
c   Periglacial process 9, 411-416.
c Carslaw, HS & Jaeger JC 1959 Conduction of Heat in Solids, Oxford University
c   Press, London
c Clymo RS and Hayward PM 1982 The ecology of Sphagnum in Bryophyte Ecology, 
c   Smith AJE (Ed.), Chapman and Hall, London
c Clymo RS 1984 The limits of peat bog growth, Phil. Trans. Royal Soc. Lond.
c   Ser. B 303: 605-654
c Collatz, GJ, Ball, JT, Grivet C & Berry, JA 1991 Physiological and
c   environmental regulation of stomatal conductance, photosynthesis and
c   transpiration: a model that includes a laminar boundary layer. Agricultural
c   and Forest Meteorology 54: 107-136
c Collatz, GJ, Ribas-Carbo, M & Berry, JA 1992 Coupled photosynthesis-stomatal
c   conductance models for leaves of C4 plants. Australian Journal of Plant
c   Physiology 19: 519-538
c Cronk JK and Fennessy SM 2001 Wetland plants: biology and ecology. CRC Press
c   LLC
c Farquhar GD & von Caemmerer 1982 Modelling of photosynthetic response to
c   environmental conditions. In: Lange, OL, Nobel PS, Osmond CB, Ziegler H
c   (eds) Physiological Plant Ecology II: Water Relations and Carbon
c   Assimilation, Vol 12B. Springer, Berlin, pp 549-587.
c Foley J A 1995 An equilibrium model of the terrestrial carbon budget
c   Tellus (1995), 47B, 310-319
c Fukusako S 1990 Thermophysical properties of ice, snow, and sea
c   ice. Int. J. Thermophys., 11(2): 353–372
c Gerten D, Schaphoff S, Haberlandt U, Lucht W, and Sitch S. 2004
c   Terrestrial vegetation and water balance – hydrological evaluation of
c   a dynamic global vegetation model. J. Hydrol., 286: 249–270
c Granberg G, Grip H, Ottosson Loefvenius M, Sundh I, Svensson BH, and
c   Nilsson M. 1999 A simple model for simulation of water content, soil
c   frost, and soil temperatures in boreal mixed mires. Water
c   Resour. Res., 35(12):3771–3782
c Happell JD, Chanton JP, Showers WJ 1995 Methane transfer across the water-air
c   interface in stagnant wooded swamps of Florida: Evaluation of mass-transfer
c   coefficients and isotopic fractionation. Limnol. Oceanogr. 40: 290-298
c Harper JL 1977 Population Biology of Plants, Academic Press, London
c Haxeltine A & Prentice IC 1996 BIOME3: an equilibrium terrestrial biosphere
c   model based on ecophysiological constraints, resource availability, and
c   competition among plant functional types. Global Biogeochemical Cycles 10:
c   693-709
c Haxeltine A & Prentice IC 1996a A general model for the ligh-tuse efficiency
c   of primary production. Functional Ecology 10: 551-561
c Hayward PM & Clymo RS 1982 Profiles of water content and pore size in
c   Sphagnum and peat, and their relation to peat bog ecology, Proc. R. Soc.
c   Lond. Ser. B, 215: 299-325
c Henderson-Sellers, A & Robinson, PJ 1986 Contemporary Climatology. Longman,
c   Essex.
c Hillel D 1982 Introduction to soil physics. Academic Press, New York.
c Hogg EH, Wein RW 1988 Seasonal change in gas content and buoyancy of
c   floating Typha mats. J. Ecol. 76: 1055-1068
c Iiyama I and Hasegawa S 2005 Gas diffusion coefficient of undisturbed
c   peat soils. Soil Sci. Plant Nutr., 51(3):431–435
c Ingram HAP 1983: Hydrology in Mires: swamp, bog, fen and moor. Gore AJP (Ed.)
c   Elsevier, Amsterdam. 
c Jaehne B, Heinz G, and Dietrich W 1987 Measurement of the diffusion
c   coefficients of sparingly a soluble gases in water. J. Geophys. Res.,
c   92(C10):10767–10776
c James DW 1968 The thermal diffusivity of ice and water between -40 and +60
c   deg C. Journal of Materials Science 3: 540-543
c Jarvis, PG & McNaughton KG 1986 Stomatal control of transpiration: scaling up
c   from leaf to region. Advances in Ecological Research 15: 1-49
c Jury WA, Gardner WR & Gardner WH 1991 Soil Physics 5th ed, John Wiley, NY
c Kellner E, Baird AJ, Oosterwoud M, Harrison K and Waddington JM 2006
c   Effect of temperature and atmospheric pressure on methane (CH4)
c   ebullition from near-surface peats. Geophys.  Res. Lett.,
c   33(18):397–411
c Kim J, Verma SB 1996 Surface exchange of water vapour between an open 
c   Sphagnum fen and the atmosphere. Bound.-Layer. Meteorol. 79, 3:243-264.
c Larcher W 1983 Physiological Plant Ecology, 2nd ed, Springer-Verlag, Berlin
c Ling F and Zhang TJ 2006 Sensitivity of ground thermal regime and
c   surface energy fluxes to tundra snow density in northern Alaska. Cold
c   Reg. Sci. Technol., 44(2):121–130
c Lloyd, J & Taylor JA 1994 On the temperature dependence of soil respiration
c   Functional Ecology 8: 315-323
c Millington RJ and Quirk JP 1961 Permeability of porous
c   solids. Trans. Faraday Soc., 57:1200– 1207
c Monsi, M & Saeki, T 1953 Ueber den Lichtfaktor in den Pflanzengesellschaften
c   und seine Bedeutung fuer die Stoffproduktion. Japanese Journal of Botany
c   14: 22-52
c Monteith, JL & Unsworth, MH 1990 Principles of Environmental Physics, 2nd ed,
c   Arnold, London
c Muller 1947 Permafrost or permanently frozen ground and related engineering
c   problems. Edwards, Ann Arbor, MI.
c  Potter CS, Davidson EA, and Verchot LV 1996 Estimation of global
c   biogeochemical controls and seasonality in soil methane
c   consumption. Chemosphere, 32(11):2219–2246
c Prentice, IC, Sykes, MT & Cramer W 1993 A simulation model for the transient
c   effects of climate change on forest landscapes. Ecological Modelling 65:
c   51-70.
c Press, WH, Teukolsky, SA, Vetterling, WT & Flannery, BT. 1986. Numerical
c   Recipes in FORTRAN, 2nd ed. Cambridge University Press, Cambridge
c Price, JS 1991. Evaporation from a blanket bog in a foggy coastal environment
c   Boundary-Layer Meteorology 57: 391-406.
c Reich, PB, Walters, MB & Ellsworth, DS, 1997. From tropics to tundra: global
c   convergence in plant functioning. Proceedings of the National Academy of
c   Sciences USA 94: 13730-13734.
c Riera JL, Schindler JE, and Kratz TK 1999 Seasonal dynamics of carbon
c   dioxide and methane in two clear-water lakes and two bog lakes in
c   northern Wisconsin, USA. Can. J. Fish. Aquat.  Sci., 56(2):265–274
c Ryan, MG 1991 Effects of climate change on plant respiration. Ecological 
c   applications 1: 157-167
c Sander R 1999a Compilation of Henry’s Law Constants for inorganic and
c   organic species of poten- tial importance in environmental
c   chemistry. Technical Report Version 3, MPI Mainz, Air Chemistry
c   Department, Max-Planck Institute of Chemistry
c Sander R 1999b Modeling atmospheric chemistry: Interactions between
c   gas-phase species and liquid cloud/aerosol particles. Surv. Geophys.,
c   20:1–31
c Schimel JP 1995 Plant transport and methane production as controls on
c   methane flux from arctic wet meadow tundra. Biogeochem., 28:183–200
c Shinozaki, K, Yoda, K, Hozumi, K & Kira, T 1964 A quantitative analysis of
c   plant form - the pipe model theory. I. basic analyses. Japanese Journal of
c   Ecology 14: 97-105
c Shinozaki, K, Yoda, K, Hozumi, K & Kira, T 1964 A quantitative analysis of
c   plant form - the pipe model theory. II. further evidence of the theory and
c   its application in forest ecology. Japanese Journal of Ecology 14: 133-139
c Smolders, AJP, Tomassen HBM, Pijnappel HW, Lamers LPM, Roelofs JGM 2001
c   Substrate-derived CO2 is important in the development of Sphagnum spp..
c   New Phytol. 152:325-332.
c Sprugel, DG, Ryan, MG, Brooks, JR, Vogt, KA, Martin, TA, Respiration from the
c   organ level to the stand (in press '93 ---> look up)
c Strack, M, Kellner, E, Waddington, JM 2005 Dynamics of biogenic gas bubbles
c   in peat and their effects on peatland biogeochemistry. Glob. Biogeochem.
c   Cycles 19, GB1003, doi:10.1029/2004GB002330
c Sturm M, Holmgren J, Konig M, and Morris K 1997 The thermal
c   conductivity of seasonal snow. J. Glaciol., 43(143):26–41
c Tarnocai C 2004 Cryosols of Arctic Canada. chapter 3, pages
c   95–118. Springer Verlag.
c van Duin, RHA 1963 The influence of soil management on the temperature
c   wave near the surface. Tech Bull 29 Inst for Land and Water Management
c   Research, Wageningen, Netherlands
c Walter, BP, Heimann, M., Matthews, E. 2001a Modeling modern methane
c   emissions from wetlands 1. Model description and results
c Wanninkhof R 1992 Relationship between wind-speed and gas-exchange
c   over the ocean. J. Geophys.  Res., 97(C5):7373–7382
c Waring, RH Schroeder, PE & Oren, R 1982 Application of the pipe model theory
c   to predict canopy leaf area. Canadian Journal of Forest Research 12:
c   556-560  
c Yamamoto S, Alcauskas JB, and Crozier TE 1976 Solubility of methane in
c   distilled water and seawater. J. Chem. Eng. Data, 21(1):78–80
c Zhuang Q, Romanovsky VE, and McGuire AD 2001. Incorporation of a
c   permafrost model into a large-scale ecosystem model: Evaluation of
c   temporal and spatial scaling issues in simulating soil thermal
c   dynamics. J. Geophys. Res., 106(D24):33649–33670

c -----------------------------------------------------------------------------
